#ifndef CONFIGURATION_INTERFACE_H
#define CONFIGURATION_INTERFACE_H
#include <map>
#include <string>
#include <memory>
#include "sctConf/configuration.h"
#include "is/info.h"
#include "is/inforeceiver.h"
#include <boost/thread.hpp>
#include <boost/shared_ptr.hpp>

namespace SctAnalysis{

  /** 
      Analysis Service's "personal" interface/cache to configuration. 
      Keeps itself up-to-date by subscribing to the IS server which notifies of changes. 
      On notification, obtains power mapping information from ConfigService servant implimentation.
  */
  class ConfigurationInterface{
  public:
    /** takes ownership of configuration and subscribes for changes to is */
    ConfigurationInterface(boost::shared_ptr<SctConfiguration::Configuration> config);
    /** destructor */
    ~ConfigurationInterface();    
    /** access information */
    /*std::string translateToModuleName(unsigned partition, unsigned crate, unsigned channel);*/
    /** 
	get Dcs identifier string CrateXX.ChannelXX for a module 
	@return null string if no match
     */
    std::string getDcsId(const std::string& modulename) const;
    /** rebuild personal map */
    void update();
    /** called when configuration changes */
    static void updateCallback(ISCallbackInfo* isc);
    /** return true if no modules are mapped */
    bool empty() const;
    /** output some information */
    void print(std::ostream& os) const;
  private:
    /// for thread-safe access:
    mutable boost::recursive_mutex m_access;   
    /// personal map of partition information
    std::map<std::string, std::string> m_map;
    /// client to the configuration service
    boost::shared_ptr<SctConfiguration::Configuration> m_config;
    /// needs inforeceiver for watching for callbacks
    boost::shared_ptr<ISInfoReceiver> m_infoReceiver;
    /// encoding for p_c_c
    std::string encode(unsigned partition, unsigned crate, unsigned channel);
  private:
    ConfigurationInterface();
  };
}
#endif
