#ifndef RXTHRESHOLDBASEDONCONFIGREGISTERALGORITHM_H
#define RXTHRESHOLDBASEDONCONFIGREGISTERALGORITHM_H

#include "AnalysisAlgorithm.h"

namespace SctAnalysis {
    /**
       Algorithm for RxThresholdBasedOnConfigRegister Test.
    */
class RxThresholdBasedOnConfigRegisterAlgorithm : public AnalysisAlgorithm {
public:
  // categorise the data from any particular RxThreshold value:
  typedef enum {undecided,white,black,blackAndWhite,coloured} Category;
public:
  static std::string categoryToString(const Category & category) {
    return
      category==undecided     ? "undecided" :
      category==white         ? "white" :
      category==black         ? "black" :
      category==blackAndWhite ? "blackAndWhite" :
      category==coloured      ? "coloured" :
      "couldNotFigureCategoryOut";
  };
public:
    virtual bool canAnalyze() const;
    virtual void analyze();
    virtual void loadData();
    virtual shared_ptr<SctData::TestResult> createTestResult() const;
    virtual boost::shared_ptr<AnalysisAlgorithm> clone(shared_ptr<const TestData> testData, const string& moduleName) const throw();
    /** 
	constructor 
     */
    RxThresholdBasedOnConfigRegisterAlgorithm(shared_ptr<const TestData> testData, const string& moduleName, const AnalysisAlgorithm& alg) throw(); 
    /** 
	destructor 
     */
    virtual ~RxThresholdBasedOnConfigRegisterAlgorithm() throw() {}

    /* fraction of the way from low to high at which to set optimum */
    void setFraction(float fraction) {m_fraction=fraction;}
    /* fraction of the way from low to high at which to set optimum */
    float getFraction() const {return m_fraction;}
private:
    float m_fraction;
    RxThresholdBasedOnConfigRegisterAlgorithm() {}
    static bool inMap;
};
}

#endif //RXTHRESHOLDBASEDONCONFIGREGISTERALGORITHM_H
