#include "ArchivingServiceInterface.hh"
#include "ArchiverArguments.h"
#include "boost/lexical_cast.hpp"

namespace SctArchiving {
  using std::string;
  using std::list;
  using std::endl;

  ArchiverArguments::ArchiverArguments(int argc, char** argv) : 
    Arguments(argc, argv), m_compressionLevel(3)
  {
    parse();
  }
  void ArchiverArguments::parse(){
    Arguments::parse();
    for (list<string>::iterator i=m_list.begin(); i!=m_list.end(); ++i){
      if (*i == "-compression"){
	m_compressionLevel=boost::lexical_cast<unsigned>(next(i));
      } else if (*i == "-persistent"){
	m_persistentDir=next(i);
      }
    }
    if (instanceName()=="") setInstanceName(ArchivingServiceI::ArchivingServiceInterface::instanceName);
  }
  void ArchiverArguments::printHelp(std::ostream& oss) const{
    oss << "ArchiverArguments:" << endl
	<< "  -compression                   : set compression level (1-9)" <<endl
	<< "  -persistent                    : set persistent directory" <<endl;
    Arguments::printHelp(oss);
  }

  std::string ArchiverArguments::getPersistentDirectory() const{
    return m_persistentDir;
  }

  unsigned ArchiverArguments::getCompressionLevel() const{
    return m_compressionLevel;
  }
}
