#include "../IOManagerDB.h"
#include "IOManager_Connect_Test.h"
namespace SctArchiving{

  // Create a global TestData object
  TestData tdGlob;

void initUnitTests(){
  // Fill global TestData object
  tdGlob.runNumber = 1234;

  std::cout << std::endl;
  std::cout << "initUnitTests: Initialising the unit tests" << std::endl;
  std::cout << "initUnitTests: Using run# " << tdGlob.runNumber << std::endl;
  std::cout << std::endl;
}

void checkCreation(){
  std::cout << "checkCreation: Trying to create new IOmanager..." << std::endl;
  IOManagerDB& man = IOManagerDB::instance();
  man.setDataSetName("MyLoc");
}

void checkDateTimeParsing(){
  std::string input="20050301T121347";
  std::string output=IOManagerDB::instance().convertToMysqlDateTimeFormat(input);
  std::cout << "checkDateTimeParsing: Checking date format conversion..." << std::endl;
  BOOST_CHECK_EQUAL(output, "20050301121347");
}

void checkDosentWorkIfDataSetNameIsWrong(){
  IOManagerDB& man = IOManagerDB::instance();
  std::string originalDataSet = man.getDataSetName();
  man.setDataSetName("this_is_a_very_unlikely_location_name");
  std::cout << "checkDosentWorkIfDataSetNameIsWrong: Trying bad DataSetName..." << std::endl;
  BOOST_CHECK_THROW(man.addRun(tdGlob.runNumber, 0), MysqlException);
  man.setDataSetName(originalDataSet);
}

void checkGetDataSetID(){
  IOManagerDB& man = IOManagerDB::instance();
  man.setDataSetName("MyLoc");
  std::cout << "checkGetDataSetID: Calling getDataSetID()..." << std::endl;
  BOOST_CHECK(man.getDataSetID()!="");
}

void checkCanAddRun(){
  IOManagerDB& man = IOManagerDB::instance();
  if (man.getRunID(tdGlob.runNumber)=="") {
    std::cout << "checkCanAddRun: adding run# " << tdGlob.runNumber << std::endl;
    man.addRun(tdGlob.runNumber, -1);
  }
  std::cout << "checkCanAddRun: Check getRunID()..." << std::endl;
  BOOST_CHECK(man.getRunID(tdGlob.runNumber)!="");
}

void checkCanAddTest(){
  // Add more info to global TestData object
  tdGlob.startScanNumber = 101;
  tdGlob.testName = "WeirdTest";
  tdGlob.startTime = "20050301T121348";
  tdGlob.endTime = "20050301T121356";
  tdGlob.nScans=7;

  IOManagerDB& man = IOManagerDB::instance();
  std::string runId = man.getRunID(tdGlob.runNumber);

  if (man.getTestID(runId, tdGlob.startScanNumber)==""){
    BOOST_CHECK(man.getRunID(tdGlob.runNumber)!="");
    std::cout << "checkCanAddTest: adding new test (run#" << tdGlob.runNumber 
              << ", 1stScan#" << tdGlob.startScanNumber << ")" << std::endl;
    man.addTest(tdGlob, false);
  }
  std::cout << "checkCanAddTest: Calling getTestID()..." << std::endl;
  BOOST_CHECK(man.getTestID(runId, tdGlob.startScanNumber)!="");
}

void checkCanAddTestAndScans(){
  // Fill global TestData object: new test, but same run
  tdGlob.startScanNumber=121;
  tdGlob.startTime="20050301T202122";
  tdGlob.endTime="20050301T232425";
  tdGlob.nScans=2;
  
  IOManagerDB& man = IOManagerDB::instance();
  std::string runId = man.getRunID(tdGlob.runNumber);

  if (man.getTestID(runId, tdGlob.startScanNumber)==""){
    BOOST_CHECK(man.getRunID(tdGlob.runNumber)!="");
    std::cout << "checkCanAddTestAndScans: adding test+scans (run#" << tdGlob.runNumber
              << ", 1stScan#" << tdGlob.startScanNumber << ")" << std::endl;
    man.addTest(tdGlob, true);
  }
  std::string testId = man.getTestID(runId, tdGlob.startScanNumber);
  BOOST_CHECK(testId != "");
  BOOST_CHECK(man.getScanID(testId, tdGlob.startScanNumber)!="");
  BOOST_CHECK(man.getScanID(testId, tdGlob.startScanNumber + 1)!="");
}

void checkGetModuleID(){
  IOManagerDB& man = IOManagerDB::instance();

  std::cout << "checkGetModuleID: Calling getModuleID() with"
            << " a good module#..." << std::endl;
  BOOST_CHECK(man.getModuleID("20220330200999") != "");

  std::cout << "checkGetModuleID: Calling getModuleID() with"
            << " a bad module#..." << std::endl;
  BOOST_CHECK(man.getModuleID("99999999999999") == "");
}

void checkCanAddRawDataRecord(){
  IOManagerDB& man = IOManagerDB::instance();
    
  std::string moduleId = man.getModuleID("20220330200999");
  std::string runId = man.getRunID(tdGlob.runNumber);
  std::string testId = man.getTestID(runId, tdGlob.startScanNumber);
  std::string scanId = man.getScanID(testId, tdGlob.startScanNumber);

  std::cout << "checkCanAddRawDataRecord: adding scan-mod comb..." << std::endl;
  man.addRawDataRecord(moduleId, scanId);
}

void checkCanAddRawDataBlob(){
  IOManagerDB& man = IOManagerDB::instance();
  std::string moduleId = man.getModuleID("20220330200999");
  std::string runId = man.getRunID(tdGlob.runNumber);
  std::string testId = man.getTestID(runId, tdGlob.startScanNumber);
  std::string scanId = man.getScanID(testId, tdGlob.startScanNumber);
  std::string strBuffer = "Well, this is ASCII data, not binary data, but anyway..";

  std::cout << "checkCanAddRawDataBlob: adding binary data..." << std::endl;
  man.addRawDataBlob(moduleId, scanId, strBuffer.c_str(), strBuffer.size());
}

void finUnitTests(){
  std::cout << "finUnitTests: Cleaning up db after unit tests..." << std::endl;
  IOManagerDB& man = IOManagerDB::instance();
  std::ostringstream select;
  select << "DELETE FROM tblRuns WHERE "
         << "RunID = " << man.getRunID(tdGlob.runNumber);

  MYSQL* m_mysql; ///< the main MYSQL object
  m_mysql = mysql_init((MYSQL*) 0);
  mysql_real_connect(m_mysql, "localhost", "daquser", "", NULL, 3306, NULL, 0);
  std::string dbname="test_dbSRD";
  mysql_select_db(m_mysql, dbname.c_str());
  mysql_query(m_mysql, select.str().c_str());
  mysql_close(m_mysql);
}

IOManager_Connect_Test::IOManager_Connect_Test(){
    add (BOOST_TEST_CASE(&initUnitTests));

    add (BOOST_TEST_CASE(&checkCreation));
    add (BOOST_TEST_CASE(&checkDateTimeParsing));
    add (BOOST_TEST_CASE(&checkDosentWorkIfDataSetNameIsWrong));

    add (BOOST_TEST_CASE(&checkGetDataSetID));
    add (BOOST_TEST_CASE(&checkCanAddRun)); // This one should add a new run
    add (BOOST_TEST_CASE(&checkCanAddRun)); // Here the run should already be in the db
    add (BOOST_TEST_CASE(&checkCanAddTest));
    add (BOOST_TEST_CASE(&checkCanAddTestAndScans));

    add (BOOST_TEST_CASE(&checkGetModuleID));
    add (BOOST_TEST_CASE(&checkCanAddRawDataRecord));
    add (BOOST_TEST_CASE(&checkCanAddRawDataBlob));

    add (BOOST_TEST_CASE(&finUnitTests));
}
}
