#include "SequenceDataStreamer_v1.h"
#include "SequenceDataWrapper.h"
#include <memory>
#include "CalibrationController/IS/SequenceData.h"

using namespace Sct;

namespace SctCalibrationController {

/* READ NOTES ON STREAMERS AND VERSIONS BEFORE EDITING THIS FILE! */
unsigned SequenceDataStreamer_v1::s_version=1;
	
SequenceDataStreamer_v1::SequenceDataStreamer_v1() throw() {}

bool SequenceDataStreamer_v1::inMap = IOManager::addToMap("SequenceData",  std::auto_ptr<Streamer>(new SequenceDataStreamer_v1()));

shared_ptr<Streamable> SequenceDataStreamer_v1::read(IStream& in, const IOManager& manager) const throw(LogicError, IoError) {
    shared_ptr<SequenceData> sd (new SequenceData);
    shared_ptr<Streamable> wrapper (new SequenceDataWrapper(sd));
    read(in, *wrapper, manager);
    return wrapper;
}

void SequenceDataStreamer_v1::write(OStream& out, const Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
  const SequenceDataWrapper& wrapper = dynamic_cast<const SequenceDataWrapper&>(ob);
  boost::shared_ptr<const SequenceData> sd = boost::dynamic_pointer_cast<const SequenceData>(wrapper.getWrapped());
  if (!sd.get()) throw Sct::IllegalStateError("Null pointer or bad cast of SequenceData object", __FILE__, __LINE__);
  out << sd->sequenceName << sd->runNumber << sd->startScanNumber << sd->nTests;
  out.put( sd->testNames, sd->testNames_size );
  out << (int)sd->status << sd->startTime << sd->endTime;
  out.put( sd->modules, sd->modules_size );
  out << sd->version;
}

void SequenceDataStreamer_v1::read(IStream& in, Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
  SequenceDataWrapper& wrapper = dynamic_cast<SequenceDataWrapper&>(ob);
  boost::shared_ptr<SequenceData> sd = boost::dynamic_pointer_cast<SequenceData>(wrapper.getWrapped());
  if (!sd.get()) throw Sct::IllegalStateError("Null pointer or bad cast of SequenceData object", __FILE__, __LINE__);
  in >> sd->sequenceName >> sd->runNumber >> sd->startScanNumber >> sd->nTests;
  if ( sd->testNames_size != 0 ) delete[] sd->testNames;
  in.get( &sd->testNames, sd->testNames_size );
  in >> (int&)sd->status >> sd->startTime >> sd->endTime;
  if ( sd->modules_size != 0 ) delete[] sd->modules;
  in.get( &sd->modules, sd->modules_size );
  in >> sd->version;

}
}
