#include "TestDataStreamer_v2.h"
#include "TestDataWrapper.h"
#include <memory>
#include "CalibrationController/IS/TestData.h"

using namespace Sct;

namespace SctCalibrationController {

/* READ NOTES ON STREAMERS AND VERSIONS BEFORE EDITING THIS FILE! */
unsigned TestDataStreamer_v2::s_version=2;
	
TestDataStreamer_v2::TestDataStreamer_v2() throw() {}

bool TestDataStreamer_v2::inMap = IOManager::addToMap("TestData",  std::auto_ptr<Streamer>(new TestDataStreamer_v2()));

shared_ptr<Streamable> TestDataStreamer_v2::read(IStream& in, const IOManager& manager) const throw(LogicError, IoError) {
    shared_ptr<TestData> td (new TestData);
    shared_ptr<Streamable> wrapper (new TestDataWrapper(td));
    read(in, *wrapper, manager);
    return wrapper;
}

void TestDataStreamer_v2::write(OStream& out, const Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
  const TestDataWrapper& wrapper = dynamic_cast<const TestDataWrapper&>(ob);
  boost::shared_ptr<const TestData> td = boost::dynamic_pointer_cast<const TestData>(wrapper.getWrapped());
  if (!td.get()) throw Sct::IllegalStateError("Null pointer or bad cast of TestData object", __FILE__, __LINE__);
  out << td->testName << td->runNumber << td->nScans << td->startScanNumber << td->testVariable;
  out.put( td->testPoints, td->testPoints_size );
  out << (int)td->status << td->startTime << td->endTime;
  out.put( td->modules, td->modules_size );
  out << td->fitAlgorithm;
  out << td->analysisAlgorithm;
  out << td->options;
  out << td->version;
}

void TestDataStreamer_v2::read(IStream& in, Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
  TestDataWrapper& wrapper = dynamic_cast<TestDataWrapper&>(ob);
  boost::shared_ptr<TestData> td = boost::dynamic_pointer_cast<TestData>(wrapper.getWrapped());
  if (!td.get()) throw Sct::IllegalStateError("Null pointer or bad cast of TestData object", __FILE__, __LINE__);
  in >> td->testName >> td->runNumber >> td->nScans >> td->startScanNumber >> td->testVariable;
  if ( td->testPoints_size != 0 ) delete[] td->testPoints;
  in.get( &td->testPoints, td->testPoints_size );
  in >> (int&)td->status >> td->startTime >> td->endTime;
  if ( td->modules_size != 0 ) delete[] td->modules;
  in.get( &td->modules, td->modules_size );
  in >> td->fitAlgorithm;
  in >> td->analysisAlgorithm;
  in >> td->options;
  in >> td->version;
}
}
