#ifndef TESTREQUEST_H
#define TESTREQUEST_H

#include "CalibrationController.hh"
#include "ScanRequest.h"
#include <ipc/object.h>
#include <memory>
#include <string>
#include <vector>

using std::vector;
using std::auto_ptr;
using std::string;

namespace SctCalibrationController {
namespace Ipc {
    
class CalibrationController;
    
/**
  See the documentation in CalibrationController.idl for these methods.
  All may throw RequestException.  This is the nice interface
  */
class TestRequest {
public:	    
    virtual ~TestRequest() {}
    virtual void startTestRealImpl(Sct_SctApi::SctApiIPC_ptr api,
			   string& name,
			   unsigned short& testVariable,
			   unsigned long& nScans,
			   vector<double>& testPoints) = 0;
    virtual Sct_CalibrationController::ScanRequest_ptr getNextScanRealImpl(unsigned long index) = 0;
    virtual bool canFeedbackRealImpl() = 0;
    virtual void endTestRealImpl() = 0;    
    virtual void setAnalysisAlgorithm(const char* name) = 0;
    virtual char* getAnalysisAlgorithm() = 0;
    virtual void setFitAlgorithm(const char* name) = 0;
    virtual char* getFitAlgorithm() = 0;
};

//typedef TestRequest MinimalTestRequest;

/**
  This is the internal implementation - to be derived from!
  */
class TestRequestImpl : 
  public IPCObject<POA_Sct_CalibrationController::TestRequest>,
  public TestRequest {
protected:
  //const boost::shared_ptr<MinimalTestRequest> m_minimalTestRequest;
  std::string m_fitAlgorithm; ///< Algorithm for fitting service to use.
  std::string m_analysisAlgorithm; ///< Algorithm for analysis service to use
public:    
  TestRequestImpl();
    virtual void startTest (/*CalibrationControllerStatus *_status, */
			    Sct_SctApi::SctApiIPC_ptr api,
			    CORBA::String_out name,
			    CORBA::UShort & testVariable,
			    CORBA::ULong & nScans,
			    Sct_CalibrationController::TestPoints_out testPointsData);
    virtual Sct_CalibrationController::ScanRequest_ptr getNextScan (/*Sct_CalibrationControllerStatus *_status, */CORBA::ULong index);    
    virtual CORBA::Boolean canFeedback (/*CalibrationControllerStatus *_status*/);
    virtual void endTest (/*CalibrationControllerStatus *_status*/);
    virtual void setAnalysisAlgorithm(const char* name);
    virtual char* getAnalysisAlgorithm();
    virtual void setFitAlgorithm(const char* name);
    virtual char* getFitAlgorithm();
};

/**
  This is the adapter for external implementations.
  These calls may fail and throw RequestExceptions
  */
/*
class TestRequestAdapter : public virtual TestRequest {
public:
    TestRequestAdapter(Sct_CalibrationController::TestRequest_ptr req);
    //
      //throws IpcException if any of the out variables are not set in the call
      //
    virtual void startTestRealImpl(Sct_SctApi::SctApiIPC_ptr api,
			   string& name,
			   unsigned short& testVariable,
			   unsigned long& nScans,
			   vector<double>& testPoints) ;
    virtual Sct_CalibrationController::ScanRequest_ptr getNextScanRealImpl(unsigned long index);
    virtual bool canFeedbackRealImpl();
    virtual void endTestRealImpl();
    
private:
    Sct_CalibrationController::TestRequest_var request;
};
*/

}
}
#endif //TESTREQUEST_H
