#ifndef SCTCALIBRATIONCONTROLLER_QUICKNMASKTEST_H
#define SCTCALIBRATIONCONTROLLER_QUICKNMASKTEST_H

#include "../ipc/TestRequest.h"
#include "../ipc/CalibrationController.h"
#include "../ipc/ScanLibrary.h"
#include "../ipc/CalibrationController.hh"

#include "../SctApiAccessException.h"
#include "../SctApiCall.h"
#include "../CalibrationControllerImpl.h"
#include "Sct/LogicErrors.h"

#include <Sct/AbcdScans.h>
#include <iostream>
#include <vector>

using namespace Sct;
using namespace std;
using namespace SctCalibrationController::Ipc;
using namespace SctCalibrationController;

namespace SctCalibrationController {
namespace Scripts {

  /** single nmask scan with checking */
    
class QuickNMaskTest : public TestRequestImpl {
private:
   static const bool registeredInTestLibrary;

public:	
    static Sct_CalibrationController::TestRequest_ptr instance() {
	static QuickNMaskTest rc;
	rc.setFitAlgorithm("NONE");
	rc.setAnalysisAlgorithm("NMaskTest");
	return rc._this();
    }
    
    virtual void startTestRealImpl(Sct_SctApi::SctApiIPC_ptr api, string& name, unsigned short& testVariable, unsigned long& nScans, vector<double>& testPoints) {	
	this->api = Sct_SctApi::SctApiIPC::_duplicate(api);

        // Compression mode X1X
        APICALL(api, modifyABCDVar(ST_COMPRESSION, 1), "QuickNMaskTest::startTest")

        // Edge detect off
        APICALL(api, modifyABCDVar(ST_EDGE_DETECT, 0), "QuickNMaskTest::startTest")

        // No masked channels
        APICALL(api, modifyABCDVar(ST_SEND_MASK, 1), "QuickNMaskTestassTest::startTest")

	name = "NMaskTest";
	testVariable = ST_ELAPSED;
	nScans = 1;
	
	testPoints.push_back(nTriggers);
    }
    
    virtual Sct_CalibrationController::ScanRequest_ptr getNextScanRealImpl(unsigned long index) {
        Sct_CalibrationController::ScanLibrary_var sl = CalibrationControllerImpl::instance()->getScanLibrary();
        Sct_CalibrationController::ScanRequest_ptr sr = sl->nmaskScan(nTriggers);
	Sct_SctApi::Scan_ptr scan = sr->getScanCorba();
	APICALL(scan, configure(ST_NMASK, 0, 128, 16), "Failed to set Scan information");
        return sr;
    }
    
    virtual bool canFeedbackRealImpl() {
	return false;
    }
    
    virtual void endTestRealImpl() {
	api = Sct_SctApi::SctApiIPC::_nil();
    }
    
private:
    QuickNMaskTest() {}
    Sct_SctApi::SctApiIPC_var api;    
    static const int nTriggers = 100;
};
}
}
#endif //SCTCALIBRATIONCONTROLLER_QUICKNMASKTEST_H
