#ifndef FITTERWORKERGROUP_H
#define FITTERWORKERGROUP_H
#include "SctService/WorkerGroup.h"
#include <string>
#include <list>
#include <iostream>
#include <boost/shared_ptr.hpp>
#include <boost/thread.hpp>

using std::string;
using boost::shared_ptr;
using namespace SctService;
class TestData;

namespace SctFitter {

/**
   Represents a group of worker threads which does the fitting.
   The fitting is done in the work() method.
 */
class FitterWorkerGroup : public WorkerGroup<string> {
public:
    /// initialise the number of calls and waits to 0;
    FitterWorkerGroup();

    /// destructor
    virtual ~FitterWorkerGroup();

    /// print list of tests
    void printTests(std::ostream& os) const;
    /// set debug flag
    void setDebug(bool value);
    /// get debug flag
    bool debug() const;
private:
    /**
       Does the fitting. First gets the appropriate FitAlgorithm,
       then does the fit, then publishes the result.
       Handles its own exceptions, sending messages to MRS.
    */
    virtual void work(string name) throw() ;

    /// number of times this worker has been called to work.
    volatile int m_calls;

    // add record of a TestData object (or null if none matches)
    virtual shared_ptr<const TestData> findTestData(unsigned long run, unsigned long scan);
    
    // find a matching TestData object
    virtual void addTestData(boost::shared_ptr<const TestData> td);
    

    /// list of tests
    std::list<boost::shared_ptr<const TestData> > m_tests;
    
    // control access to list of tests
    mutable boost::recursive_mutex m_testAccess;
    
    bool m_debug;
};

}// end of namespace SctFitter

#endif // #define FITTERWORKERGROUP_H
