/******************************************************************************
 *  scanTiming_sct.h:   defines a set of timing structures which are given as
 *                  part of the MDSP scanCtrl task's output.
 *
 ******************************************************************************/
#ifndef SCAN_TIMING_H
#define SCAN_TIMING_H 

/* Scan timing structures: */
typedef struct InitTiming {
	UINT32 total,
	       dt_sdspSetup,
	       dt_varSetup,
	       dt_cfg;
} InitTiming;

typedef struct NewbinSubTiming {
	UINT32 nExe,
	       dt_cfg,
	       avg,
	       total;
} NewbinSubTiming;

typedef struct NewbinTiming {
	UINT32 nExe,
	       dt_task,
	       avg,
	       total,
	       dt_setup,
	       dt_calc,
	       dt_cfg,
	       dt_resets;

	NewbinSubTiming loop[4]; //0: stage, 1:inner, 2: outer, 3:chip
} NewbinTiming;

typedef struct DspTiming {
	UINT32 proc,
	       idleFrac;
} DspTiming; 

typedef struct PulseTiming {
	UINT32 nExe,
	       dt_task,
	       avg,
	       total,
	       xmit,
	       sdspDwell,
	       nCorr,
	       xmitFrac;

	DspTiming sdsp[4];
} PulseTiming;

typedef struct SectionTiming {
	UINT32 nExe,
	       dt_task,
	       avg,
	       total;
} SectionTiming;

typedef struct ScanTimingInfo {
	UINT32 dt_total,
	       dt_mainLoop,
	       dt_sim,
	       unused;

	InitTiming    init;
	NewbinTiming  nb;
	SectionTiming wexp;
	PulseTiming   pulse[256];
	SectionTiming wait;
	SectionTiming prep;
	SectionTiming done;
} ScanTimingInfo;

#endif /* Multiple-inclusion protection */
