/************************************************************************************
 * calibrationControl.c
 *
 *  synopsis: Contains functions related to directing calibration runs which are
 *            under ROD control.  In such a run, the master would generate all of the
 *            needed calibration and trigger pulses as well as configuration commands.
 *            These commands are sent out the MCBSP.
 *
 *  Damon Fasching, UW Madison (510)486-5230               fasching@wisconsin.cern.ch
 *     - Initial version only has setCalibrationParams routine.
 ************************************************************************************/

#include "resources.h"

/*
 eventsPerSetting: num of calibration sequences per detector configuration
 eventPeriod:      period between calibration sequences
 calToL1Period:    within a sequence, time from the CAL to the 1st L1: 0 = no CAL pls
 l1ToL1Period:     within a sequence, time from the 1st to the 2nd L1: 0 = no 2nd L1
*/

UINT32 eventsPerSetting;  /* num of calibration sequences per detector configuration */
UINT32 eventPeriod;       /* period between calibration sequences */
UINT32 calToL1Period;     /* within a sequence, time from the CAL to the 1st L1 */
UINT32 l1ToL1Period;      /* within a sequence, time from the 1st to the 2nd L1 */

#pragma CODE_SECTION(setCalibrationParams, "xcode");

void setCalibrationParams(UINT32 numPulses, UINT32 period, UINT32 tCalToL1,
                          UINT32 tL1ToL1) {

  eventsPerSetting = numPulses;
  eventPeriod      = period;
  calToL1Period    = tCalToL1;
  l1ToL1Period     = tL1ToL1;
  
  return;
  
}
