#include <config/Configuration.h>

#include "DAL/Partition.h"
#include "DAL/Crate.h"

template<typename T> void destroyListOfObjects(::Configuration &conf, std::vector<const T*> objects);

SCTDAL::Partition *createPartition();
SCTDAL::Crate *createCrate(SCTDAL::Partition *);

void destroyPartitions();
void destroyCrates();

std::string testFilename("clearTest.data.xml");
Configuration conf("oksconfig");

int main() {
  // In case the previous run didn't tidy up
  system(("rm .oks-lock-" + testFilename).c_str());
  system(("rm " + testFilename).c_str());

  std::list<std::string> includes;
  includes.push_back("SCT.schema.xml");

  if(conf.create("", testFilename, includes) == false) {
    std::cerr << "ERROR: failed to create file \'" << testFilename << "\'\n";
    conf.abort();
  }

  SCTDAL::Partition *part = createPartition();
  createCrate(part);

  // Removing this line makes it work
  createPartition();

  destroyPartitions();
  createCrate(0);
}

SCTDAL::Partition *createPartition() {
  SCTDAL::Partition *partObj = const_cast<SCTDAL::Partition*>(conf.create<SCTDAL::Partition>(testFilename, ""));
  if(partObj == 0) {
    std::cerr << "ERROR: failed to create partition\n";
    conf.abort();
    return 0;
  }

  partObj->set_id(10);
  conf.commit();

  return partObj;
}

SCTDAL::Crate *createCrate(SCTDAL::Partition *part) {
  SCTDAL::Crate * crtObj = const_cast<SCTDAL::Crate*>(conf.create<SCTDAL::Crate>(testFilename, ""));
  if(crtObj == 0) {
    std::cerr << "ERROR: failed to create crate\n";
    conf.abort();
    return 0;
  }

  crtObj->set_id(10);

  if(part) {
    std::vector<const SCTDAL::Crate *> crates(part->get_crates());
    crates.push_back(crtObj);
    part->set_crates(crates);
  }

  conf.commit();

  return crtObj;
}

void destroyPartitions() {
  std::cout << "Destroying partitions\n";
  std::vector<const SCTDAL::Partition *> partObs ;
  conf.get<SCTDAL::Partition>(partObs);
  destroyListOfObjects<SCTDAL::Partition>(conf, partObs);
}

template<typename T> void destroyListOfObjects(::Configuration &conf, std::vector<const T*> objects) {
  typedef std::vector<const T*> TVector;

  std::cerr << "Destroying " << objects.size() << " objects\n";

  for(typename TVector::iterator i = objects.begin(); i != objects.end(); i++) {
    conf.destroy(**i);
  }

  conf.commit();
}
