//$Id: SCTReadoutModule.cpp,v 1.2 2005/09/21 15:17:52 sctroddq Exp $
/************************************************************************/
/*									*/
/* File             : SCTReadoutModule.cpp				*/
/*									*/
/* Authors          : B.Gorini, M.Joos, J.Petersen CERN			*/
/*									*/
/********* C 2004 - ROS/RCD  ********************************************/

#include <unistd.h>
#include <iostream>
#include "RCDExampleModules/SCTReadoutModule.h"
#include "RCDExampleModules/SCTScheduledUserAction.h"
#include "DFDebug/DFDebug.h"
#include "DFError/DFOutputStream.h"

using namespace ROS;
using namespace RCD;

/******************************************************/
SCTReadoutModule::SCTReadoutModule() 
/******************************************************/
{ 
  DEBUG_TEXT(DFDB_RCDEXAMPLE, 15, "SCTReadoutModule::constructor: Entered");
}

/*******************************************************/
SCTReadoutModule::~SCTReadoutModule() 
/*******************************************************/
{
  DEBUG_TEXT(DFDB_RCDEXAMPLE, 15, "SCTReadoutModule::destructor: Entered");

  // delete the User Timer Poll Handler
  if (m_scheduledUserAction0 != 0) {
    delete m_scheduledUserAction0;
  }
}

// setup gets parameters from configuration
/**************************************************************************************/
void SCTReadoutModule::setup(DFCountedPointer<Config> configuration) 
/**************************************************************************************/
{ 
  DEBUG_TEXT(DFDB_RCDEXAMPLE, 15, "SCTReadoutModule::setup: Entered");

  m_deltaTimeMs = configuration->getInt("DeltaTimeScheduler");
  DEBUG_TEXT(DFDB_RCDEXAMPLE, 20, "SCTReadoutModule::setup: MODULE delta time (ms) = "
                                  << std::dec << m_deltaTimeMs);
}

/*****************************************************/
void SCTReadoutModule::load(void)
/*****************************************************/
{

  DEBUG_TEXT(DFDB_RCDEXAMPLE, 15, "SCTReadoutModule::load: Entered");

  // handles the "time" interrupts
  m_scheduledUserAction0 = 0;
  if (m_deltaTimeMs >= 0) {
    m_scheduledUserAction0 = new SCTScheduledUserAction(m_deltaTimeMs);
    DEBUG_TEXT(DFDB_RCDEXAMPLE, 5, "SCTReadoutModule::load: Created SCTScheduledUserAction0");
//    std::string partition=getenv("TDAQ_PARTITION");
  }

  DEBUG_TEXT(DFDB_RCDEXAMPLE, 15, "SCTReadoutModule::load: Done");
}

/*****************************************************/
void SCTReadoutModule::unload(void)
/*****************************************************/
{

  DEBUG_TEXT(DFDB_RCDEXAMPLE, 15, "SCTReadoutModule::unload: Entered");

  DEBUG_TEXT(DFDB_RCDEXAMPLE, 15, "SCTReadoutModule::unload: Done");
}


/********************************************************/
void SCTReadoutModule::configure(void)
/********************************************************/
{
 
  DEBUG_TEXT(DFDB_RCDEXAMPLE, 20, "SCTReadoutModule::configure: Entered");

  DEBUG_TEXT(DFDB_RCDEXAMPLE, 15, "SCTReadoutModule::configure: Done");
}


/**********************************************************/
void SCTReadoutModule::unconfigure(void)
/**********************************************************/
{
  DEBUG_TEXT(DFDB_RCDEXAMPLE, 15, "SCTReadoutModule::unconfigure: Entered");
  
  DEBUG_TEXT(DFDB_RCDEXAMPLE, 15, "SCTReadoutModule::unconfigure: Done");
}
    

/************************************************************/    
void SCTReadoutModule::prepareForRun(void)
/************************************************************/    
{
  DEBUG_TEXT(DFDB_RCDEXAMPLE, 15, "SCTReadoutModule::prepareForRun: Entered");

  DEBUG_TEXT(DFDB_RCDEXAMPLE, 15, "SCTReadoutModule::prepareForRun: Done");

}

/************************************************************/    
void SCTReadoutModule::startTrigger(void)
/************************************************************/    
{
  DEBUG_TEXT(DFDB_RCDEXAMPLE, 15, "SCTReadoutModule::startTrigger: Entered");
  
  DEBUG_TEXT(DFDB_RCDEXAMPLE, 15, "SCTReadoutModule::startTrigger: Done");

}

/*****************************************************/
void SCTReadoutModule::stopTrigger(void)
/*****************************************************/
{
  DEBUG_TEXT(DFDB_RCDEXAMPLE, 15, "SCTReadoutModule::stopTrigger: Entered");
  
  DEBUG_TEXT(DFDB_RCDEXAMPLE, 15, "SCTReadoutModule::stopTrigger: Done");

}

/*****************************************************/
void SCTReadoutModule::stopFE(void)
/*****************************************************/
{
  DEBUG_TEXT(DFDB_RCDEXAMPLE, 15, "SCTReadoutModule::stopFE: Done");
}


/*************************************************************************/
DFCountedPointer < Config > SCTReadoutModule::getInfo()
/*************************************************************************/
{
  DEBUG_TEXT(DFDB_RCDEXAMPLE, 15, "SCTReadoutModule::getInfo: Entered");

  DFCountedPointer<Config> configuration = Config::New();
  configuration->set("Number of occurences", 
		  dynamic_cast<SCTScheduledUserAction *> (m_scheduledUserAction0)->getNumberOfTimes());
  return(configuration);

}

/******************************************************************/
void SCTReadoutModule::probe(void)
/******************************************************************/
{
  DEBUG_TEXT(DFDB_RCDEXAMPLE, 15, "SCTReadoutModule::probe: Entered");

  std::cout << " SCTReadoutModule::probe: Entered" << std::endl;

}

/******************************************************************/
void SCTReadoutModule::publishFullStatistics(void)
/******************************************************************/
{
  DEBUG_TEXT(DFDB_RCDEXAMPLE, 15, "SCTReadoutModule::publishFullStatistics: Entered");

}

/****************************************************/
void SCTReadoutModule::clearInfo()
/****************************************************/
{
  DEBUG_TEXT(DFDB_RCDEXAMPLE, 15, "SCTReadoutModule::clearInfo  Function entered");

  if (m_scheduledUserAction0 !=0 ) {
    dynamic_cast<SCTScheduledUserAction *>(m_scheduledUserAction0)->clearInfo();
  }
}

//FOR THE PLUGIN FACTORY
extern "C" 
{
  extern ReadoutModule* createSCTReadoutModule();
}
ReadoutModule* createSCTReadoutModule()
{
  return (new SCTReadoutModule());
}
