//$Id: EmulatedSequentialReadoutModule.cpp,v 1.2 2005/09/21 15:17:52 sctroddq Exp $
/************************************************************************/
/*									*/
/* File             : EmulatedSequentialReadoutModule.cpp		*/
/*									*/
/* Authors          : B.Gorini, M.Joos, J.Petersen ATD/CERN		*/
/*									*/
/********* C 2004 - ROS/RCD  ********************************************/

#include <unistd.h>
#include <iostream>
#include "ROSModules/EmulatedSequentialDataChannel.h"
#include "ROSModules/EmulatedSequentialReadoutModule.h"
#include "DFDebug/DFDebug.h"
#include "DFError/DFOutputStream.h"

using namespace ROS;

/******************************************************************/
EmulatedSequentialReadoutModule::EmulatedSequentialReadoutModule() 
/******************************************************************/
{ 
  DEBUG_TEXT(DFDB_ROSFM, 15, "EmulatedSequentialReadoutModule::constructor: Entered");
}

/*******************************************************************/
EmulatedSequentialReadoutModule::~EmulatedSequentialReadoutModule() 
/*******************************************************************/
{
  DEBUG_TEXT(DFDB_ROSFM, 15, "EmulatedSequentialReadoutModule::destructor: Entered");

  while (m_dataChannels.size() > 0) {
    DataChannel *channel = m_dataChannels.back();
    m_dataChannels.pop_back();
    delete channel;		// we created them
  }

}

// setup gets parameters from configuration
/**************************************************************************************/
void EmulatedSequentialReadoutModule::setup(DFCountedPointer<Config> configuration) 
/**************************************************************************************/
{ 
  DEBUG_TEXT(DFDB_ROSFM, 15, "EmulatedSequentialReadoutModule::setup: Entered");

  m_configuration = configuration;

  bool usePollingModule = true;

  m_numberOfDataChannels = configuration->getInt("numberOfChannels");
  DEBUG_TEXT(DFDB_ROSFM, 20, "EmulatedSequentialReadoutModule::setup: numberOfDataChannels = "
                              << m_numberOfDataChannels);

std::cout << "We are in the setup method" << std::endl;

  // channel parameters
  for(int chan = 0; chan < m_numberOfDataChannels; chan++) {
    m_id.push_back(0);
    m_rolPhysicalAddress.push_back(0);
    m_usePolling.push_back(0);
  }

  for (int chan = 0; chan < m_numberOfDataChannels; chan++)
  {
    m_id[chan] = configuration->getInt("channelId",chan);
    m_rolPhysicalAddress[chan] = configuration->getInt("channelPhysicalAddress",chan);
    DEBUG_TEXT(DFDB_ROSFM, 20, "EmulatedSequentialReadoutModule::setup: chan = "
                                << chan << "  id = " << m_id[chan]
               			<< " rolPhysicalAddress = " << m_rolPhysicalAddress[chan]);
    m_usePolling[chan] = usePollingModule;
  }

}

/*****************************************************/
void EmulatedSequentialReadoutModule::load(void)
/*****************************************************/
{
  DEBUG_TEXT(DFDB_ROSFM, 15, "EmulatedSequentialReadoutModule::load: Entered");

  //Create the DataChannel objects
  for (int chan = 0; chan < m_numberOfDataChannels; chan++) {

    // chan == the Config Id
    EmulatedSequentialDataChannel *channel = new EmulatedSequentialDataChannel(
						 m_id[chan],
					         chan,
						 m_rolPhysicalAddress[chan],
						 m_usePolling[chan],
						 m_configuration);
    m_dataChannels.push_back(channel);

  }

  DEBUG_TEXT(DFDB_ROSFM, 15, "EmulatedSequentialReadoutModule::load: Done");
}

/*****************************************************/
void EmulatedSequentialReadoutModule::unload(void)
/*****************************************************/
{
  DEBUG_TEXT(DFDB_ROSFM, 15, "EmulatedSequentialReadoutModule::unload: Entered");

  DEBUG_TEXT(DFDB_ROSFM, 15, "EmulatedSequentialReadoutModule::unload: Done");
}


/********************************************************/
void EmulatedSequentialReadoutModule::configure(void)
/********************************************************/
{
 
  DEBUG_TEXT(DFDB_ROSFM, 20, "EmulatedSequentialReadoutModule::configure: Entered");

  DEBUG_TEXT(DFDB_ROSFM, 15, "EmulatedSequentialReadoutModule::configure: Done");
}


/**********************************************************/
void EmulatedSequentialReadoutModule::unconfigure(void)
/**********************************************************/
{
  DEBUG_TEXT(DFDB_ROSFM, 15, "EmulatedSequentialReadoutModule::unconfigure: Entered");
  
  DEBUG_TEXT(DFDB_ROSFM, 15, "EmulatedSequentialReadoutModule::unconfigure: Done");
}
    

/************************************************************/    
void EmulatedSequentialReadoutModule::prepareForRun(void)
/************************************************************/    
{
  DEBUG_TEXT(DFDB_ROSFM, 15, "EmulatedSequentialReadoutModule::prepareForRun: Entered");

}

/************************************************************/    
void EmulatedSequentialReadoutModule::startTrigger(void)
/************************************************************/    
{
  DEBUG_TEXT(DFDB_ROSFM, 15, "EmulatedSequentialReadoutModule::startTrigger: Entered");

  for (int chan = 0; chan < m_numberOfDataChannels; chan++) {
    dynamic_cast<EmulatedSequentialDataChannel *>(m_dataChannels[chan])->startTrigger();
  }

}

/************************************************************/    
void EmulatedSequentialReadoutModule::stopTrigger(void)
/************************************************************/    
{
  DEBUG_TEXT(DFDB_ROSFM, 15, "EmulatedSequentialReadoutModule::stopTrigger: Entered");

  for (int chan = 0; chan < m_numberOfDataChannels; chan++) {
    dynamic_cast<EmulatedSequentialDataChannel *>(m_dataChannels[chan])->stopTrigger();
  }

}

/*****************************************************/
void EmulatedSequentialReadoutModule::stopFE(void)
/*****************************************************/
{
  DEBUG_TEXT(DFDB_ROSFM, 15, "EmulatedSequentialReadoutModule::stopFE: Done");
}

/*************************************************************************/
DFCountedPointer < Config > EmulatedSequentialReadoutModule::getInfo()
/*************************************************************************/
{
  DEBUG_TEXT(DFDB_ROSFM, 15, "EmulatedSequentialReadoutModule::getInfo: Entered");

  DFCountedPointer<Config> configuration = Config::New();
  return( configuration);

}


/****************************************************/
void EmulatedSequentialReadoutModule::clearInfo()
/****************************************************/
{
  DEBUG_TEXT(DFDB_ROSFM, 15, "EmulatedSequentialReadoutModule::clearInfo  Function entered");

}

//FOR THE PLUGIN FACTORY
extern "C" 
{
  extern ReadoutModule* createEmulatedSequentialReadoutModule();
}
ReadoutModule* createEmulatedSequentialReadoutModule()
{
  return (new EmulatedSequentialReadoutModule());
}
