#include "Tokenize_Test.h"
#include "../src/LogicErrors.h"
#include "../src/Tokenize.h"
#include <boost/test/unit_test.hpp>

using boost::unit_test_framework::test_suite;

typedef Sct::Tokenize::Tokens Tokens;
typedef Sct::Tokenize::TokenVec TokenVec;
using std::string;

namespace Sct{

  void testTokenizeConstructors() {

    const std::string s = "these:have:colons:within";
    const Tokenize tDefault(s);
    const Tokenize tColon(s,":");
    
  };

  void testTokenizeVecInterface() {

    {
      const string s = "are:these:in:a:vec";

      Tokenize z(s,":");

      TokenVec tv(5);
      tv[0]="are";
      tv[1]="these";
      tv[2]="in";
      tv[3]="a";
      tv[4]="vec";

      TokenVec zv = z.tokenVec();

      BOOST_CHECK(zv.size() == tv.size());
      BOOST_CHECK(zv.size() == 5);

      TokenVec::const_iterator zit = zv.begin();
      TokenVec::const_iterator tit = tv.begin();
      for (; zit != zv.end(); (++zit, ++tit)) {
	BOOST_CHECK( *zit == *tit );
      }; 

      for (unsigned int i=0; i<5; ++i) {
	BOOST_CHECK( tv[i] == zv[i] );
      }; 
    };

  };

  void testTokenizeFunction() {

    {
      const string s = "";

      Tokenize z(s,":");

      Tokens t;
      t.push_back("");

      BOOST_CHECK(z.size() == t.size());

      Tokens::const_iterator zit = z.begin();
      Tokens::const_iterator tit = t.begin();
      for (; zit != z.end(); (++zit, ++tit)) {
	BOOST_CHECK( *zit == *tit );
      }; 
    };

    {
      const string s = "moo";

      Tokenize z(s,":");

      Tokens t;
      t.push_back("moo");

      BOOST_CHECK(z.size() == t.size());

      Tokens::const_iterator zit = z.begin();
      Tokens::const_iterator tit = t.begin();
      for (; zit != z.end(); (++zit, ++tit)) {
	BOOST_CHECK( *zit == *tit );
      }; 
    };

    {
      const string s = ":";

      Tokenize z(s,":");

      Tokens t;
      t.push_back("");
      t.push_back("");

      BOOST_CHECK(z.size() == t.size());

      Tokens::const_iterator zit = z.begin();
      Tokens::const_iterator tit = t.begin();
      for (; zit != z.end(); (++zit, ++tit)) {
	BOOST_CHECK( *zit == *tit );
      }; 
    };

    {
      const string s = "these:have:colons:within";

      Tokenize z(s,":");

      Tokens t;
      t.push_back("these");
      t.push_back("have");
      t.push_back("colons");
      t.push_back("within");

      BOOST_CHECK(z.size() == t.size());

      Tokens::const_iterator zit = z.begin();
      Tokens::const_iterator tit = t.begin();
      for (; zit != z.end(); (++zit, ++tit)) {
	BOOST_CHECK( *zit == *tit );
      }; 
    };

    
    {
      const string s = ":these:have:colons:without:";

      Tokenize z(s,":");

      Tokens t;
      t.push_back("");
      t.push_back("these");
      t.push_back("have");
      t.push_back("colons");
      t.push_back("without");
      t.push_back("");

      BOOST_CHECK(z.size() == t.size());

      Tokens::const_iterator zit = z.begin();
      Tokens::const_iterator tit = t.begin();
      for (; zit != z.end(); (++zit, ++tit)) {
	BOOST_CHECK( *zit == *tit );
      }; 
    };

    {
      const string s = "4these2have1strange3delimiters";

      Tokenize z(s,"1234");

      Tokens t;
      t.push_back("");
      t.push_back("these");
      t.push_back("have");
      t.push_back("strange");
      t.push_back("delimiters");

      BOOST_CHECK(z.size() == t.size());

      Tokens::const_iterator zit = z.begin();
      Tokens::const_iterator tit = t.begin();
      for (; zit != z.end(); (++zit, ++tit)) {
	BOOST_CHECK( *zit == *tit );
      }; 
    };

    {
      const string s = "::lots:::of:empties::";

      Tokenize z(s,":");

      Tokens t;
      t.push_back("");
      t.push_back("");
      t.push_back("lots");
      t.push_back("");
      t.push_back("");
      t.push_back("of");
      t.push_back("empties");
      t.push_back("");
      t.push_back("");

      BOOST_CHECK(z.size() == t.size());

      Tokens::const_iterator zit = z.begin();
      Tokens::const_iterator tit = t.begin();
      for (; zit != z.end(); (++zit, ++tit)) {
	BOOST_CHECK( *zit == *tit );
      }; 
    };

  };


  Tokenize_Test::Tokenize_Test() {
    add(BOOST_TEST_CASE(&testTokenizeConstructors));
    add(BOOST_TEST_CASE(&testTokenizeFunction));
    add(BOOST_TEST_CASE(&testTokenizeVecInterface));
  }
}
