#include "SctNames.h"

#include <mrs/message.h>
#include <cstdlib>
#include <is/info.h>
#include <is/infodictionary.h>
#include <is/infoiterator.h>
#include <is/inforeceiver.h>

using namespace std;

namespace Sct {

string SctNames::getPartitionName() throw() {
     static const std::string partitionName = getenv("TDAQ_PARTITION") ? getenv ("TDAQ_PARTITION") : "SCT";
     return partitionName;
}

string SctNames::getEventDataName() throw() {
    return "EventData";
}

string SctNames::getFittedDataName() throw() {
    return "FittedData";
}

string SctNames::getDcsServerName() throw() {
    return "SCTDCS_Data";
}

string SctNames::getTestDataName() throw() {
    return "TestData";
}

string SctNames::getControlDataName() throw() {
    return "ControlData";
}

string SctNames::getRetrievedDataName() throw() {
	    return "RetrievedData";
}

string SctNames::getTempDir() throw() {
    static std::string dir = getenv("SCT_SCRATCH_DIR") ? getenv("SCT_SCRATCH_DIR") : "/tmp";
    return dir;
}

std::string SctNames::getPersistentDir() throw(){
     return m_persistent_dir;
}

void SctNames::setPersistentDir(const char* newDirectory) throw(){
  m_persistent_dir = newDirectory;
 }

string SctNames::m_persistent_dir = getenv("SCT_PERSISTENT_DIR") ? getenv ("SCT_PERSISTENT_DIR") : "/tmp";


std::string SctNames::getLogDir() throw(){
     static std::string dir = getenv("TDAQ_LOGS_PATH") ? getenv ("TDAQ_LOGS_PATH") : "/tmp";
     return dir;
}

string SctNames::getLocation() throw() {
    return "Oxford";
}

string SctNames::getUser() throw() {
    return "daquser";
}

IPCPartition& SctNames::getPartition() throw() {
    static IPCPartition partition (getPartitionName());
    return partition;
}

ISInfoDictionary& SctNames::getISDictionary() throw() {
    static ISInfoDictionary dictionary(getPartition());
    return dictionary;
}

MRSStream& SctNames::Mrs() {
    static MRSStream mrs(getPartition());
    return mrs;
}

static void dummyMRSCallBack(MRSCallbackMsg *mc) {
  std::cout << "Ignore this: Got message while testing MRS availability\n";
}

bool SctNames::isMrsValid() {
  // An alternative...
//   list<IPCPartition> l;
//   IPCPartition::getPartitions(l);
//   return l.size() > 0;

  IPCPartition p("SCT");
  return p.isValid();
}

}
