#include "Defect.h"
#include "DefectPrototype.h"

std::ostream&operator << (std::ostream& os, const SctData::Defect& d){
  os << "DEFECT NAME           : " << d.getPrototype().getName()<<std::endl;
  os << "FIRST CHANNEL         : " << d.getModuleElement().getFirst()<<std::endl;
  os << "LAST CHANNEL          : " << d.getModuleElement().getLast()<<std::endl;
  if (d.getParameter()){
    os << "PARAMETER             : " << *d.getParameter() << std::endl;
  }
  return os;
}

namespace SctData{
Defect::Defect(const DefectPrototype& proto, ModuleElement element, boost::optional<double> opt_par) :
  prototype(&proto), element(element), opt_par(opt_par)
{
}

const ModuleElement& Defect::getModuleElement() const {
  return element;
}

const DefectPrototype& Defect::getPrototype() const {
  return *prototype;
}

bool Defect::operator==(const Defect& defect) const{
  return element==defect.element && *prototype==*defect.prototype;
}

bool Defect::operator!=(const Defect& defect) const{
  return (!this->operator==(defect));
}

/** Return optional parameter. BD */
 boost::optional<double> Defect::getParameter() const {
   return opt_par;
 }
 
 /** Set optional parameter */
 void Defect::setParameter(double param) {
   opt_par = boost::optional<double>(param);
 }
 
}
