#ifndef SCTDATA_DEFECT_H
#define SCTDATA_DEFECT_H

#include "ModuleElement.h"
#include <iostream>
#include <boost/optional.hpp>

namespace SctData {
    
class DefectPrototype;

/**
  Represents a defect - basically a union of a ModuleElement and 
  a DefectPrototype.
  @author Matthew Palmer
  Added optional parameter -- to be used for things like Stuck_Cell list, 
  noise of NO_HI channels and so on. 
  @author Bilge Demirkoz
  */
class Defect {
public:
    Defect(const DefectPrototype& proto, ModuleElement element,
	   boost::optional<double> opt_par=boost::optional<double>::optional());

    /**
    Gets the module element affected by this defect.
    @note Nothrow
    */
    const ModuleElement& getModuleElement() const;

    /**
      Get the DefectPrototype
      @note Nothrow
      */
    const DefectPrototype& getPrototype() const;
    
    /**
      Compares two defects returning true if they are the same
      @note Nothrow
      */
    bool operator==(const Defect& defect) const;

    /**
      Compares two defects returning true if they are differenct
      @note Nothrow
      */
    bool operator!=(const Defect& defect) const;

    /** Return optional parameter */
    boost::optional<double> getParameter() const;

    /** Set optional parameter */
    void setParameter(double param);


private:
    const DefectPrototype* prototype;		///< The prototype - pointer to allow assignment operator to work - points to static object so no need to delete
    ModuleElement element;			///< The element
    boost::optional<double> opt_par;
};
}

std::ostream& operator << (std::ostream& os, const SctData::Defect& d);

#endif //SCTDATA_DEFECT_H
