#ifndef MARKSPACERATIOTESTRESULT_H
#define MARKSPACERATIOTESTRESULT_H

#include "TestResult.h"
#include "Sct/RangedVector.h"

namespace SctData{
  /**
     The class represents the result of a MarkSpaceRatio test.	
     @author Christopher Lester
  */
  class MarkSpaceRatioTestResult : public TestResult {
  public:
    MarkSpaceRatioTestResult() throw();
    /**
       Constructor.
    */
    MarkSpaceRatioTestResult(unsigned int runNumber, const string& moduleName) throw();
    /**
       Destructor
    */
    virtual ~MarkSpaceRatioTestResult() throw();
	
    /// Override
    virtual string getClassName() const throw();

    /// Contains results for a particular value of the MSR register
    class MsrPoint{
    public:
      // mean occupancy before bit-flip
      double occ0;
      // mean occupancy after bit-flip
      double occ1;

      // mean occupancy before bit-flip
      double occ0flip;
      // mean occupancy after bit-flip
      double occ1flip;

      /// MSR register value
      unsigned short msr;
      double getOccupancy(unsigned ilink) const;
      double getOccupancyFlipped(unsigned ilink) const;
    };

    /// add a MsrPoint
    void addPoint(const MsrPoint& p);

    /// get number of MsrPoint s
    unsigned getNPoints() const;

    /// get a particular point
    MsrPoint getPoint(unsigned index) const;

    /// get optimum M-S-R
    double getOptimum() const;
    /// set optimum M-S-R
    void setOptimum(double opt);

  private:
    static string vector_name;
    RangedVector<MsrPoint> msr_result;
    double optimum;
  };
}// end of namespace SctData
#endif //#ifndef TIMEWALKTESTRESULT_H
