#ifndef TXCURRENTTESTRESULT_H
#define TXCURRENTTESTRESULT_H

#include "TestResult.h"
#include "Sct/RangedVector.h"

namespace SctData{
  /**
     The class represents the result of a TxCurrent test.	
     @author Christopher Lester
  */
  class TxCurrentTestResult : public TestResult {
  public:
    TxCurrentTestResult() throw();
	
    /**
       Constructor.
    */
    TxCurrentTestResult(unsigned int runNumber, const string& moduleName) throw();
	
    /**
       Destructor
    */
    virtual ~TxCurrentTestResult() throw();
	
    //Override
    virtual string getClassName() const throw();

    class StreamResult {
    public:
      typedef int ThresholdType;
      StreamResult() {
        setInvalid();
      }
      void setInvalid() {
        m_minErrorFreeTxCurrent = NOT_YET_SET_OR_INVALID;
        m_maxErrorFreeTxCurrent = NOT_YET_SET_OR_INVALID;
        m_bestTxCurrent         = NOT_YET_SET_OR_INVALID;
      };
      bool isInvalid() const {
        return 
          m_minErrorFreeTxCurrent == NOT_YET_SET_OR_INVALID ||
          m_maxErrorFreeTxCurrent == NOT_YET_SET_OR_INVALID ||
          m_bestTxCurrent         == NOT_YET_SET_OR_INVALID;
      };
      int minErrorFreeTxCurrent() const { return m_minErrorFreeTxCurrent; };
      int maxErrorFreeTxCurrent() const { return m_maxErrorFreeTxCurrent; };
      int bestTxCurrent() const { return m_bestTxCurrent; };
      void setMinErrorFreeTxCurrent(int val) {
        m_minErrorFreeTxCurrent = (this->isAllowableTxCurrentPrivateVar(val)) ? val : NOT_YET_SET_OR_INVALID;
      };
      void setMaxErrorFreeTxCurrent(int val) {
        m_maxErrorFreeTxCurrent = (this->isAllowableTxCurrentPrivateVar(val)) ? val : NOT_YET_SET_OR_INVALID;
      };
      void setBestTxCurrent(int val) {
        m_bestTxCurrent = (this->isAllowableTxCurrentPrivateVar(val)) ? val : NOT_YET_SET_OR_INVALID;
      };
    private:
      static bool isAllowableTxCurrentPrivateVar(const int val) {
        return val == NOT_YET_SET_OR_INVALID || (val>=MIN_LEGITIMATE_TX_CURRENT && val <= MAX_LEGITIMATE_TX_CURRENT);
      };
    public:
      static bool isGoodPhysicalTxCurrent(const int val) {
        return val != NOT_YET_SET_OR_INVALID && val>=MIN_LEGITIMATE_TX_CURRENT && val <= MAX_LEGITIMATE_TX_CURRENT;
      };
    private:
      int m_minErrorFreeTxCurrent;
      int m_maxErrorFreeTxCurrent;
      int m_bestTxCurrent;
      static const int NOT_YET_SET_OR_INVALID=-1;
      static const int MIN_LEGITIMATE_TX_CURRENT=0;
      static const int MAX_LEGITIMATE_TX_CURRENT=255;
    };
	
    /**
       get StreamResult @param index stream in range 0->1 
    */
    StreamResult& getStreamResult(const unsigned index);
    /**
       const get StreamResult @index stream in range 0->1
    */
    const StreamResult& getStreamResult(const unsigned index) const;
  private:
    RangedVector<StreamResult> streamResults;

  };
}// end of namespace SctData
#endif //#ifndef TIMEWALKTESTRESULT_H
