#ifndef SCTTEST_CONFIGURATION_H
#define SCTTEST_CONFIGURATION_H

#include <string>
#include <ipc/object.h>
#include "SctTestApi.hh"

using std::string;

namespace SctTest {

class Configuration : public IPCNamedObject<POA_SctTestApi::Configuration> {
public:
    static Configuration& instance();

    /**
     * Sets the data file.
     * @throws Sct::FileException if there is a problem with the file
     */

    // Presumably these are "handy" non-CORBA interface methods:
    // Unfortunatley, many of them ambiguate the CORBA interface methods under the new orb, so those of them that clash will have to be renamed or removed ...

    void handy_setDataFile(string file);
    string handy_getDataFile() const;

    void handy_setFastTimes();
    void handy_setRealisticTimes();
    void handy_setScanTime(unsigned int time);
    unsigned int handy_getScanTime() const;
    void handy_setMultiDspEfficiency(double efficiency);
    double handy_getMultiDspEfficiency() const;
    void handy_setReadoutTime(unsigned int time);
    unsigned int handy_getReadoutTime() const;

    //CORBA interfafce methods:
    virtual void setDataFile (const char * file);
    virtual char * getDataFile ();
    virtual void setFastTimes ();
    virtual void setRealisticTimes ();
    virtual void setScanTime (CORBA::Long time);
    virtual CORBA::Long getScanTime ();
    virtual void setMultiDspEfficiency (CORBA::Double efficiency);
    virtual CORBA::Double getMultiDspEfficiency ();
    virtual void setReadoutTime (CORBA::Long time);
    virtual CORBA::Long getReadoutTime ();
    virtual void shutdown() {};

private:
    Configuration();

    string dataFile;            ///<The data file
    unsigned int scanTime;      ///<The time for a scan of 1 module
    unsigned int readoutTime;   ///<Time to readout 1 module
    double dspEfficiency;       ///<Efficiency of multiple DSPs (max 1.0)
};
}

#endif //SCTTEST_CONFIGURATION_H
