#include <list>
#include <utility>

#include "SctApiFwd.h"

namespace SctApi {
class RodInfo {
 public:
  /// Minimum link which is configured to receive data
  int minLink;

  /// Which modules have their redundant TTC provided by another ROD
  /**
     The key is the mid on this ROD (ch < 48)
   */
  std::map<unsigned long, unsigned long> offRODTTC;

  /// Which modules have their RX on another ROD
  /**
    Key is mid on this ROD (ch >= 48)
   */
  std::map<unsigned long, unsigned long> offRODRX;

  /**
     Cache of the fibre mappings which are retrieved from the configuration service.
   */
  boost::shared_array<unsigned char> fibreMappings;

  /// Error masks for each of 96 links
  /**
     For each link:
       bit  0: Error in header
       bit  1: Error in trailer
       bit  2: "Flagged" error (from ABCD)
       bit  3: Error in sync bit
       bit  4: Mismatched hit pattern
       bit  5: L1 mismatch
       bit  6: BC mismatch
       bit  7: Timeout error (waiting for reply from ABCD)
       bit  8: ROD FIFO almost full error
       bit  9: Data overflow (Too much data for ROD)
       bit 10: Error in chip ordering
       bit 11: Invalid chip number
       bit 12-15: ignored
   */
  unsigned int linkErrorMasks[96];

  /// Constructor defaults to using the last link
  RodInfo() : minLink(97) { for(int l=0; l<96; l++) linkErrorMasks[l] = 0; }
};
}
