#ifndef SCT_SCTAPI_TRIGGERIMPL_H
#define SCT_SCTAPI_TRIGGERIMPL_H

/**
   Implement SctApi::Trigger class
*/

//fwddec
namespace SctApi {
  class TriggerImpl;
};

// includes
#include <boost/shared_ptr.hpp>
#include "Trigger.h"

//defn
namespace SctApi {

  class TriggerImpl : public ::SctApi::Trigger {
    TriggerImpl &operator=(const TriggerImpl &);
  public:
    /// Create default L1A trigger
    TriggerImpl();

    TriggerImpl(const TriggerImpl &);

    static boost::shared_ptr<TriggerImpl> clone(const boost::shared_ptr< ::SctApi::Trigger> other);

    /// Call parent's destructor
    virtual ~TriggerImpl();

    void singleL1A();
    void doubleL1A(short unsigned int delay);
    void delayedL1A(short unsigned int delay);
    void calL1A(short unsigned int delay);
    void pulseL1A(short unsigned int delay);
    void softL1A(short unsigned int delay);
    void softCalL1A(short unsigned int delay, short unsigned int delay2);
    void softPulseL1A(short unsigned int delay, short unsigned int delay2);
    void softBc(short unsigned int delay);
    void soft();
    void bc();
    void bcL1A(short unsigned int delay);
    void bcCalL1A(short unsigned int delay, short unsigned int delay2);
    void bcPulseL1A(short unsigned int delay, short unsigned int delay2);

    void setCommIncr(unsigned short command, unsigned short incr);
    void getCommIncr(unsigned short &command, unsigned short &incr) const;

    const RODTriggers getRODTriggers(int scan_point) const;
    const TIMTriggers getTIMTriggers(int scan_point) const;

    virtual bool isValidRODTrigger() const;
    virtual bool isValidTIMTrigger() const;
    virtual bool timNeedsSequencer() const;
    /*   void copy(const ::SctApi::Trigger &trig); */

    /** clear all commands from trigger 
	postcondition: trigger is invalid - need to addCommand to it */
    virtual void clear();
    
    virtual void addCommand(UINT16 first, UINT32 second);

    virtual Source getSource() const;
    virtual void setSource(Source);
    
    /// kHz
    virtual void setFrequency(double freq);
    /// kHz
    virtual double getFrequency() const;

    // what sort of randomiser to use?
    virtual int getRandom() const;
    virtual void setRandom(int);
    
    std::string print() const;
  private:
    //   /// Command values (max 6)
    //   TArrayS trigCommand;

    //   /// Data values (max 6)
    //   TArrayI trigData;

    RODTriggers trigSequence;

    /// Which command value to increment
    int incCmd;

    /// Increment trigger data by (if 0 then don't!)
    int incData;
    
    Source m_source;
    double m_freq;

    int m_random;
  };


};

#endif
