#include <ipc/core.h>
#include <ipc/object.h>
#include <ipc/server.h>
#include <cmdl/cmdargs.h>
#include <pmg/pmg_initSync.h>

#include <boost/bind.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/thread.hpp>

#include "SctApi/SctApi.h"
#include "SctApi/Idiosyncrasy.h"
#include "sctConfIPC/configipc.h"

#include "SctApiCrateServer.h"
#include "Sct/ApplicationStartupDebugTools.h"
#include "Sct/SctNames.h"

// Not needed AT THE MOMENT but keep for future use: void dcsPollThread(SctApi::SctApi *sctApi);

int main(int argc, char** argv) 
{
  std::list< std::pair< std::string, std::string > > opt = IPCCore::extractOptions(argc,argv);
  // Things you might like to consider adding in the future:
  //opt.push_front( std::make_pair( std::string("threadPerConnectionPolicy"), std::string("0") ) );
  //opt.push_front( std::make_pair( std::string("maxServerThreadPoolSize"),   std::string("3") ) );
  //opt.push_front( std::make_pair( std::string("threadPoolWatchConnection"), std::string("0") ) );
  IPCCore::init(opt); // Try reentrant for DDC IS callbacks
  Sct::ApplicationStartupDebugTools::announceStartOfMain(argc, argv, __FILE__, __LINE__);  

  CmdArgStr partitionName('p', "partition", "partition-name", "partition to work in. (Default: SCT)");
  CmdArgStr ucidArg('u', "ucid",      "unique-crate-id",           "The unique crate id taked the form UCID:partition.crate, eg UCID:2.3 for partition2 crate3.  This program needs the ucid so that it can decide how to be different from the other crate controllers on other SBCs",CmdArg::isREQ);

  CmdLine   cmd(*argv, &partitionName, &ucidArg, NULL);
  CmdArgvIter arg_iter(--argc, ++argv);
  cmd.description("This program implements the SctApiCrateServer, i.e. the part of the SctApi that sits on each crate, rather than the higer level bit that talks to each crate.\n");

  // Set up defaults:
  partitionName=::Sct::SctNames::getPartitionName().c_str();

  // Parse arguments
  cmd.parse(arg_iter);
  // Parsing complete
    
  IPCPartition p(partitionName);
  try {
    std::string ucidStr = static_cast<const char *>(ucidArg);
    Sct::UCID ucid(ucidStr);
    SctApi::Idiosyncrasy id(ucid);
    
    std::auto_ptr<SctApi::SctApi> api(new SctApi::SctApi(id, boost::shared_ptr<SctConfiguration::Configuration>(new SctConfiguration::ConfigIPC)));

    std::auto_ptr<SctApiCrateServer> h(new SctApiCrateServer(*api));

    // Try publishing it
    if (!h->publish()) {
      std::cout << "Failed to publish SctApiCrateServer" << std::endl;
      return 0;
    } 

    std::cout << "Serving...\n";
    // The following line commented by Alan 8/11/2004 folloing
    // discussion with bruce about problems during scanning
    // if this thread runs
    // boost::thread dcsThread(boost::bind(&dcsPollThread, api.get()));
    std::cout << "sent bind to dctPollThread" << std::endl;
    pmg_initSync();
    h->run();
    std::cout << "Run over" << std::endl;
    h->withdraw();
  } catch(SctConfiguration::ConfigurationException &c) {
    std::cout << "ConfigurationException: " << c.what() << std::endl;

    MRSStream mrsStream(p);
    mrsStream << "SCTAPI_FAIL" << MRS_FATAL << MRS_QUALIF("SctApiCrateServer") << MRS_TEXT("Configuration server not running!") << ENDM;
    
    exit(1);
  } catch(std::exception & c) {
    std::cout << "Exception: " << c.what() << std::endl;

    MRSStream mrsStream(p);
    mrsStream << "SCTAPI_FAIL" << MRS_FATAL << MRS_QUALIF("SctApiCrateServer") << MRS_TEXT("Configuration server not running!") << ENDM;
    
    exit(2);
  };
}

/* Whole method needs a rethink for multicrate, so commented out for the moment since it is not used anayway

void dcsPollThread(SctApi::SctApi *sctApi) {
  // Let everything settle down first
  sleep(3);

  IPCPartition partition(::Sct::SctNames::getPartitionName());

  ISInfoDictionary isDict(partition);

  while(1) {
    std::cout << "Polling BOC DCS\n";

    std::list<SctApi::RodLabel> rodList = sctApi->listRods();

    std::string isBaseName("SCTAPIServer.BOC_DCS");

    for(std::list<SctApi::RodLabel> ::const_iterator rodI = rodList.begin();
	rodI != rodList.end();
	rodI ++) {
      std::vector<double> thisMonitor 
	= sctApi->getBOCMonitorArray(rodI->partition, rodI->crate, rodI->rod);

      std::string isRodName(isBaseName
			    + "." + boost::lexical_cast<std::string>(rodI->partition) 
			    + "." + boost::lexical_cast<std::string>(rodI->crate) 
			    + "." + boost::lexical_cast<std::string>(rodI->rod));

      for(int i=0; i<12; i++) {
	std::string isName(isRodName + ".ADC" + boost::lexical_cast<std::string>(i));

	ISInfoDouble isValue(thisMonitor[i]);

	if(isDict.contains(isName.c_str())) {
	  isDict.update(isName.c_str(), isValue);
	} else {
	  isDict.insert(isName.c_str(), isValue);
	}
      }
    }

    sleep(60);
  }
}

*/
