#ifndef SCTAPIIPC_SCTAPISERVER_H
#define SCTAPIIPC_SCTAPISERVER_H

#include <map>

#include <boost/shared_ptr.hpp>

#include <ipc/object.h>
#include <ipc/server.h>
#include <mrs/message.h>
#include <vector>

namespace SctApi {
  class SctApi;
  class Trigger;
  class TriggerImpl;
  class Scan;
  class ScanDefImpl;
}

#include "Sct_SctApi/SctApi.hh"
#include "CrateStore.h"

#include "Sct/Addressing.h"

/**
   This gets defined in the main program,
   its needed to attach the Scans and Triggers
*/

// other fwd decs:
namespace Sct {
  class ScanStatusManager;
};

// Obsolete?
// extern std::auto_ptr<IPCServer> ipcServer; 

class ScanServer;
class TriggerServer;

/**
   The SctApi ILU implementation/wrapper
 */
class SctApiServer : public IPCNamedObject<POA_Sct_SctApi::SctApiIPC>,
                     public IPCServer {

 public:
  typedef std::vector<Sct_SctApi::CrateIPC_var> CrateIPC_vars;
  typedef std::vector<Sct::UCID> CrateDescriptors; // perhaps later will be typedef to CrateIPC_vars, but for the moment will be vector of UCIDs!

  SctApiServer(const CrateDescriptors & apis,
               const char* interfaceName);
  virtual ~SctApiServer();		// destructor

 // methods

  virtual Sct_SctApi::CrateIPC_ptr getCrate(const Sct::Corba::UCID& ucid);
  virtual Sct_SctApi::CrateIPC_ptr getCrateLegacy(CORBA::ULong partition, CORBA::ULong crate);
  virtual Sct::Corba::UCIDs* getCrateUCIDs();
  virtual void initialiseAll (CORBA::ULong runNumber);
  virtual void shutdownAll ();
  virtual char* status ();
  virtual void setRunNumber (CORBA::ULong newRun);
  virtual void setScanNumber (CORBA::ULong newScan);
  virtual CORBA::ULong getRunNumber ();
  virtual CORBA::ULong getScanNumber ();
  virtual CORBA::Boolean changeRunMode(Sct_SctApi::RunType mode);
  virtual CORBA::Boolean changeEventFormat(Sct_SctApi::EventFormatType format);
  virtual void unsetDebugOption (const char * opt);
  virtual void setDebugOption (const char * opt);
  virtual Sct_SctApi::DebugOptionList * listEnabledDebugOptions ();
  virtual Sct_SctApi::DebugOptionList * listDebugOptions ();
  virtual void unsetBusy();
  virtual void standardRegisterDumpURID (const Sct::Corba::URID& urid);
  virtual void standardRegisterDumpAll ();
  virtual sctConf::ModuleList * getModuleList ();
  virtual void configureBOC (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual Sct_SctApi::BOCMonitorSequence* getBOCMonitorArray(CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual void loadConfiguration (const char * filename);
  virtual void storeModuleConfigurations (Sct_SctApi::BankType bank);
  virtual void storeBOCConfigurations ();
  virtual void configureAllModules ();
  virtual sctConf::ABCDModule retrieveModule (CORBA::ULong mid);

  /* REMOVE
  virtual CORBA::Short dspBlockDump (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::Long dspStart, CORBA::Long numWords, CORBA::Long dspNumber);
  virtual Sct_SctApi::DataBlock * dspBlockRead (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::Long dspStart, CORBA::Long numWords, CORBA::Long dspNumber);
  virtual CORBA::Short dspBlockWrite (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, const Sct_SctApi::DataBlock & buffer, CORBA::ULong dspAddress, CORBA::Long dspNumber);
  virtual CORBA::ULong dspSingleRead (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong dspAddr, CORBA::Long dspNumber);
  virtual void dspSingleWrite (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong dspAddr, CORBA::ULong val, CORBA::Long dspNumber);
  virtual Sct_SctApi::DataBlock * readSlaveDsp (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::Short s, CORBA::ULong add, CORBA::ULong words);
  */
  virtual CORBA::ULong findModuleSerial (const char * sn);
  virtual char * convertToString (CORBA::ULong mid);
  virtual CORBA::ULong findModule (CORBA::UShort mur, CORBA::UShort number);
  virtual CORBA::ULong findModuleRod (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong channel);
  virtual CORBA::ULong findBarrelModule (CORBA::ULong barrel, CORBA::ULong row, CORBA::Short number);
  virtual void convertToBarrelModule (CORBA::ULong mid, CORBA::ULong & barrel, CORBA::ULong & row, CORBA::Short & number);
  virtual CORBA::ULong findEndcapModule (CORBA::Long disk, CORBA::Long ring, CORBA::Long number);
  virtual void convertToEndcapModule (CORBA::ULong mid, CORBA::Long & disk, CORBA::ULong & ring, CORBA::ULong & number);
  virtual void getABCDModules (Sct_SctApi::BankType bank);
  virtual void getABCDModule (CORBA::ULong mid, Sct_SctApi::BankType bank);
  // REMOVE virtual sctConf::ABCDModule getABCDModuleRaw (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong slot, Sct_SctApi::BankType bank);
  virtual void setABCDModule (CORBA::ULong mid, const Sct_SctApi::BankList& banks);
  virtual void setABCDModules (const Sct_SctApi::BankList& bank);

  virtual void copyABCDModule (CORBA::ULong mid, Sct_SctApi::BankType source, const Sct_SctApi::BankList& to);
  virtual void copyABCDModules (Sct_SctApi::BankType source, const Sct_SctApi::BankList& to);
  virtual void sendABCDModule (CORBA::ULong mid, Sct_SctApi::BankType bank, CORBA::ULong type);
  virtual void sendABCDModules (Sct_SctApi::BankType bank);
  virtual void modifyABCDMask (CORBA::ULong mid, const Sct_SctApi::DataBlock & mask);
  virtual void modifyABCDTrims (CORBA::ULong mid, const Sct_SctApi::CharBlock & trims);
  virtual void modifyABCDRC (CORBA::ULong mid, CORBA::ULong chip, CORBA::UShort index, CORBA::Double p0, CORBA::Double p1, CORBA::Double p2);
  virtual void setAutoUpdateBanks(const Sct_SctApi::BankList& banks);
  virtual void modifyABCDVar (CORBA::ULong typ, CORBA::Double var);
  virtual void modifyABCDModuleVar (CORBA::ULong mid, CORBA::ULong typ, CORBA::Double var);
  virtual void modifyABCDChipVar (CORBA::ULong mid, CORBA::ULong chip, CORBA::ULong typ, CORBA::Double var);
  virtual void modifyABCDVarAllModulesOneChip (CORBA::ULong chip, CORBA::ULong typ, CORBA::Double var);
  virtual void modifyABCDVarROD (CORBA::ULong typ, CORBA::Double var, const Sct_SctApi::BankList& banks);
  virtual void modifyABCDModuleVarROD (CORBA::ULong mid, CORBA::ULong typ, CORBA::Double var, const Sct_SctApi::BankList& banks);
  virtual void modifyBOCParam (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong channel, CORBA::ULong type, CORBA::ULong val);
  virtual void lasersOff ();
  virtual void timSetFrequency (CORBA::ULong partition, CORBA::ULong crate, CORBA::Double trigFreq, CORBA::Double rstFreq);
  virtual void freeTriggers (CORBA::ULong partition, CORBA::ULong crate);
  virtual void stopTriggers (CORBA::ULong partition, CORBA::ULong crate);
  virtual void timL1A (CORBA::ULong partition, CORBA::ULong crate);
  virtual void timCalL1A (CORBA::ULong partition, CORBA::ULong crate, CORBA::Short delay);
  virtual CORBA::Short timReadRegister (CORBA::ULong partition, CORBA::ULong crate, CORBA::Short value);
  virtual void timWriteRegister (CORBA::ULong partition, CORBA::ULong crate, CORBA::Short reg, CORBA::Short value);
  CORBA::ULong sendTriggers (CORBA::ULong partition, CORBA::ULong crate, Sct_SctApi::Trigger_ptr trig, CORBA::ULong ntrigs, CORBA::UShort point_in_scan);
  virtual void sendRodTrigger (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, Sct_SctApi::Trigger_ptr trig);

  virtual void doScan (Sct_SctApi::Scan_ptr thisScan);
  virtual void doRawScan (Sct_SctApi::Scan_ptr thisScan, CORBA::Long delay, CORBA::Long width, CORBA::Boolean configure, CORBA::Boolean clockByTwo);
  virtual Sct_SctApi::ScanMonitor_ptr startAsyncScan(Sct_SctApi::Scan_ptr thisScan);
  virtual CORBA::Boolean hasFinishedScanning();
  virtual void abortScan();

  //REMOVE virtual void printBOCSetup (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual Sct_SctApi::BOCChannelList * currentBOCSetup (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual sctConf::BOCGlobalConfig currentBOCRegisters (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual void sendL1A (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual Sct_SctApi::CharBlock * probe (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual Sct_SctApi::CharBlock * probeHarness (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong harness);
  virtual Sct_SctApi::CharBlock * probeWithTrigger (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, Sct_SctApi::Trigger_ptr trig);
  virtual Sct_SctApi::CharBlock * probeHarnessWithTrigger (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, Sct_SctApi::Trigger_ptr trig, CORBA::ULong harness);
  virtual Sct_SctApi::CharMatrix * probeScanHarnessWithTrigger (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, Sct_SctApi::Scan_ptr trig, CORBA::ULong harness);
  virtual CORBA::Boolean checkAllModulesProbe (const char * chars);
  virtual Sct_SctApi::AutoResultList * autoConfigureScan ();
  virtual Sct_SctApi::Scan_ptr createScan ();
  virtual Sct_SctApi::Scan_ptr getDefaultScan ();
  // Next two NOT USED
  virtual Sct_SctApi::Trigger_ptr createTrigger ();
  // virtual Sct_SctApi::Trigger_ptr getDefaultTrigger ();
  virtual Sct_SctApi::ScanList * knownScans ();

 private:
  typedef boost::shared_ptr<ScanServer> SharedScanServer;
  boost::shared_ptr<Sct::ScanStatusManager> m_assm;

 public:
  typedef std::list<SharedScanServer> Scans;
  Scans scans;

  ScanServer *defaultScan;
  TriggerServer *defaultTrigger;

  static SctApiServer *singleton;

  virtual void shutdown();

 protected:
  /// Note that this "worker()" method is a stop-gap work-around only needed during the *transition* to multicrate.  For the moment, it returns an "arbitrary" worker, out of all of the registered CrateController workers. It is only for hanger-on methods that do not yet know about multi crate and expect SctApiServer to only know about one worker.  Eventually, every method of this class will have to avoid calling worker() and must instead distribute its work across the appropriate workers in "workers()" ... you were warned.
  Sct_SctApi::CrateIPC_var worker();   // note that this "worker()" method is a stop-gap work-around only needed during the *transition* to multicrate.  For the moment, it returns an "arbitrary" worker, out of all of the registered CrateController workers. It is only for hanger-on methods that do not yet know about multi crate and expect SctApiServer to only know about one worker.  Eventually, every method of this class will have to avoid calling worker() and must instead distribute its work across the appropriate workers in "workers()" ... you were warned.
  typedef Sct::CrateStore::Worker Worker;
  typedef Sct::CrateStore::Workers Workers;
  Workers & workers();

 protected:
  MRSStream mrsStream;
  
  // NOT NEEDED CORBA::Short ddcSingleTimeout;
  // NOT NEEDED CORBA::Short ddcMultipleTimeout;

 private:
  Sct::CrateStore m_crateStore;
  unsigned long m_localRunNumber;
  unsigned long m_localScanNumber;
};

#endif
