/******************* TriggerServer **********************/

#include "TriggerServer.h"

#include "Sct/SctNames.h"
#include "SctApi/TriggerImpl.h"
#include "Sct/SequenceUtils.h"
#include "Sct/OmniMarshalling.h"

TriggerServer::TriggerServer() 
  : IPCNamedObject<POA_Sct_SctApi::Trigger>(Sct::SctNames::getPartition(),
					    getServerName().c_str()),
    m_worker(new SctApi::TriggerImpl()) {
}

TriggerServer::TriggerServer(const boost::shared_ptr< ::SctApi::Trigger> other) 
  : IPCNamedObject<POA_Sct_SctApi::Trigger>(Sct::SctNames::getPartition(),
					    getServerName().c_str()),
    m_worker(other) {
}

TriggerServer::~TriggerServer()
{
}

void TriggerServer::clear(){
  worker()->clear();
}

void TriggerServer::addCommand(Sct_SctApi::RODTriggerFirstType first, 
                               Sct_SctApi::RODTriggerSecondType second){
  worker()->addCommand(first, second);
}

void TriggerServer::shutdown() 
{
#warning THIS IS THE OLD destroy(ipcStatus *_status) METHOD, AND NOW IT IS THE SHUTDOWN METHOD.  THIS IS QUITE PROBABLY THE WRONG THING TO DO WITH THIS METHOD ... I DONT KNOW!
#warning PARTICULARLY BEWARE THE COMMENTING OF decRef();
  // decRef();
}

int TriggerServer::nextNumber = 0;

std::string TriggerServer::getServerName() {
  std::ostringstream serverName;
  serverName << "TriggerServer" << nextNumber++;
  std::cout << "Trigger name: " << serverName.str() << std::endl;;
  return serverName.str();
}

char* TriggerServer::print () 
{
  std::cout << "Try to print a trigger\n";
  return Sct::copyStringToCorba(worker()->print());
}

void TriggerServer::singleL1A () 
{
  worker()->singleL1A();
}

void TriggerServer::doubleL1A (CORBA::UShort delay) 
{
  worker()->doubleL1A(delay);
}

void TriggerServer::delayedL1A (CORBA::UShort delay) 
{
  worker()->delayedL1A(delay);
}

void TriggerServer::calL1A (CORBA::UShort delay) 
{
  worker()->calL1A(delay);
}

void TriggerServer::pulseL1A (CORBA::UShort delay) 
{
  worker()->pulseL1A(delay);
}

void TriggerServer::softL1A (CORBA::UShort delay) 
{
  worker()->softL1A(delay);
}

void TriggerServer::softCalL1A (CORBA::UShort delay, CORBA::UShort delay2) 
{
  worker()->softCalL1A(delay, delay2);
}

void TriggerServer::softPulseL1A (CORBA::UShort delay, CORBA::UShort delay2) 
{
  worker()->softPulseL1A(delay, delay2);
}

void TriggerServer::softBc (CORBA::UShort delay) 
{
  worker()->softBc(delay);
}

void TriggerServer::soft () 
{
  worker()->soft();
}

void TriggerServer::bc () 
{
  worker()->bc();
}

void TriggerServer::bcL1A (CORBA::UShort delay) 
{
  worker()->bcL1A(delay);
}

void TriggerServer::bcCalL1A (CORBA::UShort delay, CORBA::UShort delay2) 
{
  worker()->bcCalL1A(delay, delay2);
}

void TriggerServer::bcPulseL1A (CORBA::UShort delay, CORBA::UShort delay2) 
{
  worker()->bcPulseL1A(delay, delay2);
}

void TriggerServer::setCommIncr (CORBA::UShort command, CORBA::UShort incr)
{
  worker()->setCommIncr(command, incr);
}

void TriggerServer::getCommIncr (CORBA::UShort & command, CORBA::UShort & incr)
{
  worker()->getCommIncr(command, incr);
}

::Sct_SctApi::Trigger::Source TriggerServer::getSource(){
  return (::Sct_SctApi::Trigger::Source)(int)worker()->getSource();
}

void TriggerServer::setSource(::Sct_SctApi::Trigger::Source s){
  worker()->setSource((::SctApi::Trigger::Source)(int)s);
}

CORBA::Short TriggerServer::getRandom(){
  return worker()->getRandom();
}

void TriggerServer::setRandom(CORBA::Short random){
  worker()->setRandom(random);
}

CORBA::Double TriggerServer::getFrequency(){
  return worker()->getFrequency();
}

void TriggerServer::setFrequency(CORBA::Double freq){
  worker()->setFrequency(freq);
}

CORBA::Boolean TriggerServer::isValidRODTrigger(){
  return worker()->isValidRODTrigger();
}

CORBA::Boolean TriggerServer::isValidTIMTrigger(){
  return worker()->isValidTIMTrigger();
}

CORBA::Boolean TriggerServer::timNeedsSequencer(){
  return worker()->timNeedsSequencer();
}

Sct_SctApi::TIMTriggers * TriggerServer::getTIMTriggers(CORBA::Short point) {
  ::SctApi::Trigger::TIMTriggers ttrigs=worker()->getTIMTriggers(point);
  return Sct::copyVectorToCorba<Sct_SctApi::TIMTriggers,::SctApi::Trigger::TIMTrigger>(ttrigs);
}

Sct_SctApi::RODTriggers * TriggerServer::getRODTriggers(CORBA::Short point) {

  const ::SctApi::Trigger::RODTriggers & rts = worker()->getRODTriggers(point);
  Sct_SctApi::RODTriggers * result = Sct::SequenceUtils::newSafeSequence<Sct_SctApi::RODTriggers>(rts.size());

  for (unsigned int i=0; i<rts.size(); ++i) {
    const ::SctApi::Trigger::RODTrigger & rti = rts[i];

    Sct_SctApi::RODTrigger temp;
    temp.first  = rti.first;
    temp.second = rti.second;

    (*result)[i] = temp;
  };
  
  return result;
}
