#ifndef SCTAPIIPC_TRIGGERSERVER_H
#define SCTAPIIPC_TRIGGERSERVER_H

// CORBA wrapper for SctApi::Trigger class

// fwd dec self
class TriggerServer;

//includes
#include "Sct_SctApi/SctApi.hh"
#include <ipc/object.h>
#include <omniORB4/poa.h> // For PortableServer::RefCountServantBase
#include <boost/shared_ptr.hpp>

// fwd dec others:
namespace SctApi {
  class Trigger;
};

//declaration
class TriggerServer : 
      public IPCNamedObject<POA_Sct_SctApi::Trigger>,
      public virtual PortableServer::RefCountServantBase {
  TriggerServer(const TriggerServer &);
  TriggerServer &operator=(const TriggerServer &);
  static int nextNumber;
 public:
  TriggerServer();
  explicit TriggerServer(const boost::shared_ptr< ::SctApi::Trigger> other);

  //static TriggerServer *getServer(boost::shared_ptr< ::SctApi::Trigger> trigger);
  //static void mapServer(boost::shared_ptr< ::SctApi::Trigger> trigger, TriggerServer *server);

  virtual ~TriggerServer();

  virtual char* print ();
  virtual void singleL1A ();
  virtual void doubleL1A (CORBA::UShort delay);
  virtual void delayedL1A (CORBA::UShort delay);
  virtual void calL1A (CORBA::UShort delay);
  virtual void pulseL1A (CORBA::UShort delay);
  virtual void softL1A (CORBA::UShort delay);
  virtual void softCalL1A (CORBA::UShort delay, CORBA::UShort delay2);
  virtual void softPulseL1A (CORBA::UShort delay, CORBA::UShort delay2);
  virtual void softBc (CORBA::UShort delay);
  virtual void soft();
  virtual void bc();
  virtual void bcL1A (CORBA::UShort delay);
  virtual void bcCalL1A (CORBA::UShort delay, CORBA::UShort delay2);
  virtual void bcPulseL1A (CORBA::UShort delay, CORBA::UShort delay2);
  virtual void setCommIncr (CORBA::UShort command, CORBA::UShort incr);
  virtual void getCommIncr (CORBA::UShort & command, CORBA::UShort & incr);

  virtual void clear ();
  virtual void addCommand (Sct_SctApi::RODTriggerFirstType first, Sct_SctApi::RODTriggerSecondType second);
  
  ::Sct_SctApi::Trigger::Source getSource();
  void setSource(::Sct_SctApi::Trigger::Source s);
  CORBA::Short getRandom();
  void setRandom(CORBA::Short random);
  CORBA::Double getFrequency();
  void setFrequency(CORBA::Double freq);
  
  virtual Sct_SctApi::RODTriggers * getRODTriggers(CORBA::Short scan_point);
  virtual Sct_SctApi::TIMTriggers * getTIMTriggers(CORBA::Short scan_point);

  virtual CORBA::Boolean isValidRODTrigger();
  virtual CORBA::Boolean isValidTIMTrigger();
  virtual CORBA::Boolean timNeedsSequencer();
  virtual void shutdown();

 private:
  boost::shared_ptr< ::SctApi::Trigger> m_worker;
  boost::shared_ptr< ::SctApi::Trigger> worker() { return m_worker; };
 protected:
  //static std::map<boost::shared_ptr< ::SctApi::Trigger>, TriggerServer *> serverMap;

 private:
  static std::string getServerName();

  //SctApi::Trigger &worker();
};

#endif
