#include <ipc/object.h>
#include <ipc/server.h>
#include <ipc/core.h>

#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/lexical_cast.hpp>
#include "SctApiScan_Test.h"
#include "../SctApiServer.h"
#include "../SctApiCrateServer.h"
#include "SctApi/SctApi.h"
#include "sctConf/configRegistry.h"
#include <boost/test/unit_test.hpp>
#include <iostream>
#include "Sct/DestroyingDeleter.h"
#include "SctApi/SctApiException.h"
#include "SctApi/Idiosyncrasy.h"
#include "Sct/SctNames.h"

//? Obsolete? std::auto_ptr<IPCServer> ipcServer;

using boost::unit_test_framework::test_suite;
using namespace boost::posix_time;
using namespace boost::gregorian;

const Sct::UCID anArbitraryUcid(0,0);

const SctApi::Idiosyncrasy id(anArbitraryUcid);

namespace SctApi {
  namespace Tests {
    SctConfiguration::Factory *confFact = 0;
    boost::shared_ptr<SctConfiguration::Configuration> globalConfig;

    void testGetScan() {
      try {
	IPCPartition p(::Sct::SctNames::getPartitionName());
	//? Obsolete? ipcServer.reset(new IPCServer(Sct_SctApi_C_SctApi_serverName, p));


        std::auto_ptr<SctApi::SctApi> ob(new SctApi::SctApi(id, boost::shared_ptr<SctConfiguration::Configuration>(globalConfig)));
	
	boost::shared_ptr<SctApiCrateServer> capi = ::Sct::DestroyingDeleter<SctApiCrateServer>::shared_ptr(new SctApiCrateServer(*ob));
	SctApiServer::CrateDescriptors crateDescriptors; crateDescriptors.push_back(anArbitraryUcid);
	boost::shared_ptr<SctApiServer> api = ::Sct::DestroyingDeleter<SctApiServer>::shared_ptr(new SctApiServer(crateDescriptors, Sct_SctApi::SctApiIPC::instanceName));
	
	Sct_SctApi::Scan_var newScan = api->createScan();
	
	BOOST_CHECK(!CORBA::is_nil(newScan));
      } catch (...) {
	const bool exception=true;
	BOOST_CHECK(!exception);
      }; 
    }

    void testGetScanTrigger() {
      try {
	IPCPartition p(::Sct::SctNames::getPartitionName());
      // Obsolete: ipcServer.reset(new IPCServer(Sct_SctApi_C_SctApi_serverName, p));

	std::auto_ptr<SctApi::SctApi> ob(new SctApi::SctApi(id, boost::shared_ptr<SctConfiguration::Configuration>(globalConfig/*confFact->defaultConfig()*/)));

	boost::shared_ptr<SctApiCrateServer> capi = ::Sct::DestroyingDeleter<SctApiCrateServer>::shared_ptr(new SctApiCrateServer(*ob));
	SctApiServer::CrateDescriptors crateDescriptors; crateDescriptors.push_back(anArbitraryUcid);
	boost::shared_ptr<SctApiServer> api = ::Sct::DestroyingDeleter<SctApiServer>::shared_ptr(new SctApiServer(crateDescriptors, Sct_SctApi::SctApiIPC::instanceName));
	
	try {
	  Sct_SctApi::Scan_var newScan = api->createScan();
	  BOOST_CHECK(!CORBA::is_nil(newScan)); 
	  
	  try {
	    Sct_SctApi::Trigger_var newTrigger = newScan->getTrigger1();
	    BOOST_CHECK(!CORBA::is_nil(newTrigger)); 
	    
	    try {
	      newTrigger->singleL1A();
	    } catch(...) {
	      throw SctApiException("Bad return code from singleL1A");
	    }

	    try {
	      std::cout << newTrigger->print();
	    } catch(...) {
	      throw SctApiException("Bad return code from print trigger");
	    }
	    
	  } catch(...) {
	    throw SctApiException("Bad return code from getTrigger1 ");
	  }
	  try{
	    std::cout << newScan->print();
	  } catch (...) {
	    throw SctApiException("Bad return code from printScan ");
	  }
	  
	} catch (...) {
	  throw SctApiException("Bad return code from createScan ");
	}
      } catch (...) {
	const bool exception=true;
	BOOST_CHECK(!exception);
      }
    };

    void testGetDefaultScan() {
      IPCPartition p(::Sct::SctNames::getPartitionName());
      // Obsolete: ipcServer.reset(new IPCServer(Sct_SctApi_C_SctApi_serverName, p));
     
      std::auto_ptr<SctApi::SctApi> ob(new SctApi::SctApi(id, boost::shared_ptr<SctConfiguration::Configuration>(globalConfig/*confFact->defaultConfig()*/)));
      
      boost::shared_ptr<SctApiCrateServer> capi = ::Sct::DestroyingDeleter<SctApiCrateServer>::shared_ptr(new SctApiCrateServer(*ob));
      SctApiServer::CrateDescriptors crateDescriptors; crateDescriptors.push_back(anArbitraryUcid);
      boost::shared_ptr<SctApiServer> api = ::Sct::DestroyingDeleter<SctApiServer>::shared_ptr(new SctApiServer(crateDescriptors, Sct_SctApi::SctApiIPC::instanceName));
      
      Sct_SctApi::Scan_var newScan = api->getDefaultScan();
      
      BOOST_CHECK(!CORBA::is_nil(newScan)); 
    }

    void testModifyDefaultScan() {
      IPCPartition p(::Sct::SctNames::getPartitionName());
      // Obsolete: ipcServer.reset(new IPCServer(Sct_SctApi_C_SctApi_serverName, p));

      std::auto_ptr<SctApi::SctApi> ob(new SctApi::SctApi(id, boost::shared_ptr<SctConfiguration::Configuration>(globalConfig/*confFact->defaultConfig()*/)));

      boost::shared_ptr<SctApiCrateServer> capi = ::Sct::DestroyingDeleter<SctApiCrateServer>::shared_ptr(new SctApiCrateServer(*ob));
      SctApiServer::CrateDescriptors crateDescriptors; crateDescriptors.push_back(anArbitraryUcid);
      boost::shared_ptr<SctApiServer> api = ::Sct::DestroyingDeleter<SctApiServer>::shared_ptr(new SctApiServer(crateDescriptors, Sct_SctApi::SctApiIPC::instanceName));

      Sct_SctApi::Scan_var firstDefaultScan = api->getDefaultScan();
      BOOST_CHECK(!CORBA::is_nil(firstDefaultScan)); 
      firstDefaultScan->setOption(Sct_SctApi::Scan::debug, 15);

      Sct_SctApi::Scan_var secondDefaultScan = api->getDefaultScan();
      BOOST_CHECK(!CORBA::is_nil(secondDefaultScan)); 

      int defValue1 = firstDefaultScan->getOption(Sct_SctApi::Scan::debug);
      int defValue2 = secondDefaultScan->getOption(Sct_SctApi::Scan::debug);

      BOOST_CHECK(defValue1 == 15); 
      BOOST_CHECK(defValue2 == 15); 

      // free the api before the capi as we don't want to leave it holding dangling CORBA refs which it might try to use
      api = boost::shared_ptr<SctApiServer>();
      capi = boost::shared_ptr<SctApiCrateServer>();

    }

    void testCreateIsDefault() {
      IPCPartition p(::Sct::SctNames::getPartitionName());
      // Obsolete: ipcServer.reset(new IPCServer(Sct_SctApi::SctApi::serverName, p));

      std::auto_ptr<SctApi::SctApi> ob(new SctApi::SctApi(id, boost::shared_ptr<SctConfiguration::Configuration>(globalConfig/*confFact->defaultConfig()*/)));

      boost::shared_ptr<SctApiCrateServer> capi = ::Sct::DestroyingDeleter<SctApiCrateServer>::shared_ptr(new SctApiCrateServer(*ob));
      SctApiServer::CrateDescriptors crateDescriptors; crateDescriptors.push_back(anArbitraryUcid);
      boost::shared_ptr<SctApiServer> api = ::Sct::DestroyingDeleter<SctApiServer>::shared_ptr(new SctApiServer(crateDescriptors, Sct_SctApi::SctApiIPC::instanceName));

      Sct_SctApi::Scan_var defScan = api->getDefaultScan();

      BOOST_CHECK(!CORBA::is_nil(defScan)); 

      defScan->setOption(Sct_SctApi::Scan::debug, 15);

      Sct_SctApi::Scan_var scan = api->createScan();

      BOOST_CHECK(!CORBA::is_nil(scan)); 

      int defValue = scan->getOption(Sct_SctApi::Scan::debug);

      BOOST_CHECK(defValue == 15); 
    }

    SctApiScan_Test::SctApiScan_Test() {
      int narg=0;
      char ** args=0;

      std::string backend = "xml";
      SctConfiguration::FactoryManager::instance().loadPlugin(backend); 
      confFact = &SctConfiguration::FactoryManager::instance().getFactory(backend);

      globalConfig.reset(confFact->defaultConfig());
      IPCCore::init(narg,args);
	add(BOOST_TEST_CASE(&testGetScan));
	add(BOOST_TEST_CASE(&testGetScanTrigger));
        add(BOOST_TEST_CASE(&testGetDefaultScan));
        add(BOOST_TEST_CASE(&testModifyDefaultScan));
        //        add(BOOST_TEST_CASE(&testCreateIsDefault));
    }	
}
}
