#include "configserver.h"
#include "sctConfIPC/configipc.h"
#include "RunController.h"
#include <pmg/pmg_initSync.h>
#include "ipc/core.h"
#include "mrs/message.h"
#include "rc/Controller.h"
#include "Sct/SctNames.h"
#include "Sct/ApplicationStartupDebugTools.h"
#include "Sct/MultiMessageDebugStream.h"
#include <exception>

#include <signal.h>
#include "cmdl/cmdargs.h"
#include "dal/implementation.h"
#include "dal/util.h"
#include "config/Configuration.h"
#include "sctConfIPC/configipc.h"

using namespace std;

//using namespace SctConfiguration;

//// BEGINNING OF ZEROETH SEGMENT COPIED FROM /home/sctroddq/sctsw/tdaq-01-02-00/installed/../rc/src/examples/empty_controller.cc

inline std::string str(const char * word)
{
  return word ? word : "";
} 

extern "C" void signalHandler(int)
{
  CS::ClipsServer * that = CS::ClipsServer::instance();
  if ( that ) that->stop();
}
//// END OF ZEROETH SEGMENT COPIED FROM /home/sctroddq/sctsw/tdaq-01-02-00/installed/../rc/src/examples/empty_controller.cc

int main(int argc, char** argv) 
{
  IPCCore::init(argc,argv);
  Sct::ApplicationStartupDebugTools::announceStartOfMain(argc, argv, __FILE__, __LINE__);

  // USAGE COMMENT: 
  // In tdaq-01-02-00 this main is usually called as follows:
  // AppStartupDbgTools: [ApplicationStartupDebugTools::announceStartOfMain called by [/home/sctroddq/sctsw/SctRodDaq/installed/i686-slc3-gcc323-opt/bin/config_server -n config_server --oks -P CalibrationController -s config_serverSegment -n config_server] in File[configserverMain.cc] at Line[39] with additional Message[].]
  // and the parameters after the --oks (which is probably a throwback to old times) are "invented" by the DSA supervisor.

    //// BEGINNING OF FIRST SEGMENT COPIED FROM /home/sctroddq/sctsw/tdaq-01-02-00/installed/../rc/src/examples/empty_controller.cc
   CmdArgStr    partition  ('p',"partition","partition",  "Name of the IPC partition (default $TDAQ_PARTITION)");
   CmdArgStr    parentName ('P',"parentname", "parentname", "Name of the parent");
   CmdArgStr    segName    ('s',"segmentname", "segmentname", "Name of the segment");
   CmdArgStr    name       ('n',"name",     "controller", "Name of the controller",CmdArg::isREQ);
   CmdArgStr    data       ('d',"data",     "db-name",    "Configuration DB name");
   CmdArgStr    rules      ('R',"rules",    "rules",      "CLIPS rulesbase (default Controller:TransitionTable)");
   CmdArgStr    server     ('z',"server",   "is-server",  "Name of the IS server (default RunCtrl)");
   CmdArgInt    verbosity  ('v',"verbosity","level",      "Output level (default 0 )");
   CmdArgBool   oks        ('O',"oks"                     ,"Load Configuration from OKS");
   CmdArgBool   rdb        ('Q',"rdb"                     ,"Load Configuration from RDB");
   CmdArgBool   none       ('N',"none"                    ,"Do not load Configuration");

   CmdArgStr    backend    ('B',"backend", "backend", "Backend plugin to use for SCT configuration (default xml)");


   partition=std::getenv("TDAQ_PARTITION");
   server="RunCtrl";
   verbosity=0;

   backend="xml";

   CmdLine  cmd(*argv,&partition,&parentName,&segName,&name,&data,&rules,&server,&verbosity,
		&oks,&rdb,&none,&backend,NULL);
   cmd.description("An empty run controller that has just empty user routines.");
   
   CmdArgvIter  argvIter(--argc,++argv);
   if (cmd.parse(argvIter)) 
   {
      std::cerr << "ERROR [rc_empty_controller] Error parsing command line." << std::endl ;
      return EXIT_FAILURE;
   }

   signal(SIGINT,  signalHandler);
   signal(SIGTERM, signalHandler);

   IPCPartition p(partition);

   std::auto_ptr<ConfigurationImpl> configImpl
   (
      none ? 0 : daq::core::create_config_implementation
         (   
            oks ? daq::core::OksDB_Implementation :
            rdb ? daq::core::RdbDB_Implementation :
                  daq::core::DefaultDB_Implementation
         )
   );
    
   std::auto_ptr<Configuration> config(none ? 0 : new Configuration(str(data), configImpl.get()));
   if (config.get()) 
   {
     const daq::core::Partition * part = daq::core::get_partition(*(config.get()), str(partition));
     if(part)
       config->register_converter(new daq::core::SubstituteVariables(*(config.get()), *part));
   } 
      
    //// END OF FIRST SEGMENT COPIED FROM /home/sctroddq/sctsw/tdaq-01-02-00/installed/../rc/src/examples/empty_controller.cc


  MRSStream mrsStream(::Sct::SctNames::getPartitionName());
  
  ConfigServer *config_server;
  try {
    config_server = new ConfigServer(p, ::sctConf::Configuration::instanceName, str(backend));
  } catch(SctConfiguration::ConfigurationException &c) { // non-corba type can be thrown at start-up
    mrsStream << "CONF_FAILED " << MRS_FATAL << MRS_QUALIF("sctConf") 
              << MRS_TEXT(string("Failed to create Configuration Service") + c.what()) << ENDM;
    return 1;
  }
  
  mrsStream << "CONF_START " << MRS_DIAGNOSTIC << MRS_QUALIF("sctConf") 
	    << MRS_TEXT("Config Server created") << ENDM;
  
  if (!config_server->publish()) {
    mrsStream << "CONF_FAILED " << MRS_FATAL << MRS_QUALIF("sctConf") 
              << MRS_TEXT("Failed to publish Configuration Service") << ENDM;

    cout << "Failed to publish Configuration" << endl;
    return 2;
  } 
  
  SctConfiguration::ConfigIPC wrapper(config_server->_this());

  SctConfigurationController::RunControl& user = 
    SctConfigurationController::RunControl::initialize(wrapper);

    //// BEGINNING OF SECOND SEGMENT COPIED FROM /home/sctroddq/sctsw/tdaq-01-02-00/installed/../rc/src/examples/empty_controller.cc
    RC::Controller   controller(p,user,str(name),str(parentName),str(segName),str(rules),config.get(),str(server),
				verbosity);
    //// END OF SECOND SEGMENT COPIED FROM /home/sctroddq/sctsw/tdaq-01-02-00/installed/../rc/src/examples/empty_controller.cc

  /*
  RC::Controller run_controller(Sct::SctNames::getPartition(),user, "config_server");
  */
  RC::Controller & run_controller = controller;
  cout << "Configuration server published, awaiting commands\n";

  // now that we have fulfilled our contract to the world by having made a configuration server public, we inform the world that we are ready for others that depend on us to start:
  pmg_initSync();

  try {
    run_controller.run();
  } catch(SctConfiguration::ConfigurationException & c) {  // catch non-corba exceptions
    mrsStream << "CONF_FAILED " << MRS_FATAL << MRS_QUALIF("sctConf") 
              << MRS_TEXT((string("Failed to run Configuration Service") + c.what()).c_str()) << ENDM;
    Sct::MultiMessageDebugStream m(true,true,true);
    m << "config_server about to die for SctConfiguration::ConfigurationException [" << c.what() << "] detected in " << __FILE__ << " at " << __LINE__;
    return 3;
  } catch (std::exception & e) {
    Sct::MultiMessageDebugStream m(true,true,true);
    m << "config_server about to die because of std::exception ["<<e.what()<<"] detected in " << __FILE__ << " at " << __LINE__;
    return 4;
  } catch (CORBA::Exception & e) {
    Sct::MultiMessageDebugStream m(true,true,true);
    m << "config_server about to die because of CORBA_Exception ["<<e._name()<<"] detected in " << __FILE__ << " at " << __LINE__;
    return 4;
  } catch (...) {
    Sct::MultiMessageDebugStream m(true,true,true);
    m << "config_server about to die for unknown reason detected in " << __FILE__ << " at " << __LINE__;
    return 5;
  }

  

  {
    Sct::MultiMessageDebugStream m(true,true,true);
    m << "config_server stopped very peacefully at " << __FILE__ << " at " << __LINE__ << " which suggests that it was asked very nicely to stop by someone intent on stopping it!";
  };

  cout << "Run over" << endl;

  config_server->withdraw();
  config_server->_destroy();
}
