#include "Displayer.h"
#include "SctData/DefectList.h"
#include "DisplayManager.h"
#include <cmath>
#include <TCanvas.h>
#include <TROOT.h>
#include <TStyle.h>
#include <iostream>
#include <iomanip>
#include <boost/lexical_cast.hpp>

using namespace std;
using namespace boost;
using namespace SctData;

namespace SctDataDisplay {

struct null_deleter
{
    void operator()(void const *) const
    {
    }
};

shared_ptr<TCanvas> Displayer::createCanvas(string name, string title) {
    unsigned int i=0;
    string rname;
    TCanvas* old = 0;
    do {
        ++i;
        rname = name + "_" + lexical_cast<string>(i);
	old = dynamic_cast<TCanvas*>(gROOT->GetListOfCanvases()->FindObject(rname.c_str()));
    } while(old != 0);
    int offset=gROOT->GetListOfCanvases()->GetSize()*20%200;
    gROOT->SetStyle("Plain");
    gStyle->SetOptStat(0);
    gStyle->SetTitleX(0.5);
    gStyle->SetTitleY(0.99);
    gStyle->SetTitleFontSize(0.07);
   
    shared_ptr<TCanvas> canvas = shared_ptr<TCanvas>(new TCanvas(rname.c_str(), title.c_str(), 
								 offset, offset, 900, 650));
    if ( DisplayManager::batchMode() ) canvas->SetBatch();
    return canvas;
}

void Displayer::divideCanvas(unsigned int n, TCanvas& canvas) {
    unsigned int x = (unsigned int)(ceil(sqrt((double)n)));
    unsigned int y = (unsigned int)(ceil((double)n/x));
    canvas.Divide(x, y, 0.0001, 0.0001);
}

void Displayer::displayHistogram(shared_ptr<TH1> hist, std::string option){
  if (DisplayManager::rootMode()){
    hist.reset(hist.get(),null_deleter());
  }
  displayHistogram(*hist, option);
}

void Displayer::displayHistogram(TH1& hist, std::string option){
  if (DisplayManager::rootMode()){
    shared_ptr<TDirectory> file=DisplayManager::getTDirectory();
    if (file) { 
      hist.SetDirectory(file.get());
      file->cd();
      // Make sure name starts with non-numberic character for root!
      std::string name ="h_";
      name += hist.GetName();
      hist.Write(name.c_str());
    }
  }else{
    int nlevels=100;
    double levels[nlevels];
    double h_max = hist.GetMaximum();
    if (h_max>0.5) h_max=1.0;
    for (unsigned i=0; i<nlevels; ++i){
      levels[i]=h_max*(double)i/(double)nlevels;
    }
    hist.SetContour(nlevels, levels);
    hist.Draw(option.c_str());
  }
}

void Displayer::printDefectList(const DefectList& list, std::ostream& os) {
  const DefectList::DefectCollection& defects = list.getAllDefects();
    
  for (DefectList::DefectCollection::const_iterator i=defects.begin(); i!=defects.end(); ++i) {
    os << setfill(' ') << setw(20) << i->getPrototype().getName()
       <<  "\tStart channel: " << i->getModuleElement().getFirst()
       << "\t# of channels: " << i->getModuleElement().getNChannels()
       << "\t" << i->getPrototype().getDescription() << std::endl;
  }
  os << "[ total number of defects = " <<  defects.size() << " ]" << std::endl;
  os << std::endl;
}

}
