#ifndef cdiAux_H
#define cdiAux_H

//////////////////////////////////////////////////////////////////////////////////
//
//   cdiAux.h        Luis Pedro, November 2003, Serguei Kolos May 2004
//
//
//   cdiAux class:Provides a set of generic utilities for the
//		     CDI component. It also provides a facility to allow
//		     several degrees of verbosity to STDOUT and STDERR.
//
//   info: printout a CDI message if the verbosity level of the message is
//	       equal or inferior to the level of verbosity set by the application.
//
//   error: printout a formatted error message, also taking into consideration
//	       the verbosity level.
//
//////////////////////////////////////////////////////////////////////////////////


#include <string>
#include <owl/time.h>

class cdiAux 
{
  int verbosity_;
 public:
  
  cdiAux( int verbosity = 1 ) : verbosity_(verbosity) { ; }
  
  void info ( const std::string & msg, int verbosity = 1 ) const {
    if (verbosity <= verbosity_)
      std::cout << "[CDI INFO    (" << OWLTime() <<  ") ] " << msg << std::endl;
  }
  
  void warning ( const std::string & msg, int verbosity = 1 ) const {
    if (verbosity <= verbosity_)
      std::cout << "[CDI WARNING (" << OWLTime() <<  ") ] " << msg << std::endl;
  }

  void error ( const std::string & msg, int verbosity = 0  ) const {
    if (verbosity <= verbosity_)
      std::cout << "[CDI ERROR   (" << OWLTime() <<  ") ] " << msg << std::endl;
  }
  
  void debug (const std::string & msg, int verbosity = 2 ) const {
    if (verbosity <= verbosity_)
      std::cout << "[CDI DEBUG   (" << OWLTime() <<  ") ] " << msg << std::endl;
  }

  void verbose (const std::string & msg, int verbosity = 3 ) const {
    if (verbosity <= verbosity_)
      std::cout << "[CDI VERBOSE (" << OWLTime() <<  ") ] " << msg << std::endl;
  }

  void verbose (const char*msg, int verbosity = 3 ) const {
    this->verbose(std::string(msg),verbosity);
  }
  void debug (const char*msg, int verbosity = 2 ) const {
    this->debug(std::string(msg),verbosity);
  }
  void error (const char*msg, int verbosity = 0 ) const {
    this->error(std::string(msg),verbosity);
  }
  void info (const char*msg, int verbosity = 1 ) const {
    this->info(std::string(msg),verbosity);
  }

  void warning (const char*msg, int verbosity = 1 ) const {
    this->warning(std::string(msg),verbosity);
  }

};			  

#endif //cdiAux_H
