#ifndef CDICOOLDATAHANDLE_H
#define CDICOOLDATAHANDLE_H
///////////////////////////////////////////////////////////////////////
///                                                                 ///
///                                                                 ///
///               Nuno Barros ATLAS/DAQ 2005                        ///
///     Interface class to access the LCG COOL methods              ///
///                                                                 ///
///////////////////////////////////////////////////////////////////////



namespace cool {
  class IDatabase;
};

namespace pool {
  class AttributeListSpecification;
};

namespace seal {
  class Exception;
};

#include "CoolKernel/IDatabaseSvc.h"
#include <CoolKernel/DatabaseId.h>
#include <CoolKernel/types.h>

#include "CoolKernel/IFolder.h"

#include "cdi/cdiAux.h"
#include "cdi/cdiHandle.h"

#include <is/info.h>
#include <vector>
#include <map>
#include <is/infodocument.h>
#include <is/infodictionary.h>
#include <is/inforeceiver.h>

class cdiCOOLDataHandle : public cdiHandle
{  
 public:
  
  cdiCOOLDataHandle ( int verbosity, const IPCPartition & partition,  const std::string confName = "RunParams.Conditions" );
  
  void getConfig ( std::vector<std::string> & names );  
  void process_data ( const std::string & name, ISCallbackInfo * isc );

  void storeObject ( const std::string& name, ISInfoAny &isa);

  /*
  std::string getConfName() const {
    if (configuration_name.size() != 0)
      return configuration_name;
    else
      return "";
  }
  */
 private:


  void loadSEALModules();
  
  std::string dumpFailure(  ISInfo::Status status );

  void get_folder_specs(std::string folder, pool::AttributeListSpecification &payloadSpec);
  bool existDB(std::string dbProfile);
  std::string getProfile ( );
  void init ( const std::string & profile );
  void create_folder ( const std::string & name, ISInfoAny & isa );
  void add_table ( pool::AttributeListSpecification &table, const std::string & name );
  bool is_table_exist ( const std::string & name );
  void convertTime (ISInfoAny & isAny, seal::IntBits<64>::ULeast &time);//unsigned long &time);
  template <class T>
    void convert( ISInfoAny & isa, std::vector<std::string> & to );

  void fillTypes( ); 

  // private members
 private:
  
  cdiAux	mout;
  IPCPartition	partition;
  
  std::string m_fName;

  // the connection string....not needed any more
  std::string m_ConnStr;

  // the services that are mostly used in COOL
  cool::IDatabaseSvc* m_dbSvc; 

  // this is the identifier that will hold the connection string
  cool::DatabaseId m_dbId;
  
  // This is the Database Pointer.Every operation in a database shall use this variable
  cool::IDatabasePtr m_db;  
  
  // This is the folder where data will be stored
  cool::IFolderPtr m_fld;

  // we will store the data type name as of the 
  // attribute list specification
  std::map <ISType::Basic, std::string>	m_dTypes;
  
  // variable that will hold an attribute list for each folder
  //  std::map <std::string,pool::AttributeList> m_Tables;
  // instead of holding the AttributeList it holds simply the schema of the table
  // (aka AttributeListSpecification)
  std::map <std::string,pool::AttributeListSpecification> m_Tables;
  std::vector<std::string> m_objs;
 public:
  const std::string configuration_name;


};
#endif //CDIDATAHANDLE_H

