#ifndef SCTCONF_CONFUTILITIES_H
#define SCTCONF_CONFUTILITIES_H

#include "ConfigurationException.h"

namespace SctConfiguration {

/** different geometries of endcap module */
enum EndcapType {ENDCAP_OUTER, ENDCAP_MIDDLE, ENDCAP_SHORT_MIDDLE, ENDCAP_INNER, ENDCAP_UNKNOWN};
  
/** 
    get the endcap geometry type for a particular number on a disk, 
    as given in configuration.h 
    @throw ConfigurationException if the arguments dont make sense
*/

/* Original form replaced by Liverpool by new variant below.
EndcapType getEndcapType(int disk, unsigned number){
  char message[1000];
  sprintf(message,"Disk out of range:%d", disk);
  if (abs(disk)>9) throw ConfigurationException(message);

  if (number<13) {
    return ENDCAP_OUTER;
  } else if (number<23) {
    switch (abs(disk)) {
    case 9: {
      char message[1000];
      sprintf(message,"getEndcapType: out of range: disk %d has no middle modules (number=%d)", disk, number);
      throw ConfigurationException(message);
    }
    case 8: return ENDCAP_SHORT_MIDDLE;
    default: return ENDCAP_MIDDLE;
    }
  } else if (number<33) {
    switch (abs(disk)){
    case 9: case 8: case 7: case 1: {
      char message[1000];
      sprintf(message, "getEndcapType: out of range: disk %d has no inner modules (number=%d)", disk, number);
      throw ConfigurationException(message);
    }
    default: return ENDCAP_INNER;
    }
  } else {
    char message[1000];
    sprintf(message, "getEndcapType: out of range: number should never take value=%d (disk=%d)", number, disk);
    throw ConfigurationException(message);
  }
} */

  /// New Liverpool implementation 

  /// A function which never returns and always throws, whilst complaining.
  EndcapType sillyEndcapDiskAndNumber(const int disk, const unsigned number) {
    std::ostringstream os;
    os <<"Disk "<<disk<<" and number "<<number
       <<" are just plain silly.  Error detected in "
       <<__FILE__<<" at line " << __LINE__;
    throw ConfigurationException(os.str());
    
    // what we would return if we ever got here!  At the moment the policy is to throw instead of return MODULE_UNKNOWN, though!
    return ENDCAP_UNKNOWN;                  
  };


  EndcapType getEndcapType(const int disk, const unsigned number) {
    
    if(number < 0) return sillyEndcapDiskAndNumber(disk, number);  
    
    if(number < 13)  return ENDCAP_OUTER;
    
    if(number < 23) {
      if( disk == 8 || disk == -8) return ENDCAP_SHORT_MIDDLE;
      return ENDCAP_MIDDLE;
    }
    
    if(number < 33) return ENDCAP_INNER;
    
    return sillyEndcapDiskAndNumber(disk, number);                  
    
  } 
  
}

#endif
