#include "commonUtils.h"

#include <ostream>

namespace {
  const char *DCSChannelParams[] = {
    "LVchStat", "LVch_Vcc", "LVps_Vcc", "LVretVcc", "LVch_Icc", "LVch_Vdd", "LVps_Vdd", "LVretVdd", "LVch_Idd", "LVchVCSV", 
    "LVchVCSI", "LVchPINV", "LVchPINI", "MOch_Tm0", "MOch_Tm1", "LVchDTrp", "LVchCTrp", "LVchTLim", "LVchTTrp", "LVchCLKS", 
    "LVchCLKR", "LVchVOut", "HVchStat", "HVchVolt", "HVchCurr", "HVchITrp", "HVchRamp", 0};

  const char *DCSCardParams[] = {"LVcaStat", "LVcaTemp", "LVcaPowr", "LVcaMask", "LVcaTrip", 
				 "HVcaStat", "HVcaMask", 0};


  const char *DCSCrateParams[] = {0};

}

namespace SCTConfigCommonUtils {

void printModuleConfig(const ABCDModule &conf, std::ostream &output) {
  output << "Module global settings\n";
  output << " Present " << (int)conf.present << " Active " << (int)conf.active << " Select " << (int)conf.select << " Group " << (int)conf.groupId << std::endl;
  output << " TX fibre " << (int)conf.pTTC << " rx fibre0 " << (int)conf.rx[0] << " rx fibre1 " << (int)conf.rx[1] << std::endl;
  output << " Serial Number " << conf.serialNumber 
	    << " Mid 0x" << std::hex << conf.mid << std::dec << std::endl;

//   output << " BOC configuration " << conf.bocConfig << std::endl;

  for(int c=0; c<N_SCT_CHIPS; c++) {
    output << "Chip: " << c << (conf.chip[c].active?" active ":" inactive ") ; 
    if(conf.chip[c].active) {
      output << "Adr="<< std::hex << "0x" << (int)conf.chip[c].address << std::dec << std::endl;
      output << "Config 0x" << std::hex << *(UINT16*)(&conf.chip[c].basic.config) << std::dec << std::endl;

      output << "R" << conf.chip[c].basic.config.readoutMode << " ";
      output << "C" << conf.chip[c].basic.config.calibMode << " ";
      output << "T" << conf.chip[c].basic.config.trimRange << " ";
      output << "D" << conf.chip[c].basic.config.edgeDetect << " ";
      output << "K" << conf.chip[c].basic.config.mask << " ";
      output << "A" << conf.chip[c].basic.config.accumulate << " ";
      output << "I" << conf.chip[c].basic.config.inputBypass << " ";
      output << "O" << conf.chip[c].basic.config.outputBypass << " ";
      output << "M" << conf.chip[c].basic.config.master << " ";
      output << "E" << conf.chip[c].basic.config.end << " ";
      output << "F" << conf.chip[c].basic.config.feedThrough << " ";

      output << std::endl;

      output << "Mask " << std::hex
	     << " 0x" << conf.chip[c].basic.mask[3]
	     << " 0x" << conf.chip[c].basic.mask[2]
	     << " 0x" << conf.chip[c].basic.mask[1]
	     << " 0x" << conf.chip[c].basic.mask[0] << std::dec << std::endl;

      output << "Rc function type " << (int)conf.chip[c].caldata.rc_function << " :";
      for(int r=0; r<3; r++) {
        output << " " << conf.chip[c].caldata.rc_params[r];
      }
      output << std::endl;
      output << "Regs ";
      output << " vthr " << (int)conf.chip[c].basic.vthr;
      output << " vcal " << (int)conf.chip[c].basic.vcal;
      output << " delay " << (int)conf.chip[c].basic.delay;
      output << " preamp " << (int)conf.chip[c].basic.preamp;
      output << " shaper " << (int)conf.chip[c].basic.shaper;
      output << std::endl;

      output << "Trim target = " << (int)conf.chip[c].target;
      for(int i=0; i<128; i++) {
        if(!(i%32)) output << std::endl;
        output << std::hex << (int)conf.chip[c].trim[i] << std::dec;
      }
      output << std::endl;
    } else {
      output << std::endl;
    }
  }
  output << std::endl;
}

std::list<std::string> listDCSChannelParams() {
  std::list<std::string> result;
  for(int i=0; DCSChannelParams[i] != 0; i++) {
    result.push_back(DCSChannelParams[i]);
  }

  return result;
}

std::list<std::string> listDCSCardParams() {
  std::list<std::string> result;
  for(int i=0; DCSCardParams[i] != 0; i++) {
    result.push_back(DCSCardParams[i]);
  }

  return result;
}

std::list<std::string> listDCSCrateParams() {
  std::list<std::string> result;
  for(int i=0; DCSCrateParams[i] != 0; i++) {
    result.push_back(DCSCrateParams[i]);
  }

  return result;
}

}
