#include <iostream>
#include <dlfcn.h>

#include "configRegistry.h"

namespace SctConfiguration {
  std::map<std::string, Factory *> configurationRegistry;

  Factory &FactoryManager::getFactory(std::string backend) {
    listFactories();
    if (configurationRegistry.find(backend) == configurationRegistry.end()) {
      std::cout << "Can't find backend '" << backend << "' in registry\n";
      throw ConfigurationException("Can't find backend " + backend);
    }
    return *configurationRegistry[backend];
  }

  void FactoryManager::listFactories() {
    std::cout << "Configuration factories:\n";
    for(std::map<std::string, Factory *>::const_iterator mi = configurationRegistry.begin();
	mi != configurationRegistry.end(); mi++) {
      std::cout << "  " << mi->first << std::endl;
    }
  }

  bool FactoryManager::addFactory(std::string backend, Factory &factory) {
    std::cout << "Configuration backend: '" << backend << "' registered\n";
    configurationRegistry[backend] = &factory;
    return true;
  }

  bool FactoryManager::loadPlugin(std::string name) {
    std::string soName = "libsctConf" + name + ".so";
    std::cout << "Attempting load of plugin at " << soName << std::endl;

    void *handle = dlopen(soName.c_str(), RTLD_NOW);

    listFactories();

    if(handle) {
      std::cout << "Load successful\n";
      return true;
    } else {
      std::cout << "Load failed: " << dlerror() << std::endl;
      return false;
    }
  }

  FactoryManager &FactoryManager::instance() {
    if(!singleton) singleton = new FactoryManager();
    return *singleton;
  }

  FactoryManager *FactoryManager::singleton = 0;
}

