#ifndef SCT_CONFIGURAITON_REGISTRY_H
#define SCT_CONFIGURAITON_REGISTRY_H

#include <map>
#include <string>

#include "configuration.h"

namespace SctConfiguration {
  /// Backend specific configuration factory
  class Factory {
  public:
    /// Construct default configuration for this backend
    virtual Configuration *defaultConfig() = 0;
    /// Construct configuration from file
    virtual Configuration *fromFile(std::string fileName) = 0;
  };

  class FactoryManager {
  public:
    /// Construct default configuration for this backend
    Factory &getFactory(std::string backend);
    /// Return completion
    bool addFactory(std::string backend, Factory &factory);
    /// Get singleton
    static FactoryManager& instance();

    void listFactories();

    /// Load plugin
    bool loadPlugin(std::string name);
  private:
    std::map<std::string, Factory *> configurationRegistry;
    FactoryManager() throw() {} ///< singleton
    static FactoryManager* singleton; ///< pointer to the singleton.
  };

}

#endif
