// For stat
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <boost/lexical_cast.hpp>

#include "dalImpl.h"
#include "dalImplUtils.h"

#include "Barrel.h"
#include "Channel.h"
#include "Crate.h"
#include "Geography.h"
#include "GeographyMUR.h"
#include "Partition.h"
#include "Power.h"
#include "PowerChannel.h"
#include "PowerCrate.h"
#include "PowerParam.h"
#include "Rod.h"
#include "RodMUR.h"
#include "RodModule.h"
#include "RodRModule.h"
#include "Row.h"
#include "SCT_Chip.h"
#include "SCT_Module.h"
#include "Slave.h"
#include "Tim.h"

void ConfigurationDALImpl::setModuleGroup(const std::string module, short group) {

#warning "undefined"
  std::cerr << "Undefined at " << __LINE__ << std::endl;
}

void ConfigurationDALImpl::setModuleErrorMask(unsigned int partition, unsigned int crate, unsigned int rod,
					      unsigned int channel, unsigned int link, unsigned int mask) {
  unsigned int order = channel / 6;
  unsigned int id = (channel % 6) + 1;

  const SCTDAL::Rod *rodObj = lookup->getRod(partition, crate, rod);
  if(!rodObj) {
    throw SctConfiguration::ConfigurationException("No ROD to map fibres to");
  }
  SCTDAL::RodMUR *murObj = const_cast<SCTDAL::RodMUR*>(lookup->getMurInRod(rodObj, order));
  if(!murObj) {
    throw SctConfiguration::ConfigurationException("No MUR to map fibres to");
  }
  SCTDAL::Channel *channelObj = builder->getOrCreateChannelInRodMur(murObj, id);
  if(!channelObj) {
    throw SctConfiguration::ConfigurationException("Failed to create channel in MUR");
  }

  if(link == 0) {
    channelObj->set_stream0ErrMask(mask);
  } else {
    channelObj->set_stream1ErrMask(mask);
  }
}

void ConfigurationDALImpl::setDefaultModuleErrorMask(unsigned int mask) {
#warning "undefined possibly remove"
  //  std::cerr << "Undefined at " << __LINE__ << std::endl;
}

void ConfigurationDALImpl::configureModuleFromStructure(const std::string module, const ABCDModule moduleConf) {
  SCTDAL::SCT_Module *moduleObject = const_cast<SCTDAL::SCT_Module *> (lookup->getModule(module));

  if(!moduleObject) {
    moduleObject = const_cast<SCTDAL::SCT_Module*>(conf().create<SCTDAL::SCT_Module>(lastFilename, module));
    if(moduleObject == 0) {
      conf().abort();
      throw SctConfiguration::ConfigurationException("Failed to create SCT_Module");
    }
  }

  moduleObject->set_sn(module);
  moduleObject->set_active(moduleConf.active);
#warning "Should it get it from this structure too?"
  //  moduleObject->set_group(moduleConf.group);
  moduleObject->set_select(moduleConf.select);
  
  std::vector<const SCTDAL::SCT_Chip*> chips = moduleObject->get_Chips();

  for(int chipNum=0; chipNum<12; chipNum++) {
    SCTDAL::SCT_Chip *chipOb = 0;
#warning "Find old chip"
    if(!chipOb) {
      chipOb = const_cast<SCTDAL::SCT_Chip*>(conf().create<SCTDAL::SCT_Chip>(lastFilename, module + boost::lexical_cast<std::string>(chipNum)));
      if(chipOb == 0) {
	conf().abort();
	throw SctConfiguration::ConfigurationException("Failed to create SCT_Chip");
      }
      chips.push_back(chipOb);
    }

    const ABCDChip &currChip = moduleConf.chip[chipNum]; 

    chipOb->set_id(chipNum);
    chipOb->set_address(currChip.address);
    chipOb->set_active(currChip.active);

    chipOb->set_vthr(currChip.basic.vthr);
    chipOb->set_vcal(currChip.basic.vcal);
    chipOb->set_delay(currChip.basic.delay);
    chipOb->set_preamp(currChip.basic.preamp);
    chipOb->set_shaper(currChip.basic.shaper);

    const int BUF_SIZE = 100;
    char buffer[BUF_SIZE];
    snprintf(buffer, BUF_SIZE, "p0 %e p1 %e p2 %e",
             currChip.caldata.rc_params[0], currChip.caldata.rc_params[1], currChip.caldata.rc_params[2]);
    chipOb->set_rc_function(buffer);
    chipOb->set_rc_function_type(currChip.caldata.rc_function);
        
    const ABCDConfig &chipConf = currChip.basic.config;
    std::vector<short unsigned int> configs;
    configs.push_back(chipConf.readoutMode);
    configs.push_back(chipConf.calibMode);
    configs.push_back(chipConf.trimRange);
    configs.push_back(chipConf.edgeDetect);
    configs.push_back(chipConf.mask);
    configs.push_back(chipConf.accumulate);
    configs.push_back(chipConf.inputBypass);
    configs.push_back(chipConf.outputBypass);
    configs.push_back(chipConf.master);
    configs.push_back(chipConf.end);
    configs.push_back(chipConf.feedThrough);
    chipOb->set_config(configs);

    chipOb->set_c_factor(currChip.caldata.c_factor);
    chipOb->set_target(currChip.target * 2.5);

    std::vector<unsigned long> trims;

    for(int cluster=0; cluster<16; cluster++) {
      int t=cluster * 8; // u*32; t<(u+1)*32; t+=8) {
      snprintf(buffer, BUF_SIZE, "0x%x%x%x%x%x%x%x%x ",
	       currChip.trim[t+0], currChip.trim[t+1], currChip.trim[t+2], currChip.trim[t+3],
	       currChip.trim[t+4], currChip.trim[t+5], currChip.trim[t+6], currChip.trim[t+7]);

      trims.push_back(strtol(buffer, NULL, 16));
    }

    chipOb->set_trims(trims);

    std::vector<unsigned long> masks(&currChip.basic.mask[0], &currChip.basic.mask[4]);
    reverse(masks.begin(), masks.end());
    chipOb->set_mask(masks);
  }

  moduleObject->set_Chips(chips);

  conf().commit();
}

void ConfigurationDALImpl::configureModuleFromFile(const std::string filename) {
#warning "undefined"
  std::cerr << "Undefined at configureModuleFromFile " << __LINE__ << std::endl;
  throw SctConfiguration::ConfigurationException("Can I avoid implementing this one?");
}

void ConfigurationDALImpl::configureROD(unsigned int partition, unsigned int crate, unsigned int rod, SctConfiguration::RodConfig rodConf) {
//   SCTDAL::Crate *crateOb = builder->getOrCreateCrate(partition, crate);

  SCTDAL::Rod *rodOb = builder->getOrCreateRod(partition, crate, rod);

//   typedef std::vector<const SCTDAL::Rod *> RodVector;
//   RodVector rods = crateOb->get_rods();

  if(!rodOb) {
    conf().abort();
    throw SctConfiguration::ConfigurationException("Failed to create ROD");
//     rodOb = const_cast<SCTDAL::Rod*>(conf().create<SCTDAL::Rod>(lastFilename, ""));
//     rodOb->set_id(rod);
  }

  rodOb->set_slot(rodConf.baseAddress >> 24);

  std::vector<const SCTDAL::Slave *> slaves = rodOb->get_slaves();

  for(int s=0; s<4; s++) {
    SCTDAL::Slave *slaveOb = 0;

#warning "Find old slave"

    if(!slaveOb) {
      slaveOb = const_cast<SCTDAL::Slave*>(conf().create<SCTDAL::Slave>(lastFilename, ""));
      if(slaveOb == 0) {
	conf().abort();
	throw SctConfiguration::ConfigurationException("Failed to create Slave");
      }
      slaveOb->set_id(s);
      slaves.push_back(slaveOb);
    }

    slaveOb->set_idramFile(rodConf.slaves[s].idramFile);
    slaveOb->set_ipramFile(rodConf.slaves[s].ipramFile);
    slaveOb->set_extFile(rodConf.slaves[s].extFile);
  }

  rodOb->set_slaves(slaves);

  conf().commit();
}

void ConfigurationDALImpl::configureTIM(unsigned int partition, unsigned int crate, SctConfiguration::TimConfig timConf) {
  SCTDAL::Crate *crateOb = builder->getOrCreateCrate(partition, crate);

  SCTDAL::Tim *timOb = const_cast<SCTDAL::Tim*>(conf().create<SCTDAL::Tim>(lastFilename, ""));
  if(timOb == 0) {
    conf().abort();
    throw SctConfiguration::ConfigurationException("Failed to create Tim");
  }

  crateOb->set_tim(timOb);

  // Neither
  //  timOb->set_baseAddress(timConf.baseAddress);
  //  timOb->set_slot(timConf.baseAddress >> 24);
  //  host->set_mapSize(partition);
  timOb->set_triggerFrequency(timConf.trigFrequency);
  timOb->set_resetFrequency(timConf.resetFrequency);
  conf().commit();
}

void ConfigurationDALImpl::configureBOC(unsigned int partition, unsigned int crate, unsigned int rod, const SctConfiguration::BOCGlobalConfig &conf) {
#warning "undefined"
  std::cerr << "Undefined at " << __LINE__ << std::endl;
}

void ConfigurationDALImpl::setFibreMapping(unsigned int partition, unsigned int crate, unsigned int rod, 
					   unsigned int channel, unsigned int tx, unsigned int rx0, unsigned int rx1) {
  unsigned int order = channel / 6;
  unsigned int id = (channel % 6) + 1;

  const SCTDAL::Rod *rodObj = lookup->getRod(partition, crate, rod);
  if(!rodObj) {
    throw SctConfiguration::ConfigurationException("No ROD to map fibres to");
  }
  SCTDAL::RodMUR *murObj = const_cast<SCTDAL::RodMUR*>(lookup->getMurInRod(rodObj, order));
  if(!murObj) {
    throw SctConfiguration::ConfigurationException("No MUR to map fibres to");
  }
  SCTDAL::Channel *channelObj = builder->getOrCreateChannelInRodMur(murObj, id);
  if(!channelObj) {
    throw SctConfiguration::ConfigurationException("Failed to create channel in MUR");
  }

  channelObj->set_outputFibre(tx);
  channelObj->set_stream0Fibre(rx0);
  channelObj->set_stream1Fibre(rx1);

  conf().commit();
}

template<typename T> void destroyAllObjects(::Configuration &conf, std::vector<const T*> objects) {
  typedef std::vector<const T*> TVector;
//   TVector objects;
//   conf.get<typename T>(objects);

  std::cerr << "Destroying " << objects.size() << " objects\n";

  for(typename TVector::iterator i = objects.begin(); i != objects.end(); i++) {
    conf.destroy(**i);

    conf.commit();
  }
}

void ConfigurationDALImpl::clearAll() {
#warning "Make this work properly!"

#if 0
  std::cerr << "Clearing all\n";

  conf().commit();
  conf().unload();

  delete m_conf;
  m_conf = new ::Configuration("oksconfig");

  system(("rm " + lastFilename).c_str());

  createFile(lastFilename);

  //  conf().load(lastFilename);
#elif 0
  // Have to do it downwards in hierarchy as checks for relationships to dead objects
  std::vector<const SCTDAL::Partition *> partObs ;
  conf().get<SCTDAL::Partition>(partObs);
  destroyAllObjects<SCTDAL::Partition>(conf, partObs);

  std::vector<const SCTDAL::Crate *> crateObs ;
  conf().get<SCTDAL::Crate>(crateObs);
  destroyAllObjects<SCTDAL::Crate>(conf, crateObs);

  std::vector<const SCTDAL::Rod *> rodObs ;
  conf().get<SCTDAL::Rod>(rodObs);
  destroyAllObjects<SCTDAL::Rod>(conf, rodObs);

  std::vector<const SCTDAL::Slave *> slaveObs ;
  conf().get<SCTDAL::Slave>(slaveObs);
  destroyAllObjects<SCTDAL::Slave>(conf, slaveObs);

  std::vector<const SCTDAL::RodMUR *> murObs ;
  conf().get<SCTDAL::RodMUR>(murObs);
  destroyAllObjects<SCTDAL::RodMUR>(conf, murObs);
#endif

//   conf().unload();
//   conf().load(lastFilename);
}

void ConfigurationDALImpl::namePartition(unsigned int partition, const std::string name) {
  SCTDAL::Partition *partObj = builder->getOrCreatePartition(partition);
  if(partObj == 0) {
    conf().abort();
    throw SctConfiguration::ConfigurationException("Failed to create Partition");
  }

  partObj->set_name(name);
  partObj->set_id(partition);

  conf().commit();
}

void ConfigurationDALImpl::configureBOCChannel(unsigned int MUR, unsigned int position, const SctConfiguration::BOCChannelConfig &chanConf) {
  SCTDAL::RodMUR *murObj = const_cast<SCTDAL::RodMUR*>(lookup->getMur(MUR));
  SCTDAL::Channel *channelObj = builder->getOrCreateChannelInRodMur(murObj, position);

  channelObj->set_outputCurrent(chanConf.current);
  channelObj->set_outputDelay(chanConf.delay);
  channelObj->set_outputMarkSpace(chanConf.markSpace);

  channelObj->set_stream0Threshold(chanConf.threshold0);
  channelObj->set_stream0Delay(chanConf.delay0);

  channelObj->set_stream1Threshold(chanConf.threshold1);
  channelObj->set_stream1Delay(chanConf.delay1);

  conf().commit();
}

void ConfigurationDALImpl::mapModuleMUR(unsigned int MUR, unsigned int order, unsigned int RMUR, unsigned int rorder, std::string number) {
  SCTDAL::SCT_Module *moduleObj = const_cast<SCTDAL::SCT_Module *>(lookup->getModule(number));
  if(!moduleObj) {
    conf().abort();
    throw SctConfiguration::ConfigurationException("Can't map to non-existant module: " + number);
  }

  SCTDAL::RodMUR *murObj = const_cast<SCTDAL::RodMUR*>(lookup->getMur(MUR));
  // add number to modules at position order

  std::vector<const SCTDAL::RodModule*> links = murObj->get_rodModules();

  SCTDAL::RodModule *moduleLink = 0;
  for(SCTDAL::RodModuleIterator i = links.begin(); i != links.end(); i++) {
    if((*i)->get_position() == order) {
      moduleLink = const_cast<SCTDAL::RodModule*>(*i);
    }
  }

  if(!moduleLink) {
    moduleLink = const_cast<SCTDAL::RodModule*>(conf().create<SCTDAL::RodModule>(lastFilename, ""));
    if(!moduleLink) {
      conf().abort();
      throw SctConfiguration::ConfigurationException("Can't create RodModule class");
    }

    moduleLink->set_position(order);
    links.push_back(moduleLink);
  }

  moduleLink->set_module(moduleObj);

  murObj->set_rodModules(links);

  SCTDAL::RodMUR *rmurObj = const_cast<SCTDAL::RodMUR*>(lookup->getMur(RMUR));
  // add number to rmodules at position rorder
  std::vector<const SCTDAL::RodRModule*> rlinks = rmurObj->get_rodRModules();

  SCTDAL::RodRModule *rmoduleLink = 0;
  for(SCTDAL::RodRModuleIterator i = rlinks.begin(); i != rlinks.end(); i++) {
    if((*i)->get_position() == rorder) {
      rmoduleLink = const_cast<SCTDAL::RodRModule*>(*i);
    }
  }

  if(!rmoduleLink) {
    rmoduleLink = const_cast<SCTDAL::RodRModule*>(conf().create<SCTDAL::RodRModule>(lastFilename, ""));
    if(!rmoduleLink) {
      conf().abort();
      throw SctConfiguration::ConfigurationException("Can't create RodModule class");
    }

    rmoduleLink->set_position(rorder);
    rlinks.push_back(rmoduleLink);
  }

  rmoduleLink->set_module(moduleObj);

  rmurObj->set_rodRModules(rlinks);

  conf().commit();
}

void ConfigurationDALImpl::unmapModuleMUR(unsigned int MUR, unsigned int order) {
#warning "undefined"
  std::cerr << "Undefined at " << __LINE__ << std::endl;
}

void ConfigurationDALImpl::mapRODMUR(unsigned int partition, unsigned int crate, unsigned int rod, unsigned int order, unsigned int number) {
  SCTDAL::Rod *rodOb = builder->getOrCreateRod(partition, crate, rod);
  //  rodOb->print(2, true, std::cerr);

  std::vector<const SCTDAL::RodMUR *> murs = rodOb->get_rodMURs();

  SCTDAL::RodMUR *murOb = 0;

  for(SCTDAL::RodMURIterator i = murs.begin(); i!=murs.end(); i++) {
    if((*i)->get_order() == order) {
      murOb = const_cast<SCTDAL::RodMUR *>(*i);
    }
  }

  if(murOb && murOb->get_id() != number) {
    murs.erase(find(murs.begin(), murs.end(), murOb));
    murOb = 0;
  }

  if(!murOb) {
    murOb = const_cast<SCTDAL::RodMUR *>(lookup->getMur(number));
  }

  if(!murOb) {
    murOb = const_cast<SCTDAL::RodMUR*>(conf().create<SCTDAL::RodMUR>(lastFilename, boost::lexical_cast<std::string>(number)));
    if(murOb == 0) {
      conf().abort();
      throw SctConfiguration::ConfigurationException("failed to create mur");
    }

    murOb->set_id(number);
  }

  murOb->set_order(order);

  murs.push_back(murOb);
  rodOb->set_rodMURs(murs);

  conf().commit();
}

void ConfigurationDALImpl::unmapRODMUR(unsigned int number) {
#warning "undefined"
  std::cerr << "Undefined at " << __LINE__ << std::endl;
}

void ConfigurationDALImpl::mapBarrelMUR(unsigned int MUR, unsigned int barrel, unsigned int row, int position) {
  const SCTDAL::RodMUR *rodMurObj = lookup->getMur(MUR);

  if(rodMurObj == 0) {
#warning "Make empty RodMUR"
  }

  SCTDAL::Row *rowOb = builder->getOrCreateBarrelRow(barrel, row);

  SCTDAL::GeographyMUR *murOb = 0;
  std::vector<const SCTDAL::GeographyMUR*> murs = rowOb->get_murs();

  for(SCTDAL::GeographyMURIterator i = murs.begin(); i != murs.end(); i++) {
#warning "Or check position"
    if((*i)->get_id() == MUR) {
      murOb = const_cast<SCTDAL::GeographyMUR*>(*i);
#warning "Check for too many matches"
    }
  }

  if(!murOb) {
    murOb = const_cast<SCTDAL::GeographyMUR*>(conf().create<SCTDAL::GeographyMUR>(lastFilename, boost::lexical_cast<std::string>(MUR)));
    if(murOb == 0) {
      conf().abort();
      throw SctConfiguration::ConfigurationException("Failed to create GeographyMUR");
    }
    murOb->set_id(MUR);
    murs.push_back(murOb);
  }

  if(position == -1) {
    murOb->set_position("-1");
  } else if(position == 1) {
    murOb->set_position("1");
  }

  rowOb->set_murs(murs);

  conf().commit();
}

void ConfigurationDALImpl::unmapBarrelMUR(unsigned int MUR) {
#warning "undefined"
  std::cerr << "Undefined at " << __LINE__ << std::endl;
}

void ConfigurationDALImpl::mapEndcapMUR(unsigned int MUR, int disk, unsigned int quadrant, unsigned int position) {
#warning "undefined"
  std::cerr << "Undefined at " << __LINE__ << std::endl;
}

void ConfigurationDALImpl::unmapEndcapMUR(unsigned int MUR) {
#warning "undefined"
  std::cerr << "Undefined at " << __LINE__ << std::endl;
}

void ConfigurationDALImpl::swapMURNames(unsigned int MUR1, unsigned int MUR2) {
#warning "undefined"
  std::cerr << "Undefined at " << __LINE__ << std::endl;
}

void ConfigurationDALImpl::mapPowerChannel(unsigned int MUR, unsigned int number, unsigned int partition, unsigned int crate, unsigned int channel) {
#warning "Change schema to relationship to MUR"
  SCTDAL::PowerChannel *chanObj = builder->getOrCreatePowerChannel(crate, channel);

  chanObj->set_MUR(MUR);
  chanObj->set_module(number);

  conf().commit();
}

void ConfigurationDALImpl::unmapPowerChannel(unsigned int MUR, unsigned int number) {
#warning "undefined"
  std::cerr << "Undefined at " << __LINE__ << std::endl;
}

void ConfigurationDALImpl::modifyDefaultPowerParam(std::string state, std::string name, std::string type, float value) {
#warning "undefined"
  std::cerr << "Undefined at " << __LINE__ << std::endl;
}

void ConfigurationDALImpl::modifyHVCardParam(unsigned int crate, unsigned int card,
					     std::string name, std::string type, float value) {
#warning "undefined"
  std::cerr << "Undefined at " << __LINE__ << std::endl;
}

void ConfigurationDALImpl::modifyLVCardParam(unsigned int crate, unsigned int card,
					     std::string name, std::string type, float value) {
#warning "undefined"
  std::cerr << "Undefined at " << __LINE__ << std::endl;
}

void ConfigurationDALImpl::modifyPowerParam(unsigned int MUR, unsigned int number,
					    std::string state, std::string name, std::string type, float value) {
  const SCTDAL::PowerChannel *chanObj = lookup->getPowerChannelByMUR(MUR, number);
  if(!chanObj) {
    throw SctConfiguration::ConfigurationException("MUR/module pair not mapped to power");
  }
  SCTDAL::PowerParam *paramObj = builder->getOrCreatePowerParamInChannel(const_cast<SCTDAL::PowerChannel*>(chanObj), state, name);

  if(type == "loAlarm") {
    paramObj->set_loAlarm(value);
  } else if(type == "loWarn") {
    paramObj->set_loWarn(value);
  } else if(type == "value") {
    paramObj->set_value(value);
  } else if(type == "hiWarn") {
    paramObj->set_hiWarn(value);
  } else if(type == "hiAlarm") {
    paramObj->set_hiAlarm(value);
  }

  conf().commit();
}

void ConfigurationDALImpl::modifyPowerParamCC(unsigned int crate, unsigned int channel,
					      std::string state, std::string name, std::string type, float value) {
  const SCTDAL::PowerChannel *chanObj = lookup->getPowerChannel(crate, channel);
  if(!chanObj) {
    // Allow modification of unmapped power channel?
    throw SctConfiguration::ConfigurationException("MUR/module pair not mapped to power");
  }
  SCTDAL::PowerParam *paramObj = builder->getOrCreatePowerParamInChannel(const_cast<SCTDAL::PowerChannel*>(chanObj), state, name);

  if(type == "loAlarm") {
    paramObj->set_loAlarm(value);
  } else if(type == "loWarn") {
    paramObj->set_loWarn(value);
  } else if(type == "value") {
    paramObj->set_value(value);
  } else if(type == "hiWarn") {
    paramObj->set_hiWarn(value);
  } else if(type == "hiAlarm") {
    paramObj->set_hiAlarm(value);
  }

  conf().commit();
}

