/**
BookKeeper
@author robinson
*/
package DisplayGUI;
import Preferences.*;
import javax.swing.JCheckBox;
import Sct_CalibrationController.TestData;
import is.*;
import Sct.Serializable;
import Sct.IS.*;
import java.io.*;
import java.util.*;

public class BookKeeper implements displayParams, SCTInfo {
    private static BookKeeper instance = new BookKeeper();
    java.io.File topLevel;
//    public JCheckBox bookkeepingEnabled = new JCheckBox("Enable Book-keeping");
    public javax.swing.JMenuItem bookkeepingEnabledItem, runPromptEnabledItem, testPromptEnabledItem;
    boolean bookkeepingEnabled = false,runPromptEnabled = false, testPromptEnabled=false;
    //public JCheckBox runPromptEnabled = new JCheckBox("Prompt for comments on new Run");
    //public JCheckBox testPromptEnabled = new JCheckBox("Prompt for comments on new Test");
    SortedMap testMap;
    BookKeeperPrompt prompt = null;

    PreferencesInterface prefs;

    private BookKeeper() {
 //       bookkeepingEnabled.setEnabled(true);
        Refresh();
        testMap = new TreeMap();
        }

    public static BookKeeper getInstance() {
        return instance;
    }



    public void Refresh() {
        prefs = PreferencesInterface.getInstance();

 //       bookkeepingEnabled.setSelected(prefs.getPreference(PreferencesInterface.BOOKKEEPING).equals("TRUE"));
      bookkeepingEnabledItem = new javax.swing.JMenuItem();
	bookkeepingEnabledItem.setFont(menuFont);
	runPromptEnabledItem = new javax.swing.JMenuItem();
	runPromptEnabledItem.setFont(menuFont);
	testPromptEnabledItem = new javax.swing.JMenuItem();
	testPromptEnabledItem.setFont(menuFont);
 	bookkeepingEnabled = prefs.getPreference(PreferencesInterface.BOOKKEEPING).equals("TRUE");
	runPromptEnabled = prefs.getPreference(PreferencesInterface.RUN_PROMPT).equals("TRUE");
	testPromptEnabled = prefs.getPreference(PreferencesInterface.TEST_PROMPT).equals("TRUE");
	setMenuItems();

        bookkeepingEnabledItem.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                    bookkeepingEnabled = !bookkeepingEnabled;
			  setMenuItems();
                    PreferencesInterface.getInstance().setPreference(PreferencesInterface.BOOKKEEPING,bookkeepingEnabled ? "TRUE" : "FALSE");
                    runPromptEnabledItem.setEnabled(bookkeepingEnabled);
                    testPromptEnabledItem.setEnabled(bookkeepingEnabled);
                 }
             });
        runPromptEnabledItem.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                    runPromptEnabled = !runPromptEnabled;
			  setMenuItems();
                    PreferencesInterface.getInstance().setPreference(PreferencesInterface.RUN_PROMPT,runPromptEnabled ? "TRUE" : "FALSE");
                 }
             });
        testPromptEnabledItem.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                    testPromptEnabled = !testPromptEnabled;
			  setMenuItems();
                    PreferencesInterface.getInstance().setPreference(PreferencesInterface.TEST_PROMPT,testPromptEnabled ? "TRUE" : "FALSE");
                 }
             });
         }

     private void setMenuItems() {
      bookkeepingEnabledItem.setText(bookkeepingEnabled ? "Disable BookKeeping (currently enabled)" : "Enable Bookkeeping (currently disabled)");
	runPromptEnabledItem.setText(runPromptEnabled ? "Disable prompt at run start (currently enabled)" : "Enable prompt at run start (currently disabled)");
	testPromptEnabledItem.setText(testPromptEnabled ? "Disable prompt for new tests (currently enabled)" : "Enable prompt for new tests (currently disabled)");
	}
     public boolean directoriesOK() {

        if(prefs.getPreference(PreferencesInterface.WEB_LOOKUP_DIR).equals(PreferencesInterface.unDefinedString)) {
           System.err.println("SctGUI::BookKeeper - ERROR - Bookkeeping directory is not defined in your Preferences!");
           return false;
           }
        java.io.File dir = new java.io.File(prefs.getPreference(PreferencesInterface.WEB_LOOKUP_DIR));
        if(!dir.exists()) {
           System.err.println("SctGUI::BookKeeper - ERROR - The bookkeeping directory defined in your Preferences does not exist!");
           return false;
           }
        topLevel = new java.io.File(dir,"bookkeeping");
        if(!topLevel.exists()) {
           if(!topLevel.mkdir()) {
                javax.swing.JOptionPane.showMessageDialog(null,"Failed to write to bookkeeping directory.\nAre you sure that you have write access?");
                topLevel=null;
                return false;
                }
           }
         return true;
        }



    public void addTest(String objectName) {

        if(!bookkeepingEnabled) return;

        if(!directoriesOK()) return;

        boolean testPrompt=true;

         try {
           TestData testData = new TestData(); 
           GuiComponents.System.ISInterface.getInstance().getRepository().getValue(objectName, testData); 
           String runNo = Integer.toString(testData.runNumber);

           java.io.File runDir = new java.io.File(topLevel,"RUN"+guiUtilities.DaveUtils.padString(runNo,6,'0'));
           if(!runDir.exists()) {
              testMap = new TreeMap();
              if(!runDir.mkdir()) {
                System.err.println("SctGUI::BookKeeper - ERROR - Failed to create run directory");
                return;
                }
              if(runPromptEnabled) {
                testPrompt=false;
                File runFile = new File(runDir,"comments.txt");               
                if(prompt==null) prompt = new BookKeeperPrompt();
                prompt.showPrompt(runFile,runNo,null,null);
                }
              }
// write test file
            Integer scanNo = new Integer(testData.startScanNumber);
            if(!testMap.containsKey(scanNo) && testPromptEnabled && testPrompt) {
                File testFile = new File(runDir,"TEST"+guiUtilities.DaveUtils.padString(scanNo.toString(),6,'0')+".txt");
                if(prompt==null) prompt = new BookKeeperPrompt();
                prompt.showPrompt(testFile,runNo,scanNo.toString(),testData.testName);
               }
            testMap.put(scanNo,getTestString(testData));
            writeTestFile(runNo,Integer.toString(testData.startScanNumber));

          }catch(Exception e) {
              System.err.println("SctGUI::BookKeeper - Cant access "+objectName+" - "+e.toString());
              }
          }

    public void showCommentPrompt(int run, int scan, String testName) {

        if(!directoriesOK()) return;

        String runNo = Integer.toString(run);
        String scanNo = (testName!=null) ? Integer.toString(scan) : null;

        File runDir = new File(topLevel,"RUN"+guiUtilities.DaveUtils.padString(runNo,6,'0'));
        File cFile = (scanNo==null) ? new File(runDir,"comments.txt") : new File(runDir,"TEST"+guiUtilities.DaveUtils.padString(scanNo,6,'0')+".txt");
        if(prompt==null) prompt = new BookKeeperPrompt();
        prompt.showPrompt(cFile,runNo,scanNo,testName);
        }

    public String getTestString(TestData testData) {
        StringBuffer s = new StringBuffer();
        s.append("RUN="+Integer.toString(testData.runNumber));
        s.append(";NAME="+testData.testName);
        s.append(";START="+Integer.toString(testData.startScanNumber));
        s.append(";NSCANS="+Integer.toString(testData.nScans));
        s.append(";DATE="+guiUtilities.DaveUtils.getTodaysDate());
        s.append(";STIME="+testData.startTime);
        s.append(";ETIME="+testData.endTime);
        s.append(";STATUS="+Integer.toString(testData.status));
        s.append(";NMODULES="+Integer.toString(testData.modules.length));
        return s.toString();
        }

    public void writeTestFile(String runNo, String scanNo) {
       try {
        File runDir = new File(topLevel,"RUN"+guiUtilities.DaveUtils.padString(runNo,6,'0'));
        File testIndex = new File(runDir,"index.txt");
        BufferedWriter out = new BufferedWriter(new FileWriter(testIndex));

        for (Iterator i=testMap.entrySet().iterator(); i.hasNext(); ) {
            Map.Entry e = (Map.Entry) i.next();
            out.write((String)e.getValue());
            out.newLine();
            }
        out.close();
        }catch(Exception e) {
           System.err.println("SctGUI::BookKeeper - FAILed to update test file - "+e.toString());
           }
        }

        

}
         
