/*
 * CalibrationContollerPanelMenu class
 * defines and handles CC panel menu actions
 */
 
package DisplayGUI;

import javax.swing.JMenu;
import javax.swing.JMenuItem;

import GuiComponents.System.SystemInterface;
import Sct_CalibrationController.CalibrationControllerPackage.*;

/**
 *
 * @author  robinson
 */


/** Menu bar for GUI */
public class CalibrationControllerPanelMenu extends JMenu implements displayParams {

	gui guiControl;

    private String[] ccUpdateStrings = {"Update","No-Update","Wait"};

	private static final int UPDATE=0;
	private static final int NO_UPDATE=1;
	private static final int WAIT=2;

    private javax.swing.JCheckBox[] ccUpdateModes = new javax.swing.JCheckBox[ccUpdateStrings.length];

	public CalibrationControllerPanelMenu(gui guiControl) {
		super("CalibrationController");
		this.guiControl = guiControl;
       	setFont(menuFont);
       	JMenuItem item = new JMenuItem("Ping");
       	item.setFont(menuFont);
       	item.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                   try {
                      if (SystemInterface.getInstance().getCalibrationController() != null)  {
                         ipc.servantPackage.info info = SystemInterface.getInstance().getCalibrationController().information();
                         System.out.println("CalibrationController running: " + info.pid + " by " + info.owner + " on " +info.host + " since " + info.time);
                      } else System.out.println("CalibrationController not running");
                      }catch(Exception eNoAbort){System.err.println("Error aborting calibrationController: "+eNoAbort.toString());}
                    }
              });
       	add(item);

       	addSeparator();

       	javax.swing.ButtonGroup thisGroup = new javax.swing.ButtonGroup();
       	for(int i=0;i<ccUpdateModes.length;i++) {
         	ccUpdateModes[i] = new javax.swing.JCheckBox(ccUpdateStrings[i]);
         	thisGroup.add(ccUpdateModes[i]);
         	ccUpdateModes[i].setFont(menuFont);
         	add(ccUpdateModes[i]);
         	}
       	item = new JMenuItem("Abort");
       	item.setFont(menuFont);
       	item.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                   try {
//                       System.out.println("abort of CC not yet implemented");
                       SystemInterface.getInstance().getCalibrationController().abort();
                       }catch(Exception eNoAbort){System.err.println("Error aborting calibrationController: "+eNoAbort.toString());}
                    }
              });
       	addSeparator();
       	add(item);


       	ccUpdateModes[UPDATE].setSelected(true);

		for(int i=0;i<ccUpdateModes.length;i++) {
	   // ccUpdateModes[i].setEnabled(false);
        	switch(i) {
        		case UPDATE:
            		ccUpdateModes[i].addActionListener(new java.awt.event.ActionListener() {
             		public void actionPerformed(java.awt.event.ActionEvent evt) {
                   		try {
                       	SystemInterface.getInstance().getCalibrationController().setUpdateOption(UpdateOption.update);
                       	checkCCUpdateMode();
                   		}catch(Exception e){System.err.println("Error : "+e.toString());}
                   		}
              		});
            	break;
            	case NO_UPDATE:
            		ccUpdateModes[i].addActionListener(new java.awt.event.ActionListener() {
             		public void actionPerformed(java.awt.event.ActionEvent evt) {
                   		try {
                       		SystemInterface.getInstance().getCalibrationController().setUpdateOption(UpdateOption.noupdate);
                       		checkCCUpdateMode();
                   		}catch(Exception e){System.err.println("Error : "+e.toString());}
                   		}
              		});
            	break;
            	case WAIT:
            		ccUpdateModes[i].addActionListener(new java.awt.event.ActionListener() {
             		public void actionPerformed(java.awt.event.ActionEvent evt) {
                   		try {
                       		SystemInterface.getInstance().getCalibrationController().setUpdateOption(UpdateOption._wait);
                       		checkCCUpdateMode();
                   		}catch(Exception e){System.err.println("Error : "+e.toString());}
                   		}
              			});
            	break;
            	default:
            	}
        }
	}
   public void checkCCUpdateMode() {
         if(guiControl.calibrationControllerPanel==null || guiControl.calibrationControllerPanel.status==null) {   // why is this necessary????
             System.out.println("CalibrationControllerPanel.status object not yet available... update mode not yet known");
             return;
             }
         String updateMode = guiControl.calibrationControllerPanel.status.updateOption;
         if(updateMode.equals("NOUPDATE")) ccUpdateModes[NO_UPDATE].setSelected(true);
         else if(updateMode.equals("WAIT")) ccUpdateModes[WAIT].setSelected(true);
         else   ccUpdateModes[UPDATE].setSelected(true);
         }

}
