package DisplayGUI;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import java.awt.Graphics2D.*;
import java.awt.geom.*;
import java.util.*;
import GuiComponents.System.*;

public class DisplayPanel extends JPanel implements guiInterface, TestDataInfo  {
        gui guiControl;

        FontMetrics metrics;
		int maxWidth;
        private int displayOption=0; // for testdata 0:rod 1: sctdb 2: rod-sctdb comparison

        private JMenu dataPopupMenu, sctdbPopupMenu;

        private Rectangle draggedRectangle;
        
        private boolean firstTime = true;
		private Rectangle area;
        private Dimension dim;

        private ModuleCell redCell,lastClickedCell,popupCell;
        private String mouseLocationText, mouseLocationSN;
		private int lastClickedRodSlot=-1;

        private DisplayTitle leftTitle, middleTitle, rightTitle, optionTitle;

        ConfigurationInterface config;

        private java.util.List changedModuleList;

    public DisplayPanel(gui parent){
        guiControl=parent;
        setBackground(Color.white);
        addMouseMotionListener(this);
        addMouseListener(this);
        dim = getSize();
        maxWidth = (int)dim.getWidth();

        setMinimumSize(new Dimension(14*boxWidth,yOffSet+58*boxHeight));
        setPreferredSize(new Dimension(14*boxWidth,yOffSet+58*boxHeight));

        config = ConfigurationInterface.getInstance();
        config.setWindowSize(maxWidth,(int)dim.getHeight());

        leftTitle = null;
        middleTitle = null;
        rightTitle = null;
        optionTitle=null;

        changedModuleList = new ArrayList();


    }

    


    // Handles the event of the user pressing down the mouse button.
    public void mousePressed(MouseEvent e){
        if(guiControl.displayStatus.getStatus()!=DisplayStatus.MODULE_GROUP_SELECTION) return;
		if(guiControl.selectorPane.getDisplayParameterIndex()>0) return; // must be first parameter in configParams list
        draggedRectangle = new Rectangle(new Point(e.getX(),e.getY()));   
    }

    // Handles the event of a user dragging the mouse while holding
    // down the mouse button.
    public void mouseDragged(MouseEvent e){

 
        if(guiControl.displayStatus.getStatus()!=DisplayStatus.MODULE_GROUP_SELECTION) return;
	if(guiControl.selectorPane.getDisplayParameterIndex()>0) return; // must be first parameter in configParams list
        redCell=null;
        int width = e.getX() - (int)draggedRectangle.getX();
        int height = e.getY() - (int)draggedRectangle.getY();
        int thisModuleGroup = guiControl.selectorPane.getMouseClickIndex();
// re-define or resize the dragged rectangle
        if(width<0 || height<0) {
             int oldWidth = (int)draggedRectangle.getWidth();
             int oldHeight = (int)draggedRectangle.getHeight();
             draggedRectangle = new Rectangle(new Point(e.getX(),e.getY()),new Dimension(Math.abs(width)+oldWidth,Math.abs(height)+oldHeight));
             }
        else draggedRectangle.setSize(new Dimension(width,height));

		config = ConfigurationInterface.getInstance();
		int viewCategoryIndex = guiControl.selectorPane.getViewCategoryIndex();
        for (java.util.Iterator m = config.getSerialNumberList(viewCategoryIndex,guiControl.selectorPane.getViewIndex()).iterator(); m.hasNext(); ) {
               String thisSN=(String)m.next();
               ModuleCell thisCell = config.getModuleCell(thisSN);
               if(thisCell.getPolygon(viewCategoryIndex).intersects(draggedRectangle)) {
                   switch(e.getModifiers()) {
                     case InputEvent.BUTTON2_MASK:
                     case InputEvent.BUTTON3_MASK:
                     if(thisCell.getModuleGroup()==thisModuleGroup) {
                           thisCell.setModuleGroup(-1);
                           changedModuleList.add(thisCell);
                           }
                     break;
                   default:
                       if(!thisCell.isSelected()) {
                              thisCell.setModuleGroup(thisModuleGroup);
                              changedModuleList.add(thisCell);
                              }
                     }
               }
          }
        if(guiControl.displayStatus.getStatus()==DisplayStatus.MODULE_GROUP_SELECTION && guiControl.selectorPane.getDisplayParameterIndex()==0) setDisplayColors();

		repaint();

    }

    // Handles the event of a user releasing the mouse button.
    public void mouseReleased(MouseEvent e){
       if(draggedRectangle!=null) {
           draggedRectangle=null;
           repaint();
           }
       if(guiControl.displayStatus.getStatus()==DisplayStatus.MODULE_GROUP_SELECTION && guiControl.selectorPane.getDisplayParameterIndex()==0 && changedModuleList.size()>0) {
           guiControl.tablesDisplayPane.updateTables();
           ConfigurationInterface.getInstance().updateModuleGroups(changedModuleList);
           changedModuleList.clear();
           }
    }
       
     // This method is required by MouseListener.
     public void mouseMoved(MouseEvent e){
       if(draggedRectangle!=null) return;

	   config = ConfigurationInterface.getInstance();

       ModuleCell thisCell = config.getCell(guiControl.selectorPane.getViewCategoryIndex(),guiControl.selectorPane.getViewIndex(),e.getX(),e.getY());
       if(thisCell==null) {
           redCell = null;
           if(mouseLocationText!=null) {
               mouseLocationText = null;
               mouseLocationSN=null;
               repaint();
           }
           return;
       }

// check for redCell
       if(thisCell==redCell)  return;  // no change
       else {
//            if(redCell!=null) redCell.setSelected(false);
//            thisCell.setSelected(true);
            redCell = thisCell;
       }
// update mouse Text
       String sn = thisCell.getSerialNo();
       String infoString = sn+" ("+config.getSNInfo(sn,SNInfo.ROD_CRATE)+","+config.getSNInfo(sn,SNInfo.ROD_NUMBER)+","+config.getSNInfo(sn,SNInfo.ROD_CHANNEL)+","+config.getSNInfo(sn,SNInfo.DCS_CRATE)+","+config.getSNInfo(sn,SNInfo.DCS_CHANNEL)+") ";
       switch(guiControl.displayStatus.getStatus()) {
          case DisplayStatus.IS_MONITOR:
               if(guiControl.tablesDisplayPane.getTestSelection()!=null && thisCell.getData()!=null) {
                 mouseLocationText = new String(infoString+" : "+thisCell.getData().intValue());
                 }
               break;
          case DisplayStatus.MODULE_GROUP_SELECTION:
		  	if(guiControl.selectorPane.getDisplayParameterIndex()>0) { // BOC parameter
               		ColorScale thisColorScale = guiControl.colorScalePane.getColorScale();
               		if(thisColorScale!=null) mouseLocationText = new String(infoString+" : "+thisColorScale.getLabel(thisCell.getData()));
               		else mouseLocationText = new String(infoString+" : No Data");
				}
			else { // module group
               		if(guiControl.selectorPane.getViewIndex()<4) mouseLocationText = new String(infoString+" MUR: "+thisCell.getMUR()+" Position: "+thisCell.getModulePosition());
               		else mouseLocationText = new String(infoString+" MUR: "+thisCell.getMUR()+" Quad/Pos: "+thisCell.getQuadrant()+"/"+thisCell.getEndCapPosition());
               		}
			   break;
          default:  
               ColorScale thisColorScale = guiControl.colorScalePane.getColorScale();
               if(thisColorScale!=null) mouseLocationText = new String(infoString+" : "+thisColorScale.getLabel(thisCell.getData()));
               else mouseLocationText = new String(infoString+" : No Data");
               break;
          }


       repaint();
     }

     // These methods are required by MouseMotionListener.
     public void mouseClicked(MouseEvent e){

	   config = ConfigurationInterface.getInstance();
       ModuleCell thisCell = config.getCell(guiControl.selectorPane.getViewCategoryIndex(),guiControl.selectorPane.getViewIndex(),e.getX(),e.getY());
       if(thisCell==null) {
          switch(e.getModifiers()) {
           case InputEvent.BUTTON2_MASK:
           case InputEvent.BUTTON3_MASK:

           if(guiControl.selectorPane.getViewCategoryIndex()==ConfigurationInterface.RODCRATE_VIEWS) {
		   		int viewIndex = guiControl.selectorPane.getViewIndex();
		   		lastClickedRodSlot = config.getRodSlotNumber(viewIndex,e);
				switch(lastClickedRodSlot) {
					case -1: guiControl.menuActions.showPopupMenu(e); break;
					case 13: guiControl.menuActions.showTIMRegisterMenu(e); break;
					default:
						if(config.getASerialNumberForSlot(guiControl.selectorPane.getViewIndex(),lastClickedRodSlot)!=null) guiControl.menuActions.showRODMenu(e);
						else guiControl.menuActions.showPopupMenu(e);
					}
				}
           else guiControl.menuActions.showPopupMenu(e); // general display popup menu, not specific to a module
           break;
          default:
          }
         return;
        }
       
       lastClickedCell = thisCell;

       switch(guiControl.displayStatus.getStatus()) {

         case DisplayStatus.MODULE_GROUP_SELECTION:

		 	if(guiControl.selectorPane.getDisplayParameterIndex()>0) { // BOC config parameters

           		guiControl.tablesDisplayPane.resultsTable.setSelectedSerialNo(thisCell.getSerialNo(),true);
           		switch(e.getModifiers()) {
               		case InputEvent.BUTTON2_MASK:
               		case InputEvent.BUTTON3_MASK:
                  		popupCell=thisCell;
                   		JPopupMenu pMenu = guiControl.guiMenus.getMenu(GuiMenus.MODULE_BOC).getPopupMenu();
                   		pMenu.show(e.getComponent(),e.getX(),e.getY());
                   		pMenu.setInvoker(guiControl.guiMenus.getMenu(GuiMenus.MODULE_BOC));
               		break;
              		default:
          			}

				}

			else { // module group

            	int thisModuleGroup =guiControl.selectorPane.getMouseClickIndex();
              	switch(e.getModifiers()) {
               		case InputEvent.BUTTON2_MASK:
               		case InputEvent.BUTTON3_MASK:
                     	if(thisCell.getModuleGroup()==thisModuleGroup) {
                           	thisCell.setModuleGroup(-1);
                           	changedModuleList.add(thisCell);
                           	}
                     	break;
              		default:
                     	if(!thisCell.isSelected()) {
                        	thisCell.setModuleGroup(thisModuleGroup);
                            changedModuleList.add(thisCell);
                            }
             		}

           		repaint();

           		if(changedModuleList.size()>0) {
              		guiControl.tablesDisplayPane.updateTables();
              		config.updateModuleGroups(changedModuleList);
              		changedModuleList.clear();
              		}

			  }
           break;

       case DisplayStatus.IS_MONITOR:
           guiControl.tablesDisplayPane.listISObjects(thisCell);
           guiControl.tablesDisplayPane.resultsTable.setSelectedSerialNo(thisCell.getSerialNo(),false);
           switch(e.getModifiers()) {
               case InputEvent.BUTTON2_MASK:
               case InputEvent.BUTTON3_MASK:
                  popupCell=thisCell;
                   JPopupMenu pMenu = guiControl.guiMenus.getMenu(GuiMenus.MODULE_DATA_MANAGER).getPopupMenu();
                   pMenu.show(e.getComponent(),e.getX(),e.getY());
                   pMenu.setInvoker(guiControl.guiMenus.getMenu(GuiMenus.MODULE_DATA_MANAGER));
               break;
              default:
           }
           break;
       case DisplayStatus.TEST_DATA:
           guiControl.tablesDisplayPane.resultsTable.setSelectedSerialNo(thisCell.getSerialNo(),true);
           switch(e.getModifiers()) {
               case InputEvent.BUTTON2_MASK:
               case InputEvent.BUTTON3_MASK:
                  popupCell=thisCell;
                   JPopupMenu pMenu = guiControl.guiMenus.getMenu(GuiMenus.MODULE_RESULTS).getPopupMenu();
                   pMenu.show(e.getComponent(),e.getX(),e.getY());
                   pMenu.setInvoker(guiControl.guiMenus.getMenu(GuiMenus.MODULE_RESULTS));
               break;
              default:
           }
           break;
       case DisplayStatus.SCTDB_DATA:
       case DisplayStatus.SCTDB_RODDAQ_DATA:
           guiControl.tablesDisplayPane.resultsTable.setSelectedSerialNo(thisCell.getSerialNo(),true);
           switch(e.getModifiers()) {
               case InputEvent.BUTTON2_MASK:
               case InputEvent.BUTTON3_MASK:
                  popupCell=thisCell;
                    JPopupMenu pMenu = guiControl.guiMenus.getMenu(GuiMenus.MODULE_SCTDB).getPopupMenu();
                   pMenu.show(e.getComponent(),e.getX(),e.getY());
                   pMenu.setInvoker(guiControl.guiMenus.getMenu(GuiMenus.MODULE_SCTDB));
                   break;
                default:
                }
               break;
       case DisplayStatus.IV_DATA:
           guiControl.tablesDisplayPane.resultsTable.setSelectedSerialNo(thisCell.getSerialNo(),true);
           switch(e.getModifiers()) {
               case InputEvent.BUTTON2_MASK:
               case InputEvent.BUTTON3_MASK:
                  popupCell=thisCell;
                    JPopupMenu pMenu = guiControl.guiMenus.getMenu(GuiMenus.MODULE_IVSCAN).getPopupMenu();
                   pMenu.show(e.getComponent(),e.getX(),e.getY());
                   pMenu.setInvoker(guiControl.guiMenus.getMenu(GuiMenus.MODULE_IVSCAN));
               break;
              default:
           }
	   	break;
       	case DisplayStatus.DCS_DATA:
           	guiControl.tablesDisplayPane.resultsTable.setSelectedSerialNo(thisCell.getSerialNo(),true);
           	switch(e.getModifiers()) {
               	case InputEvent.BUTTON2_MASK:
               	case InputEvent.BUTTON3_MASK:
                  	popupCell=thisCell;
                    JPopupMenu pMenu = guiControl.guiMenus.getMenu(GuiMenus.MODULE_DCS).getPopupMenu();
                   	pMenu.show(e.getComponent(),e.getX(),e.getY());
                   	pMenu.setInvoker(guiControl.guiMenus.getMenu(GuiMenus.MODULE_DCS));
               		break;
              	default:
           	}


           	break;
      	case DisplayStatus.PROBE_DATA:
           guiControl.tablesDisplayPane.resultsTable.setSelectedSerialNo(thisCell.getSerialNo(),true);
           switch(e.getModifiers()) {
               	case InputEvent.BUTTON2_MASK:
               	case InputEvent.BUTTON3_MASK:
                	popupCell=thisCell;
                    JPopupMenu pMenu = guiControl.guiMenus.getMenu(GuiMenus.MODULE_PROBE).getPopupMenu();
                   	pMenu.show(e.getComponent(),e.getX(),e.getY());
                   	pMenu.setInvoker(guiControl.guiMenus.getMenu(GuiMenus.MODULE_PROBE));
               		break;
              	default:
				}


           	break;
        	default:
	    guiControl.tablesDisplayPane.resultsTable.setSelectedSerialNo(thisCell.getSerialNo(),true);
        }
       
       }

     public void mouseExited(MouseEvent e){}
     public void mouseEntered(MouseEvent e){}
                         

     public void refreshDisplay(){
     	redCell=null;
	  	repaint();
     	}

    public void initPanelParameters() {  //must do this because config needs the panel size before it is painted
            dim = getSize();
            maxWidth = (int)dim.getWidth();
            ConfigurationInterface.getInstance().setWindowSize(maxWidth,(int)dim.getHeight());
            }


    public void paintComponent(Graphics g){

        super.paintComponent(g);

        Graphics2D g2 = (Graphics2D)g;
		g2.setStroke(new BasicStroke(1.0f));

        if ( firstTime ) {
//            dim = getSize();
            area = new Rectangle(dim);
//            maxWidth = (int)dim.getWidth();
            firstTime = false;
            g2.setFont(displayTextFont);
            metrics = g2.getFontMetrics();
//            config.setWindowSize(maxWidth,(int)dim.getHeight());
	}
        // Clears the rectangle that was previously drawn.
        g2.setPaint(Color.white);
        g2.fill(area);

        
        ConfigurationInterface.getInstance().drawViewOutline(g2,guiControl.selectorPane.getViewCategoryIndex(),guiControl.selectorPane.getViewIndex());
        drawTitle(g2);
        if(mouseLocationText!=null) drawMouseLocation(g2,mouseLocationText);
        drawGraphicsCells(g2);
        drawMouseSelections(g2);

    }


    
    public void drawGraphicsCells(Graphics2D g2) {
        ColorScale thisColorScale = guiControl.colorScalePane.getColorScale();
	  config = ConfigurationInterface.getInstance();
	  int viewCategoryIndex = guiControl.selectorPane.getViewCategoryIndex();
	  switch(viewCategoryIndex) {
	  	case ConfigurationInterface.RODCRATE_VIEWS:
        		for (java.util.Iterator m = config.getSerialNumberList(viewCategoryIndex,guiControl.selectorPane.getViewIndex()).iterator(); m.hasNext(); ) {
               		String thisSN=(String)m.next();
               		ModuleCell thisCell = config.getModuleCell(thisSN);
               		g2.setPaint(thisCell.getColor());
				java.awt.Polygon thisP = thisCell.getPolygon(viewCategoryIndex);
               		g2.fill(thisCell.getPolygon(viewCategoryIndex));
               		g2.setColor(Color.black);
               		g2.draw(thisCell.getPolygon(viewCategoryIndex));
               		}
			break;
		case ConfigurationInterface.SCT_VIEWS:
        		if(guiControl.selectorPane.getViewIndex()<4) {
        		for (java.util.Iterator m = config.getSerialNumberList(viewCategoryIndex,guiControl.selectorPane.getViewIndex()).iterator(); m.hasNext(); ) {
               		String thisSN=(String)m.next();
               		ModuleCell thisCell = config.getModuleCell(thisSN);
               		g2.setPaint(thisCell.getColor());
               		g2.fill(thisCell.getPolygon(viewCategoryIndex));
               		g2.setColor(Color.black);
               		g2.draw(thisCell.getPolygon(viewCategoryIndex));
               		}
          		}
         		else {
          // for endcaps, must draw/paint middle ring first
         		for (java.util.Iterator m = config.getSerialNumberList(guiControl.selectorPane.getViewCategoryIndex(),guiControl.selectorPane.getViewIndex()).iterator(); m.hasNext(); ) {
               String thisSN=(String)m.next();
               ModuleCell thisCell = config.getModuleCell(thisSN);
               int ring = thisCell.getPosition();
               if(ring<13 || ring>22) continue;
               g2.setPaint(thisCell.getColor());
               g2.fill(thisCell.getPolygon(viewCategoryIndex));
               g2.setColor(Color.black);
               g2.draw(thisCell.getPolygon(viewCategoryIndex));
               }
         for (java.util.Iterator m = config.getSerialNumberList(guiControl.selectorPane.getViewCategoryIndex(),guiControl.selectorPane.getViewIndex()).iterator(); m.hasNext(); ) {
               String thisSN=(String)m.next();
               ModuleCell thisCell = config.getModuleCell(thisSN);
               int ring = thisCell.getPosition();
               if(ring>=13 && ring<=22) continue;
               g2.setPaint(thisCell.getColor());
               g2.fill(thisCell.getPolygon(viewCategoryIndex));
               g2.setColor(Color.black);
               g2.draw(thisCell.getPolygon(viewCategoryIndex));
               }
           }
	    break;
	    default:
	    }
    }

   public void drawMouseSelections(Graphics2D g2) {
		if(redCell!=null) {
            g2.setColor(Color.red);
            g2.draw(redCell.getPolygon(guiControl.selectorPane.getViewCategoryIndex()));
          	}
       	if(draggedRectangle!=null) {
            g2.setColor(Color.red);
            g2.draw(draggedRectangle);
           	}
       	g2.setColor(Color.lightGray);
      	}



    public void drawMouseLocation(Graphics2D g2, String mouseLocationText) {
        g2.setFont(displayTextFont);
//        g2.setPaint(darkBlueColor);
        g2.setPaint(Color.white);
        metrics = g2.getFontMetrics();
        int stringWidth = metrics.stringWidth(mouseLocationText);
        int stringHeight = metrics.getHeight();
        g2.drawString(mouseLocationText,maxWidth/2+7*boxWidth - stringWidth-5,yOffSet+boxHeight-stringHeight);
    }

    public void drawTitle(Graphics2D g2) {
        g2.setPaint(Color.blue);
        int leftX = maxWidth/2-6*boxWidth;
        int rightX = maxWidth/2+7*boxWidth;
        Rectangle top = new Rectangle(leftX,0,13*boxWidth,yOffSet);
        g2.fill(top);
        leftX+=5;

        // left Title
        if(leftTitle!=null) leftTitle.drawText(g2, leftX, 0);

        // right Title
        int rightTitleWidth= 0;
        if(rightTitle!=null) rightTitleWidth = rightTitle.drawText(g2,rightX-5,1);

        // middle Title
        if(middleTitle!=null) middleTitle.drawText(g2,rightX-rightTitleWidth-20,1);

        g2.setFont(displayTextFont);
        g2.setPaint(Color.white);
        int y =yOffSet+boxHeight-g2.getFontMetrics().getHeight();

		String viewName = guiControl.selectorPane.getViewName();
		int sWidth=0;
		if(viewName!=null) {
        	g2.drawString(viewName,leftX,y);
        	sWidth = g2.getFontMetrics().stringWidth(viewName);
			}

//        if(optionTitle!=null) optionTitle.drawCenteredText(g2,maxWidth/2-20,yOffSet+boxHeight);
        if(optionTitle!=null) g2.drawString(optionTitle.title,leftX+20+sWidth,y);
    }


    public void setDisplayOption(int displayOption) {
        this.displayOption=displayOption;
        guiControl.colorScalePane.setColorScale();
        setDisplayColors();
        refreshDisplay();
        guiControl.tablesDisplayPane.listTestResults();
        }
   public int getDisplayOption() {
        return displayOption;
        }
   public void resetDisplayOption() {
        displayOption=0;
        }
   public ModuleCell getLastClickedCell() {
        return lastClickedCell;
        }
	public int getLastClickedRodSlot() {
		return lastClickedRodSlot;
		}
    public void setLastClickedCell(String sn) {
    		lastClickedCell = ConfigurationInterface.getInstance().getModuleCell(sn);
		}

    public void setDisplayColors() {
//       System.out.println("**setDisplayColors**");
       TestSelection testSelection = guiControl.tablesDisplayPane.getTestSelection();
       IVTestSelection ivTestSelection = guiControl.tablesDisplayPane.getIVTestSelection();
       int testIndex = (testSelection!=null) ? testSelection.getTestIndex() : -1;

       int displayStatus = guiControl.displayStatus.getStatus();
       Map countMap = null;
       if(displayStatus==DisplayStatus.IS_MONITOR && testSelection!=null) countMap = guiControl.isInterface.isCounter.getModuleMap(guiControl.selectorPane.getDisplayParameterIndex(),testSelection.getRunScanRegex(),testSelection.isRetrievedData());
       Map dcsMap = null;
//       if(displayStatus==DisplayStatus.DCS_DATA && guiControl.tablesDisplayPane.getTestControlObjectName()!=null) dcsMap=guiControl.isInterface.getDCSMap(guiControl.tablesDisplayPane.getTestControlObjectName());
       if(guiControl.tablesDisplayPane.getTestControlObjectName()!=null) dcsMap=guiControl.isInterface.getDCSMap(guiControl.tablesDisplayPane.getTestControlObjectName());
       int parameterIndex = guiControl.selectorPane.getDisplayParameterIndex();
       int mouseOption = guiControl.selectorPane.getMouseClickIndex();

       SummaryReader.DCSInfo dcsInfo;

	   config = ConfigurationInterface.getInstance();

// enumerate through all modules
       	MODULELOOP:
        for (java.util.Iterator m = config.getSerialNumberList(guiControl.selectorPane.getViewCategoryIndex(),guiControl.selectorPane.getViewIndex()).iterator(); m.hasNext(); ) {
            String sn=(String)m.next();
            ModuleCell thisCell = config.getModuleCell(sn);
           	Double theData=null;
           	String header=null;
           	switch(displayStatus) {

             	case DisplayStatus.TEST_DATA:
                  	if(testSelection!=null) header = testSelection.getResultObjectHeader();
                  	if(header==null) {
                    	fillCellData(thisCell,null);
                       	continue MODULELOOP;
                       	}
                   	dcsInfo = (dcsMap!=null) ? (SummaryReader.DCSInfo)dcsMap.get(sn) : null;
                   	switch(displayOption) {
                     	case 1: // SCTDB Data only
                        	header = testSelection.getSCTDBObjectHeader();
                        	if((theData = guiControl.isInterface.getSCTDBData(header+sn,testIndex))==null) {
                            	fillCellData(thisCell,null);
                            	continue MODULELOOP;
                            	}
                        	break;
                     	case 2: // Compare SCTDB and ROD data
                         	if((theData=guiControl.isInterface.getTestData(header,sn,testIndex,dcsInfo))==null) {
                             	fillCellData(thisCell,null);
                             	continue MODULELOOP;
                             	}
                          	Double sctdbData;
                          	if((sctdbData = guiControl.isInterface.getSCTDBData(testSelection.getSCTDBObjectHeader()+sn,testIndex))==null) {
                             	fillCellData(thisCell,null);
                             	continue MODULELOOP;
                             	}
                          	double diff = Math.abs(theData.doubleValue()-sctdbData.doubleValue());
                          	if(testIndex==TEST_FULLBYPASS && parameterIndex>2) theData=new Double(1.0);
                          	else theData = (diff>testDataTolerances[testIndex][parameterIndex]) ? new Double(2.0) : new Double(1.0);
                          	break;
                   		case 0: // ROD Data only
                   		default:
                         	if((theData=guiControl.isInterface.getTestData(header,sn,testIndex,dcsInfo))==null) {
                            	fillCellData(thisCell,null);
                            	continue MODULELOOP;
                            	}
                    	}

					break;

             	case DisplayStatus.IV_DATA:
                 	if(ivTestSelection==null) {
                      	fillCellData(thisCell,null);
                      	continue MODULELOOP;
                      	}
                 	header = ivTestSelection.getResultObjectHeader();
                 	if(header==null || (theData = guiControl.isInterface.getTestData(header,thisCell.getSerialNo(),TEST_IV,null))==null) {
                       	fillCellData(thisCell,null);
                       	continue MODULELOOP;
                       	}
                 	break;

             	case DisplayStatus.SCTDB_DATA:
                 	if(testSelection==null) {
                      	fillCellData(thisCell,null);
                      	continue MODULELOOP;
                      	}
                 	header = testSelection.getSCTDBObjectHeader();
                 	if(header==null || (theData = guiControl.isInterface.getSCTDBData(header+thisCell.getSerialNo(),testIndex))==null) {
                       	fillCellData(thisCell,null);
                       	continue MODULELOOP;
                       	}
                	break;

             	case DisplayStatus.SCTDB_RODDAQ_DATA:
                 	if(testSelection==null) {
                      	fillCellData(thisCell,null);
                      	continue MODULELOOP;
                      	}
                 	header = testSelection.getSCTDBRodDaqObjectHeader();
                 	if(header==null || (theData = guiControl.isInterface.getSCTDBData(header+thisCell.getSerialNo(),testIndex))==null) {
                       	fillCellData(thisCell,null);
                       	continue MODULELOOP;
                       	}
                	break;

             	case DisplayStatus.DCS_DATA:
                	if((theData=guiControl.isInterface.getDCSData(sn,parameterIndex))==null) {
                        fillCellData(thisCell,null);
                        continue MODULELOOP;
                        }
                	break;

             	case DisplayStatus.PROBE_DATA:
	          		theData = SctApiInfo.getInstance().getProbeData(sn,mouseOption);
                	break;

             	case DisplayStatus.IS_MONITOR:
                	if(testSelection==null || countMap==null) {
                    	fillCellData(thisCell,null);
                        continue MODULELOOP;
                        }
                    theData = (countMap.containsKey(sn)) ? new Double(((java.util.List)countMap.get(sn)).size()) : new Double(0.);
                	break;

             	case DisplayStatus.MODULE_GROUP_SELECTION:
				if(parameterIndex>0) theData = SctApiInfo.getInstance().getProbeData(sn,mouseOption);
                  	else theData = new Double(thisCell.getModuleGroup());
             	default:
                  	fillCellData(thisCell,null);
           		}
          	fillCellData(thisCell,theData);
          	}
// now define titles
		int displayedTestIndex = guiControl.isInterface.getListFilterIndex();
        int selectedTestIndex = (testSelection!=null) ? testSelection.getTestIndex() : -1;
        int runno = (testSelection!=null) ? testSelection.getRunNo() : -1;
        int scanno = (testSelection!=null) ? testSelection.getScanNo() : -1;
        String statusString="No Data";
        String runScanString="No Data";
        String dataDescription = guiControl.selectorPane.getDisplayParameter();
        String option = guiControl.selectorPane.getOption();

	leftTitle = new DisplayTitle(DisplayStatus.statusNames[displayStatus],java.awt.Color.yellow,displayTitleFont);
	middleTitle = (dataDescription!=null) ? new DisplayTitle(dataDescription, java.awt.Color.white,displayTitleFont) : null;
	rightTitle=null;
      optionTitle=(option!=null) ? new DisplayTitle(option,java.awt.Color.white,displayTextFont) : null;

        switch(displayStatus) {
        	case DisplayStatus.TEST_DATA:
              	switch(displayOption) {
                	case 0:
                     	rightTitle = (runno!=-1) ? new DisplayTitle("Run "+Integer.toString(runno)+" Scan "+Integer.toString(scanno),java.awt.Color.yellow,displayTitleFont) : new DisplayTitle("No Data",java.awt.Color.red,displayTitleFont);
                    	break;
                	case 1:
                     	rightTitle = new DisplayTitle("SctDaq Reference",java.awt.Color.red,displayTitleFont);
                     	break;
                	case 2:
                     	rightTitle = new DisplayTitle("SctDaq Comparison",java.awt.Color.red,displayTitleFont);
                	default:
                	}
              	break;
            case DisplayStatus.IV_DATA:
              	rightTitle = (ivTestSelection!=null) ? new DisplayTitle("Run "+Integer.toString(ivTestSelection.getRunNo())+" Scan "+Integer.toString(ivTestSelection.getScanNo()),java.awt.Color.yellow,displayTitleFont) :  new DisplayTitle("No Data",java.awt.Color.red,displayTitleFont);
              	leftTitle = new DisplayTitle("IV Scans",java.awt.Color.yellow,displayTitleFont);
              	middleTitle = (ivTestSelection!=null) ? new DisplayTitle(guiControl.selectorPane.getIVParameter(),java.awt.Color.white,displayTitleFont) : null;
              	break;
            case DisplayStatus.SCTDB_DATA:
              	statusString="SCTDAQ Reference Data";
              	if(selectedTestIndex>=0) statusString=testMenuNames[selectedTestIndex];
              	else if(displayedTestIndex>=0) statusString=testMenuNames[displayedTestIndex];
              	leftTitle = new DisplayTitle(statusString,java.awt.Color.yellow,displayTitleFont);
              	rightTitle= (runno!=-1) ? new DisplayTitle("SCTDAQ Reference",java.awt.Color.red,displayTitleFont) : new DisplayTitle("No Data",java.awt.Color.red,displayTitleFont);
              	break;
            case DisplayStatus.SCTDB_RODDAQ_DATA:
              	rightTitle = (runno!=-1) ? new DisplayTitle("Run "+Integer.toString(runno)+" Scan "+Integer.toString(scanno),java.awt.Color.yellow,displayTitleFont) : new DisplayTitle("No Data",java.awt.Color.red,displayTitleFont);
              	break;
            case DisplayStatus.PROBE_DATA:
              	leftTitle= new DisplayTitle("Module Probe Data",java.awt.Color.yellow,displayTitleFont);
              	middleTitle = null;
              	break;
            case DisplayStatus.IS_MONITOR:
              	rightTitle = (runno!=-1) ? new DisplayTitle("Run "+Integer.toString(runno)+" Scan "+Integer.toString(scanno),java.awt.Color.yellow,displayTitleFont) : new DisplayTitle("No Data",java.awt.Color.red,displayTitleFont);
              	break;
            case DisplayStatus.MODULE_GROUP_SELECTION:
              	if(parameterIndex==0) middleTitle= (option!=null) ? new DisplayTitle(option+" Count : "+Integer.toString(guiControl.tablesDisplayPane.resultsTable.getRowCount()),java.awt.Color.white,displayTitleFont) : null;
              	break;
            default:
            }

   }

    private void fillCellData(ModuleCell thisCell, Double theData) {
         if(theData==null) {
            thisCell.setData(null,java.awt.Color.white);
            return;
            }
         ColorScale thisColorScale;
         if((thisColorScale = guiControl.colorScalePane.getColorScale())!=null) thisCell.setData(theData,thisColorScale.getColor(theData));
         else thisCell.setData(null,java.awt.Color.white);
         }






 
}
