/*
 * runStatus.java
 *
 * Created on 27 June 2003, 16:47
 */

package DisplayGUI;

/**
 *
 * @author  robinson
 */
public class DisplayStatus {
    
    public static final int UNDEFINED = 0;
    public static final int CONFIGURED = 1;
    public static final int MODULE_GROUP_SELECTION = 1;
    public static final int PROBE_DATA=2;
    public static final int IS_MONITOR = 3;
    public static final int TEST_DATA = 4;
    public static final int SCTDB_DATA = 5;
    public static final int SCTDB_RODDAQ_DATA=6;
    public static final int DCS_DATA = 7;
    public static final int IV_DATA=8;

    public static final String[] statusNames = {"DISPLAY UNCONFIGURED","Module Configuration","Probe Data","Data Manager","Test Results","SCTDAQ Reference","SctRodDaq Reference","DCS Conditions","IVscan Data"};

    public int status;
    public int configuredStatus;
    gui guiControl;
    
    /** Creates a new instance of runStatus */
    public DisplayStatus(gui parent) {
        guiControl=parent;
        status = UNDEFINED;
        configuredStatus=UNDEFINED;
    }
    public void setStatus(int status) {
        this.status=status;
    }
    public int getStatus() {
        return status;
    }

    public void setConfigurationMode() {

// by default, everything in module group 0 to start with
          ConfigurationInterface conf = ConfigurationInterface.getInstance();
          conf.getConfiguration(0);
          if(conf.isConfigured()) {
            guiControl.displayPane.refreshDisplay();
            configuredStatus=CONFIGURED;
            setModuleGroupSelectionMode();
            guiControl.menuActions.enableDisplayMenuItems(true);
	    
	    if(guiControl.menuActions.enableAutoUpload) guiControl.isInterface.uploadAssemblyInfo();


            }         
          }

    public void setModuleGroupSelectionMode() {
        unsubscribeAll();
        status=MODULE_GROUP_SELECTION;
        boolean displayWriteAccess=true; 
//        guiControl.displayPane.setWriteAccess(displayWriteAccess);
        showParameterSelector();
        // reread BOC config, otherwise it may contain probe data!
	  if(guiControl.selectorPane.getDisplayParameterIndex()>0) SctApiInfo.getInstance().probeConfig(guiControl.selectorPane.getViewCategoryIndex(),guiControl.selectorPane.getViewIndex(),guiControl.selectorPane.getDisplayParameterIndex());
        guiControl.colorScalePane.setColorScale();
        guiControl.tablesDisplayPane.updateTables();
//        System.out.println("DisplayStatus:: setDisplayColors");
        guiControl.displayPane.setDisplayColors();
        guiControl.displayPane.refreshDisplay();
        guiControl.menuActions.updateMenuStates();
    }
    public void setISMode() {
        unsubscribeAll();
        status = IS_MONITOR;
        showParameterSelector();
        guiControl.colorScalePane.setColorScale();
//        guiControl.isInterface.setServerName(0);  // set Event Data to start with
        guiControl.isInterface.subscribe();
//        guiControl.tablesDisplayPane.updateTables();
        guiControl.menuActions.updateMenuStates();
//        guiControl.displayPane.setDisplayColors();
//        guiControl.displayPane.refreshDisplay();
    }
    public void setTestDataMode(int testDataType) {
        if(status==TEST_DATA && testDataType==guiControl.isInterface.getListFilterIndex()) return;
        unsubscribeAll();
        int oldStatus=status;
        status = TEST_DATA;
        showParameterSelector();
        guiControl.displayPane.resetDisplayOption();
        guiControl.colorScalePane.setColorScale();
        guiControl.isInterface.subscribe(testDataType);
        if(testDataType==-1) guiControl.guiMenus.resetTestListFilter(); // set test filter to 'All tests'
        guiControl.menuActions.updateMenuStates();
    }
    public void setSCTDBDataMode(int testDataType) {
        if(status==SCTDB_DATA && testDataType==guiControl.isInterface.getListFilterIndex()) return;
        unsubscribeAll();
        int oldStatus=status;
        status = SCTDB_DATA;
        showParameterSelector();
        guiControl.colorScalePane.setColorScale();
        guiControl.isInterface.subscribe(testDataType);
        if(testDataType==-1) guiControl.guiMenus.resetTestListFilter(); // set test filter to 'All tests'
        guiControl.menuActions.updateMenuStates();
    }
    public void setSCTDBRodDaqMode(int testDataType) {
        if(status==SCTDB_RODDAQ_DATA && testDataType==guiControl.isInterface.getListFilterIndex()) return;
        unsubscribeAll();
        int oldStatus=status;
        status = SCTDB_RODDAQ_DATA;
        showParameterSelector();
        guiControl.colorScalePane.setColorScale();
        guiControl.isInterface.subscribe(testDataType);
        if(testDataType==-1) guiControl.guiMenus.resetTestListFilter(); // set test filter to 'All tests'
        guiControl.menuActions.updateMenuStates();
    }
    public void setDCSDataMode() {
        unsubscribeAll();
        int oldStatus=status;
        status = DCS_DATA;
        showParameterSelector();
        guiControl.colorScalePane.setColorScale();
        guiControl.isInterface.subscribe();
        guiControl.menuActions.updateMenuStates();
    }
    public void setIVDataMode() {
        unsubscribeAll();
        int oldStatus=status;
        status = IV_DATA;
        showParameterSelector();
        guiControl.colorScalePane.setColorScale();
        guiControl.isInterface.subscribe();
        guiControl.menuActions.updateMenuStates();

// if not IV scans already present, prompt the user to import some!
        IVTestSelection iv = guiControl.tablesDisplayPane.getIVTestSelection();
        if(iv==null) new IVScanLister(ConfigurationInterface.getInstance().getSerialNumberList(guiControl.selectorPane.getViewCategoryIndex(),guiControl.selectorPane.getViewIndex())).setVisible(true);
    }
    public void setProbeDataMode() {
        unsubscribeAll();
        int oldStatus=status;
        status = PROBE_DATA;
	  // probe the modules by default DANGEROUS!!
	  // SctApiInfo.getInstance().probe(guiControl.selectorPane.getViewCategoryIndex(),guiControl.selectorPane.getViewIndex());
        showParameterSelector();
        guiControl.colorScalePane.setColorScale();
        guiControl.menuActions.updateMenuStates();
        guiControl.displayPane.setDisplayColors();
        guiControl.displayPane.refreshDisplay();
	  guiControl.tablesDisplayPane.updateTables();
    }

    public void unsubscribeAll() {
        guiControl.isInterface.unsubscribe();
        }
    private void showParameterSelector() {
        guiControl.selectorPane.initParamList();
        }
}
