/*
 * GuiMenus class
 * This class defines the standard (static) gui menus and
 * the methods to access them
 *
 * @author robinson
 *
 */
package DisplayGUI;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JMenu;

import Preferences.*;
import guiUtilities.ChoiceDialog;
//
import Sct.Version;
import GuiComponents.System.*;
import GuiComponents.scripting.*;
import GuiComponents.Console.JConsole;
import Sct_SctApi.*;

/** Menu bar for GUI */
public class GuiMenus extends JMenuBar implements displayParams,TestDataInfo {
    gui guiControl;

	private DisplayChoiceToolbar displayChoiceToolbar = null;
	private javax.swing.JFrame hardwareWindow = null;
	private ServiceStatusFrame serviceStatusFrame=null;
	private ModuleCommentPrompt moduleCommentPrompt = null;
    protected ISTree isTree;
	private ManualConfiguration manualConfiguration;

    private javax.swing.JCheckBox[] testFilterButtons = new javax.swing.JCheckBox[testMenuNames.length+1];
    public boolean autoSelectMode = false;
    private JMenuItem autoSelectModeItem = new JMenuItem();
	private TestsMenu[] testsMenu = new TestsMenu[2];
	private JMenuItem[] testDataOptions = new JMenuItem[3];

	public static final int PANEL_PROBE=0;
	public static final int MODULE_BOC=1;
	public static final int MODULE_PROBE=2;
	public static final int PANEL_DISPLAY=3;
	public static final int MENUBAR_DISPLAY=4;
	public static final int MENUBAR_SYSTEM=5;
	public static final int MODULE_DCS=6;
	public static final int PANEL_DCS=7;
	public static final int MODULE_RESULTS=8;
	public static final int MENUBAR_PREFERENCES=9;
	public static final int MENUBAR_HELP=10;
	public static final int MODULE_SCTDB=11;
	public static final int MODULE_DATA_MANAGER=12;
	public static final int MODULE_IVSCAN=13;
	public static final int PANEL_TESTMENU=14;
	public static final int MENUBAR_TESTMENU=15;
	public static final int MENUBAR_TOOLS=16;
	public static final int OPTIONS_DISPLAY=17;
	public static final int TEST_FILTER=18;
	public static final int OPTIONS_IVSCAN=19;
	public static final int OPTIONS_CONFIG=20;
	public static final int PANEL_TESTDATA=21;

	private int N_MENUS=22;

	private JMenu[] guiMenu;

	public GuiMenus(gui guiControl) {
		this.guiControl = guiControl;

		guiMenu = new JMenu[N_MENUS];
		for(int imenu=0;imenu<N_MENUS;imenu++) {
			switch(imenu) {
				case PANEL_PROBE:
					guiMenu[imenu]=getProbeMenu();
					break;
				case MODULE_PROBE:
					guiMenu[imenu]=getModuleProbeMenu();
					break;
				case MODULE_BOC:
					guiMenu[imenu]=getModuleBOCMenu();
					break;
				case MENUBAR_DISPLAY:
					guiMenu[imenu]=getDisplayMenu();
       				guiMenu[imenu].addSeparator();
       				JMenuItem menuItem = new JMenuItem("Toolbar ...");
       				menuItem.setFont(menuFont);
       				menuItem.addActionListener(new java.awt.event.ActionListener() {
            			public void actionPerformed(java.awt.event.ActionEvent evt) {
							showToolBar();
							}
             			});
       				guiMenu[imenu].add(menuItem);
					guiMenu[imenu].setForeground(menuColor);
					break;
				case PANEL_DISPLAY:
					guiMenu[imenu]=getDisplayMenu();
					guiMenu[imenu].setText("Select Display");
					break;
				case MENUBAR_SYSTEM:
					guiMenu[imenu]=getSystemMenu();
					break;
				case MODULE_DCS:
					guiMenu[imenu]=getDDCModuleMenu();
					break;
				case PANEL_DCS:
					guiMenu[imenu]=getDCSPanelMenu();
					break;
				case MODULE_RESULTS:
					guiMenu[imenu]=getModuleResultMenu();
					break;
				case MENUBAR_PREFERENCES:
					guiMenu[imenu]=getPreferencesMenu();
					break;
				case MENUBAR_HELP:
					guiMenu[imenu]=getSctHelpMenu();
					guiMenu[imenu].setForeground(menuColor);
					break;
				case MODULE_SCTDB:
					guiMenu[imenu]=getModuleSCTDBMenu();
					break;
				case MODULE_DATA_MANAGER:
					guiMenu[imenu]=getModuleDataManagerMenu();
					break;
				case MODULE_IVSCAN:
					guiMenu[imenu]=getIVScanMenu();
					break;
				case PANEL_TESTMENU:
					guiMenu[imenu]=getTestMenu(0);
					guiMenu[imenu].setText("Invoke Tests");
					break;
				case MENUBAR_TESTMENU:
					guiMenu[imenu]=getTestMenu(1);
					guiMenu[imenu].setForeground(menuColor);
					break;
				case MENUBAR_TOOLS:
					guiMenu[imenu]=getToolsMenu();
					break;
				case OPTIONS_DISPLAY:
					guiMenu[imenu]=getDisplayOptionMenu();
					break;
				case TEST_FILTER:
					guiMenu[imenu]=getTestFilterMenu();
					break;
				case OPTIONS_IVSCAN:
					guiMenu[imenu]=getIVScanOptionMenu();
					break;
				case OPTIONS_CONFIG:
					guiMenu[imenu]=getConfigsOptionMenu();
					break;
				case PANEL_TESTDATA:
					guiMenu[imenu]=getPanelTestMenu();
					break;
				default:
				}
			guiMenu[imenu].setFont(menuFont);
			guiMenu[imenu].setBackground(backgroundColor);
            // guiMenu[imenu].setForeground(menuColor);
			}
		}

	public JMenu getMenu(int menu) {
		return guiMenu[menu];
		}
	public javax.swing.JPopupMenu getPopupMenu(int menu) {
		return guiMenu[menu].getPopupMenu();
		}

	//**************************************************
   private JMenu getPanelTestMenu() {
       JMenu thisMenu = new JMenu("Test Results");
       testDataOptions[0] = new JMenuItem("ROD Data");
       testDataOptions[0].setFont(menuFont);
       testDataOptions[0].addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                    guiControl.displayPane.setDisplayOption(0);
                     }
             });
       testDataOptions[1] = new JMenuItem("SCTDAQ Reference Data");
       testDataOptions[1].setFont(menuFont);
       testDataOptions[1].addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                    guiControl.displayPane.setDisplayOption(1);
                     }
             });

       testDataOptions[2] = new JMenuItem("Comparison ROD-SCTDAQ");
       testDataOptions[2].setFont(menuFont);
       testDataOptions[2].addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                    guiControl.displayPane.setDisplayOption(2);
                     }
             });
       thisMenu.add(testDataOptions[0]);
       thisMenu.add(testDataOptions[1]);
       thisMenu.add(testDataOptions[2]);

       thisMenu.addSeparator();

       JMenuItem item = new JMenuItem("Summary List...");
       item.setFont(menuFont);
       item.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {

                  TestSelection testSelection = guiControl.tablesDisplayPane.getTestSelection();
                  IVTestSelection ivTestSelection = guiControl.tablesDisplayPane.getIVTestSelection();
                  int testIndex = (testSelection!=null) ? testSelection.getTestIndex() : -1;
		  if(testIndex==-1) {
		    javax.swing.JOptionPane.showMessageDialog(null,"No test is currently selected.");
		    return;
		    }
		  int parameterIndex = guiControl.selectorPane.getDisplayParameterIndex();
                  if(parameterIndex<0) {
		  javax.swing.JOptionPane.showMessageDialog(null,"No test parameter is currently selected.");
		    return;
		    }

		  String header = testSelection.getResultObjectHeader();

		  java.util.Vector theTable = new java.util.Vector();
		  java.util.Vector theLine = new java.util.Vector();
		  theLine.addElement("Serial No");
		  if(parameterIndex>3) for(int i=4;i<chipOptions.length;i++) theLine.addElement(chipOptions[i]);
		  else theLine.addElement("Data");
		  theTable.addElement(theLine);

          java.util.Map dcsMap = null;
//       if(displayStatus==DisplayStatus.DCS_DATA && guiControl.tablesDisplayPane.getTestControlObjectName()!=null) dcsMap=guiControl.isInterface.getDCSMap(guiControl.tablesDisplayPane.getTestControlObjectName());
          if(guiControl.tablesDisplayPane.getTestControlObjectName()!=null) dcsMap=guiControl.isInterface.getDCSMap(guiControl.tablesDisplayPane.getTestControlObjectName());

// enumerate through all modules

        for (java.util.Iterator m = ConfigurationInterface.getInstance().getSerialNumberList(guiControl.selectorPane.getViewCategoryIndex(),guiControl.selectorPane.getViewIndex()).iterator(); m.hasNext(); ) {
               String sn=(String)m.next();
               SummaryReader.DCSInfo dcsInfo = (dcsMap!=null) ? (SummaryReader.DCSInfo)dcsMap.get(sn) : null;
               theLine = guiControl.isInterface.getTestDataLine(header,sn,testIndex,dcsInfo);
		    if(theLine!=null) theTable.addElement(theLine);
		    }
                   String param = guiControl.selectorPane.getDisplayParameter();
                   param+=" Data for "+testNames[testSelection.getTestIndex()]+" Run "+Integer.toString(testSelection.getRunNo())+" Scan "+Integer.toString(testSelection.getScanNo());
                   guiUtilities.SpreadsheetViewer.getInstance().update(param,theTable);

                     }
             });
      thisMenu.add(item);

       item = new JMenuItem("SCTDAQ Reference Comparison Report");
       item.setFont(menuFont);
       item.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                     guiControl.menuActions.makeComparisonReport(true);
                     }
             });
      thisMenu.add(item);

       item = new JMenuItem("SctRodDaq Reference Comparison Report");
       item.setFont(menuFont);
      item.addActionListener(new java.awt.event.ActionListener() {
               public void actionPerformed(java.awt.event.ActionEvent evt) {
	          		guiControl.menuActions.makeComparisonReport(false);
                     }
            });
      thisMenu.add(item);


      thisMenu.addSeparator();


       item = new JMenuItem("Upload to SCT Database");
       item.setFont(menuFont);
       item.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                    TestSelection t = guiControl.tablesDisplayPane.getTestSelection();
                    if(t==null) {
                      javax.swing.JOptionPane.showMessageDialog(null,"There is no Data to upload!");
                      return;
                      }
                    int response = javax.swing.JOptionPane.showConfirmDialog(null,"Are you sure want to upload the data for Run/Scan "+t.getRunNo()+"/"+t.getScanNo()+"?");
                    if(response!=javax.swing.JOptionPane.YES_OPTION) return;
                    UploadManager upload = new UploadManager(guiControl);
                     }
             });
       thisMenu.add(item);

       return thisMenu;
       }


	private JMenu getConfigsOptionMenu() {
	   JMenu configMenu = new JMenu("Configuration");
       JMenuItem item = new JMenuItem("Get Configuration");
       item.setFont(menuFont);
       configMenu.add(item);
       item.addActionListener(new java.awt.event.ActionListener() {
      		public void actionPerformed(java.awt.event.ActionEvent evt) {
                    ConfigurationInterface.getInstance().Refresh();
                    guiControl.displayStatus.setConfigurationMode();
                    }
             });
	   item= new JMenuItem("Manual Configuration...");
	   item.setFont(menuFont);
	   configMenu.add(item);
	   item.addActionListener(new java.awt.event.ActionListener() {
       		public void actionPerformed(java.awt.event.ActionEvent evt) {
                   guiControl.displayStatus.setModuleGroupSelectionMode();
                   if(manualConfiguration==null) manualConfiguration = new ManualConfiguration(guiControl);
                   else manualConfiguration.showFrame();
                   }
             });

		 return configMenu;
		 }

	private JMenu getIVScanOptionMenu() {
       JMenu thisMenu = new JMenu("IV scans");

       JMenuItem item = new JMenuItem("IV Scan Data Directory...");
       item.setFont(menuFont);
       thisMenu.add(item);
       item.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                   Preference preference = new Preference(guiControl,PreferencesInterface.IV_DIRECTORY);
                   }
              });


       item = new JMenuItem("Import IV Scans...");
       item.setFont(menuFont);
       thisMenu.add(item);
       item.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                   new IVScanLister(ConfigurationInterface.getInstance().getSerialNumberList(guiControl.selectorPane.getViewCategoryIndex(),guiControl.selectorPane.getViewIndex())).setVisible(true);
                   }
              });

       item = new JMenuItem("Plot Selected IVScans");
       item.setFont(menuFont);
       thisMenu.add(item);
       item.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                   guiControl.menuActions.plotIVScanData(null);
                   }
              });

       item = new JMenuItem("Upload to SCT Database");
       item.setFont(menuFont);
       thisMenu.add(item);
       item.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                    IVTestSelection i = guiControl.tablesDisplayPane.getIVTestSelection();
                    if(i==null) {
                      javax.swing.JOptionPane.showMessageDialog(null,"There is no IV scan Data to upload!");
                      return;
                      }
                    int response = javax.swing.JOptionPane.showConfirmDialog(null,"Are you sure want to upload the IV scans for Run/Scan "+i.getRunNo()+"/"+i.getScanNo()+"?");
                    if(response!=javax.swing.JOptionPane.YES_OPTION) return;
                    UploadManager upload = new UploadManager(guiControl);
                   }
              });

       return thisMenu;
       }

   private JMenu getTestFilterMenu() {
       JMenu thisMenu = new JMenu("Test Filter");

       JMenuItem item = new JMenuItem("Add/Edit Run Comments...");
       item.setFont(menuFont);
       item.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                         TestSelection testSelection = guiControl.tablesDisplayPane.getTestSelection();
                         if(testSelection==null) {
                             javax.swing.JOptionPane.showMessageDialog(null,"No test selected.");
                             return;
                             }
                         BookKeeper.getInstance().showCommentPrompt(testSelection.getRunNo(), testSelection.getScanNo(), null);
                 }
             });
       thisMenu.add(item);

       item = new JMenuItem("Add/Edit Test Comments...");
       item.setFont(menuFont);
       item.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                         TestSelection testSelection = guiControl.tablesDisplayPane.getTestSelection();
                         if(testSelection==null) {
                             javax.swing.JOptionPane.showMessageDialog(null,"No test selected.");
                             return;
                             }
                         BookKeeper.getInstance().showCommentPrompt(testSelection.getRunNo(), testSelection.getScanNo(), testSelection.getTestName());
                 }
             });
       thisMenu.add(item);

       thisMenu.addSeparator();

       javax.swing.ButtonGroup thisGroup = new javax.swing.ButtonGroup();
       testFilterButtons[0] = new javax.swing.JCheckBox("All tests");
       thisGroup.add(testFilterButtons[0]);
       testFilterButtons[0].setFont(menuFont);
       testFilterButtons[0].addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                   switch(guiControl.displayStatus.getStatus()) {
                   case DisplayStatus.TEST_DATA:
                     guiControl.displayStatus.setTestDataMode(-1);      
                     break;
                   case DisplayStatus.SCTDB_DATA:
                    guiControl.displayStatus.setSCTDBDataMode(-1);
                     break;
                   case DisplayStatus.SCTDB_RODDAQ_DATA:
                    guiControl.displayStatus.setSCTDBRodDaqMode(-1); 
                     break;
                   default:
                   }
                 }
              });
        thisMenu.add(testFilterButtons[0]);
       for(int i=1;i<testFilterButtons.length;i++) {
         if(!testIsDefined[i-1]) continue;
         testFilterButtons[i] = new javax.swing.JCheckBox(testNames[i-1]);
         thisGroup.add(testFilterButtons[i]);
         testFilterButtons[i].setFont(menuFont);
         thisMenu.add(testFilterButtons[i]);
         }
       testFilterButtons[0].setSelected(true);

       for(int i=1;i<testFilterButtons.length;i++) {
         if(!testIsDefined[i-1]) continue;
         switch(i-1) {
          case TEST_FULLBYPASS:
             testFilterButtons[i].addActionListener(new java.awt.event.ActionListener() {
               public void actionPerformed(java.awt.event.ActionEvent evt) {
                   switch(guiControl.displayStatus.getStatus()) {
                   case DisplayStatus.TEST_DATA:
                     guiControl.displayStatus.setTestDataMode(TEST_FULLBYPASS);
                     break;
                   case DisplayStatus.SCTDB_DATA:
                    guiControl.displayStatus.setSCTDBDataMode(TEST_FULLBYPASS);
                     break;
                   case DisplayStatus.SCTDB_RODDAQ_DATA:
                     guiControl.displayStatus.setSCTDBRodDaqMode(TEST_FULLBYPASS);
                     break;
                   default:
                   }
                   }
              });
            break;
          case TEST_PIPELINE:
             testFilterButtons[i].addActionListener(new java.awt.event.ActionListener() {
               public void actionPerformed(java.awt.event.ActionEvent evt) {
                   switch(guiControl.displayStatus.getStatus()) {
                   case DisplayStatus.TEST_DATA:
                     guiControl.displayStatus.setTestDataMode(TEST_PIPELINE);
                     break;
                   case DisplayStatus.SCTDB_DATA:
                    guiControl.displayStatus.setSCTDBDataMode(TEST_PIPELINE);
                     break;
                   case DisplayStatus.SCTDB_RODDAQ_DATA:
                     guiControl.displayStatus.setSCTDBRodDaqMode(TEST_PIPELINE);
                     break;
                   default:
                   }
                   }
              });
            break;
          case TEST_STROBEDELAY:
             testFilterButtons[i].addActionListener(new java.awt.event.ActionListener() {
               public void actionPerformed(java.awt.event.ActionEvent evt) {
                   switch(guiControl.displayStatus.getStatus()) {
                   case DisplayStatus.TEST_DATA:
                     guiControl.displayStatus.setTestDataMode(TEST_STROBEDELAY);
                     break;
                   case DisplayStatus.SCTDB_DATA:
                    guiControl.displayStatus.setSCTDBDataMode(TEST_STROBEDELAY);
                     break;
                   case DisplayStatus.SCTDB_RODDAQ_DATA:
                     guiControl.displayStatus.setSCTDBRodDaqMode(TEST_STROBEDELAY);
                     break;
                   default:
                   }
                   }
              });
            break;
          case TEST_3PTGAIN:
             testFilterButtons[i].addActionListener(new java.awt.event.ActionListener() {
               public void actionPerformed(java.awt.event.ActionEvent evt) {
                   switch(guiControl.displayStatus.getStatus()) {
                   case DisplayStatus.TEST_DATA:
                     guiControl.displayStatus.setTestDataMode(TEST_3PTGAIN);
                     break;
                   case DisplayStatus.SCTDB_DATA:
                    guiControl.displayStatus.setSCTDBDataMode(TEST_3PTGAIN);
                     break;
                   case DisplayStatus.SCTDB_RODDAQ_DATA:
                     guiControl.displayStatus.setSCTDBRodDaqMode(TEST_3PTGAIN);
                     break;
                   default:
                   }
                   }
              });
            break;
          case TEST_TRIM:
             testFilterButtons[i].addActionListener(new java.awt.event.ActionListener() {
               public void actionPerformed(java.awt.event.ActionEvent evt) {
                   switch(guiControl.displayStatus.getStatus()) {
                   case DisplayStatus.TEST_DATA:
                     guiControl.displayStatus.setTestDataMode(TEST_TRIM);
                     break;
                   case DisplayStatus.SCTDB_DATA:
                    guiControl.displayStatus.setSCTDBDataMode(TEST_TRIM);
                     break;
                   case DisplayStatus.SCTDB_RODDAQ_DATA:
                     guiControl.displayStatus.setSCTDBRodDaqMode(TEST_TRIM);
                     break;
                   default:
                   }
                   }
              });
            break;
          case TEST_NPTGAIN:
             testFilterButtons[i].addActionListener(new java.awt.event.ActionListener() {
               public void actionPerformed(java.awt.event.ActionEvent evt) {
                   switch(guiControl.displayStatus.getStatus()) {
                   case DisplayStatus.TEST_DATA:
                     guiControl.displayStatus.setTestDataMode(TEST_NPTGAIN);
                     break;
                   case DisplayStatus.SCTDB_DATA:
                    guiControl.displayStatus.setSCTDBDataMode(TEST_NPTGAIN);
                     break;
                   case DisplayStatus.SCTDB_RODDAQ_DATA:
                     guiControl.displayStatus.setSCTDBRodDaqMode(TEST_NPTGAIN);
                     break;
                   default:
                   }
                   }
              });
            break;
          case TEST_NOISE:
             testFilterButtons[i].addActionListener(new java.awt.event.ActionListener() {
               public void actionPerformed(java.awt.event.ActionEvent evt) {
                   switch(guiControl.displayStatus.getStatus()) {
                   case DisplayStatus.TEST_DATA:
                     guiControl.displayStatus.setTestDataMode(TEST_NOISE);
                     break;
                   case DisplayStatus.SCTDB_DATA:
                    guiControl.displayStatus.setSCTDBDataMode(TEST_NOISE); 
                     break;
                   case DisplayStatus.SCTDB_RODDAQ_DATA:
                     guiControl.displayStatus.setSCTDBRodDaqMode(TEST_NOISE);
                     break;
                   default:
                   }
                   }
              });
            break;
          case TEST_TIMEWALK:
             testFilterButtons[i].addActionListener(new java.awt.event.ActionListener() {
               public void actionPerformed(java.awt.event.ActionEvent evt) {
                   switch(guiControl.displayStatus.getStatus()) {
                   case DisplayStatus.TEST_DATA:
                     guiControl.displayStatus.setTestDataMode(TEST_TIMEWALK);
                     break;
                   case DisplayStatus.SCTDB_DATA:
                    guiControl.displayStatus.setSCTDBDataMode(TEST_TIMEWALK);
                     break;
                   case DisplayStatus.SCTDB_RODDAQ_DATA:
                     guiControl.displayStatus.setSCTDBRodDaqMode(TEST_TIMEWALK);
                     break;
                   default:
                   }
                   }
              });
            break;
          case TEST_NMASK:
             testFilterButtons[i].addActionListener(new java.awt.event.ActionListener() {
               public void actionPerformed(java.awt.event.ActionEvent evt) {
                   switch(guiControl.displayStatus.getStatus()) {
                   case DisplayStatus.TEST_DATA:
                     guiControl.displayStatus.setTestDataMode(TEST_NMASK);
                     break;
                   case DisplayStatus.SCTDB_DATA:
                    guiControl.displayStatus.setSCTDBDataMode(TEST_NMASK);
                     break;
                   case DisplayStatus.SCTDB_RODDAQ_DATA:
                     guiControl.displayStatus.setSCTDBRodDaqMode(TEST_NMASK);
                     break;
                   default:
                   }
                   }
              });
            break;
	     case TEST_RXTHRESHOLD:
             testFilterButtons[i].addActionListener(new java.awt.event.ActionListener() {
               public void actionPerformed(java.awt.event.ActionEvent evt) {
                   switch(guiControl.displayStatus.getStatus()) {
                   case DisplayStatus.TEST_DATA:
                     guiControl.displayStatus.setTestDataMode(TEST_RXTHRESHOLD);
                     break;
                   case DisplayStatus.SCTDB_DATA:
                    guiControl.displayStatus.setSCTDBDataMode(TEST_RXTHRESHOLD);
                     break;
                   case DisplayStatus.SCTDB_RODDAQ_DATA:
                     guiControl.displayStatus.setSCTDBRodDaqMode(TEST_RXTHRESHOLD);
                     break;
                   default:
                   }
                   }
              });
            break;
	     case TEST_RXTHRESHOLDBASEDONCONFIGREGISTER:
             testFilterButtons[i].addActionListener(new java.awt.event.ActionListener() {
               public void actionPerformed(java.awt.event.ActionEvent evt) {
                   switch(guiControl.displayStatus.getStatus()) {
                   case DisplayStatus.TEST_DATA:
                     guiControl.displayStatus.setTestDataMode(TEST_RXTHRESHOLDBASEDONCONFIGREGISTER);
                     break;
                   case DisplayStatus.SCTDB_DATA:
                    guiControl.displayStatus.setSCTDBDataMode(TEST_RXTHRESHOLDBASEDONCONFIGREGISTER);
                     break;
                   case DisplayStatus.SCTDB_RODDAQ_DATA:
                     guiControl.displayStatus.setSCTDBRodDaqMode(TEST_RXTHRESHOLDBASEDONCONFIGREGISTER);
                     break;
                   default:
                   }
                   }
              });
            break;
	     case TEST_RXDELAY:
             testFilterButtons[i].addActionListener(new java.awt.event.ActionListener() {
               public void actionPerformed(java.awt.event.ActionEvent evt) {
                   switch(guiControl.displayStatus.getStatus()) {
                   case DisplayStatus.TEST_DATA:
                     guiControl.displayStatus.setTestDataMode(TEST_RXDELAY);
                     break;
                   case DisplayStatus.SCTDB_DATA:
                    guiControl.displayStatus.setSCTDBDataMode(TEST_RXDELAY);
                     break;
                   case DisplayStatus.SCTDB_RODDAQ_DATA:
                     guiControl.displayStatus.setSCTDBRodDaqMode(TEST_RXDELAY);
                     break;
                   default:
                   }
	           }
              });
            break;
            default:
            }
          }
      return thisMenu;
      }

   public void resetTestListFilter() {
      testFilterButtons[0].setSelected(true);
      }

   private void setAutoSelectItem() {
   	autoSelectModeItem.setText(autoSelectMode ? "Disable test auto-select (currently enabled)" : "Enable test auto-select (currently disabled)");
	}

    private JMenu getDisplayOptionMenu() {
       JMenu thisMenu = new JMenu("Display");

       JMenuItem item = new JMenuItem("Clear Result Cache");
       item.setFont(menuFont);
       item.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                    System.out.println("SctGUI: Clearing cache (was "+guiControl.isInterface.cache.size()+" objects deep)");
                    guiControl.isInterface.cache.clear();
                 }
             });
       thisMenu.add(item);

       item = new JMenuItem("Reset all Color Scales");
       item.setFont(menuFont);
       item.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                   int response = javax.swing.JOptionPane.showConfirmDialog(null,"This will permanently reset all color scales to their defaults. Continue?","Confirm",javax.swing.JOptionPane.YES_NO_OPTION,javax.swing.JOptionPane.INFORMATION_MESSAGE);
                   if(response==javax.swing.JOptionPane.YES_OPTION) {
                      ColorScaleManager.getInstance().clearAll();
                      guiControl.colorScalePane.setColorScale();
                      guiControl.displayPane.setDisplayColors();
                      guiControl.displayPane.refreshDisplay();
                      }

                 }
             });
       thisMenu.add(item);

	 autoSelectModeItem = new JMenuItem();
       autoSelectModeItem.setFont(menuFont);
       autoSelectMode = PreferencesInterface.getInstance().getPreference(PreferencesInterface.AUTO_TEST_SELECT).equals("TRUE");
       setAutoSelectItem();
	 thisMenu.add(autoSelectModeItem);
       autoSelectModeItem.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                    autoSelectMode = !autoSelectMode;
			  setAutoSelectItem();
                    PreferencesInterface.getInstance().setPreference(PreferencesInterface.AUTO_TEST_SELECT,autoSelectMode ? "TRUE" : "FALSE");
                 }
             });

       return thisMenu;
       }


	private JMenu getToolsMenu() {
       JMenu menu = new JMenu("Tools");
       menu.setBackground(backgroundColor);
       menu.setForeground(menuColor);
        JMenuItem item = new JMenuItem("IS Tree...");
       item.setFont(menuFont);
       item.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
				if(isTree==null) isTree = new ISTree(guiControl,guiControl.isInterface.getISTree());
            	else isTree.setTree(guiControl.isInterface.getISTree());
                         }
              });
       menu.add(item);

       JMenu menu2 = new JMenu("Scripting");
	   menu2.setFont(menuFont);
	   menu2.setBackground(backgroundColor);
      // menu2.setForeground(menuColor);
       /*scriptJavaScriptItem = new JMenuItem("JavaScript");
       scriptJavaScriptItem.setFont(menuFont);
       scriptJavaScriptItem.addActionListener(this);
       menu2.add(scriptJavaScriptItem);*/
       item = new JMenuItem("BeanShell");
       item.setFont(menuFont);
       item.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
       			guiControl.menuActions.addNewConsole(ScriptingInterface.instance().getBeanShellInterpreter(), "BeanShell");
                         }
              });
	   menu2.add(item);
       item = new JMenuItem("Jython");
       item.setFont(menuFont);
       item.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
       		guiControl.menuActions.addNewConsole(ScriptingInterface.instance().getJythonInterpreter(), "Jython");
                         }
              });
       menu2.add(item);
       menu.add(menu2);

       item = new JMenuItem("Run GarbageCollector");
       item.setFont(menuFont);
       item.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
               Runtime r = Runtime.getRuntime();
               long freeMem = r.freeMemory();
               long maxMem = r.maxMemory();
               long totalMem = r.totalMemory();
               System.out.println("Total Mem: " + totalMem/1024. + ", Max Mem:" + maxMem/1024. + " , Free Mem: " + freeMem/1024.);
               r.gc();
               try {
                 Thread.sleep(500);
               } catch (InterruptedException ie) {}
               r.gc();
               freeMem = r.freeMemory();
               maxMem = r.maxMemory();
               totalMem = r.totalMemory();
               System.out.println("Total Mem: " + totalMem/1024. + ", Max Mem:" + maxMem/1024. + " , Free Mem: " + freeMem/1024.  );
                   }
              });
       menu.add(item);
       item = new JMenuItem("Refresh Display");
       item.setFont(menuFont);
       item.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
               guiControl.displayPane.setDisplayColors();
               guiControl.displayPane.refreshDisplay();
                   }
              });
       menu.add(item);

       item = new JMenuItem("Launch SctApi GUI");
       item.setFont(menuFont);
       item.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                javax.swing.JFrame frame = new javax.swing.JFrame("SctApi GUI");
                frame.getContentPane().add(new GuiComponents.Panels.SctApiGui());

                frame.pack();
                frame.setVisible(true);
            }
           });
       menu.add(item);
	return menu;
	}

	private JMenu getTestMenu(int mindex) {

       JMenu thisMenu = new JMenu("Tests");
	   testsMenu[mindex] = new TestsMenu(guiControl);
       thisMenu.add(testsMenu[mindex].getMenu(TestsMenu.BOCTESTS_MENU));
       thisMenu.add(testsMenu[mindex].getMenu(TestsMenu.DIGITALTESTS_MENU));
	   thisMenu.add(testsMenu[mindex].getMenu(TestsMenu.ANALOGUETESTS_MENU));
       thisMenu.add(testsMenu[mindex].getMenu(TestsMenu.TIMINGTESTS_MENU));
       thisMenu.add(testsMenu[mindex].getMenu(TestsMenu.QUICKTESTS_MENU));
       thisMenu.add(testsMenu[mindex].getMenu(TestsMenu.ALLTESTS_MENU));
       thisMenu.add(testsMenu[mindex].getMenu(TestsMenu.SEQUENCES_MENU));
 //      thisMenu.addSeparator();
 //      thisMenu.add(getSCTDAQMenu());
       return thisMenu;
       }

	private JMenu getIVScanMenu() {
       JMenu ivScanMenu = new JMenu("IVScan");
       JMenuItem item1 = new JMenuItem("Test Report");
       ivScanMenu.add(item1);
       item1.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
//                System.out.println("Selected "+popupCell.getSerialNo());
                   IVTestSelection ivTestSelection = guiControl.tablesDisplayPane.getIVTestSelection();
                   if(ivTestSelection==null) {
                       javax.swing.JOptionPane.showMessageDialog(null,"No data selected.");
                       return;
                       }
                   SummaryReader.Test testResult = (SummaryReader.Test) guiControl.isInterface.getTest(ivTestSelection.getResultObjectHeader(),guiControl.displayPane.getLastClickedCell().getSerialNo(),null);
                   if(testResult!=null) SummaryReader.ReaderReportUtilities.showIVTestReport(guiControl.displayPane.getLastClickedCell().getSerialNo(),testResult);
                   else javax.swing.JOptionPane.showMessageDialog(null,"No data for "+guiControl.displayPane.getLastClickedCell().getSerialNo());
              }
          });

       item1 = new JMenuItem("Plot IVScan");
       item1.setFont(menuFont);
       ivScanMenu.add(item1);
       item1.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                   guiControl.menuActions.plotIVScanData(guiControl.displayPane.getLastClickedCell().getSerialNo());
              }
          });

       item1 = new JMenuItem("Dump test result");
       item1.setFont(menuFont);
       ivScanMenu.add(item1);
       item1.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                     IVTestSelection ivTestSelection = guiControl.tablesDisplayPane.getIVTestSelection();
                     if(ivTestSelection==null)  {
                       javax.swing.JOptionPane.showMessageDialog(null,"No data selected.");
                       return;
                       }
                     String resultObject=ivTestSelection.getResultObjectHeader()+guiControl.displayPane.getLastClickedCell().getSerialNo();
                     if(!Sct.IS.SctNames.getISRepository().contains(resultObject)) {
                           javax.swing.JOptionPane.showMessageDialog(null,"No data for "+guiControl.displayPane.getLastClickedCell().getSerialNo());
                           return;
                           }
                     TestSummaryIS result = new TestSummaryIS();
                     Sct.IS.SctNames.getISRepository().getValue(resultObject, result); 
                     java.util.Vector thisData = new java.util.Vector();
                     thisData.addElement(guiControl.displayPane.getLastClickedCell().getSerialNo()+".txt");
                     thisData.addElement(result.dataString);
                     guiUtilities.RawDataViewer.getInstance().showNewData(thisData,"Result String for "+resultObject);
                     }

          });
       item1 = new JMenuItem("Full SCTDB Module Report");
       item1.setFont(menuFont);
       ivScanMenu.add(item1);
       item1.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
             String sn = guiControl.displayPane.getLastClickedCell().getSerialNo();
             ProdDatabase.ReportDownloader d = new ProdDatabase.ReportDownloader(guiUtilities.HTMLViewer.MODULESUMMARY,sn,0);
             if(d.isOkToStart()) d.start();
             }

          });
       item1 = new JMenuItem("Edit/Add Module Comments...");
       item1.setFont(menuFont);
       item1.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                showModuleCommentPrompt(guiControl.displayPane.getLastClickedCell().getSerialNo());
                     }

          });
       ivScanMenu.add(item1);
	return ivScanMenu;

       }



	//**************************************************************************
    private JMenu getModuleDataManagerMenu() {
       JMenu dataManagerSNMenu = new JMenu("Data Manager");

       JMenuItem item = new JMenuItem("Plot Selected Data");
       item.setFont(menuFont);
       item.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                    switch(guiControl.displayStatus.getStatus()) {
                       case DisplayStatus.IS_MONITOR:
                          String objectname;
                          if((objectname = guiControl.tablesDisplayPane.getSelectedObject())!=null) guiControl.isInterface.launchExternalViewer(objectname);
                          else javax.swing.JOptionPane.showMessageDialog(null,"No plot data available.");
                        break;
                    default:
                     }
                 }
             });
       dataManagerSNMenu.add(item);

       item = new JMenuItem("Plot Selected Data (with Options)");
       item.setFont(menuFont);
       item.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                    switch(guiControl.displayStatus.getStatus()) {
                       case DisplayStatus.IS_MONITOR:
                          String objectname;
                          if((objectname = guiControl.tablesDisplayPane.getSelectedObject())!=null) guiControl.isInterface.launchExternalViewerWithOptions(objectname);
                          else javax.swing.JOptionPane.showMessageDialog(null,"No plot data available.");
                        break;
                    default:
                     }
                 }
             });
       dataManagerSNMenu.add(item);



       item = new JMenuItem("View Selected IS Object");
       item.setFont(menuFont);
       item.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                    switch(guiControl.displayStatus.getStatus()) {
                       case DisplayStatus.IS_MONITOR:
                          String objectname;
                          if((objectname = guiControl.tablesDisplayPane.getSelectedObject())!=null) guiControl.isInterface.viewISData(objectname);
                          else javax.swing.JOptionPane.showMessageDialog(null,"No data available.");
                        break;
                    default:
                     }
                 }
             });

       dataManagerSNMenu.add(item);
       item = new JMenuItem("View Module Configuration");
       item.setFont(menuFont);
       item.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
            javax.swing.JFrame frame2 = new javax.swing.JFrame("Module Configuration");
            frame2.getContentPane().add(new GuiComponents.Panels.ModuleConfigurationPanel(guiControl.displayPane.getLastClickedCell().getSerialNo()));
            frame2.pack();
            frame2.show();
                     }

          });
      dataManagerSNMenu.add(item);

      item = new JMenuItem("Edit/Add Module Comments...");
       item.setFont(menuFont);
       item.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                showModuleCommentPrompt(guiControl.displayPane.getLastClickedCell().getSerialNo());
                     }

          });

       dataManagerSNMenu.add(item);
	   return dataManagerSNMenu;
       }

	private JMenu getModuleSCTDBMenu() {
       JMenu sctdbDataSNMenu = new JMenu("Data Options");
       sctdbDataSNMenu.setFont(menuFont);

       JMenuItem item1 = new JMenuItem("Test Report");
       item1.setFont(menuFont);
       sctdbDataSNMenu.add(item1);
       item1.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
               TestSelection testSelection = guiControl.tablesDisplayPane.getTestSelection();
               if(testSelection==null)  {
                       javax.swing.JOptionPane.showMessageDialog(null,"No data selected.");
                       return;
                       }
               String resultObject= (testSelection.isSCTDBRodDaqData()) ? testSelection.getSCTDBRodDaqObjectHeader()+guiControl.displayPane.getLastClickedCell().getSerialNo() : testSelection.getSCTDBObjectHeader()+guiControl.displayPane.getLastClickedCell().getSerialNo();
               if(!Sct.IS.SctNames.getISRepository().contains(resultObject)) {
                  javax.swing.JOptionPane.showMessageDialog(null,"No data for "+guiControl.displayPane.getLastClickedCell().getSerialNo());
                  return;
                  }
               SCTDB_Data result = new SCTDB_Data();
               Sct.IS.SctNames.getISRepository().getValue(resultObject, result);
               ProdDatabase.ReportDownloader d = new ProdDatabase.ReportDownloader(guiUtilities.HTMLViewer.SCTDAQTESTDATA, result.testNumber, 0);
               if(d.isOkToStart()) d.start();
             }

          });
       item1 = new JMenuItem("Full Module Report");
       item1.setFont(menuFont);
       sctdbDataSNMenu.add(item1);
       item1.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
             String sn = guiControl.displayPane.getLastClickedCell().getSerialNo();
             ProdDatabase.ReportDownloader d = new ProdDatabase.ReportDownloader(guiUtilities.HTMLViewer.MODULESUMMARY,sn,0);
             if(d.isOkToStart()) d.start();
             }

          });

       return sctdbDataSNMenu;
       }

	private JMenu getSctHelpMenu() {
	   JMenu menu = new JMenu("Help");

       JMenuItem helpItem = new JMenuItem("View User Manual");
       helpItem.setFont(menuFont);
       menu.add(helpItem);
       helpItem.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                   guiControl.htmlViewer.show("http://www.hep.phy.cam.ac.uk/~robinson/DisplayGUI.html","Online manual");
                   }
              });

       JMenuItem aboutItem = new JMenuItem("About");
       aboutItem.setFont(menuFont);
       menu.add(aboutItem);
       aboutItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                String v = Version.instance.getAllInformation();
                String s = "SctGui is the user interface for end users.  See the manual for more information";
                s += "\nSctGui Author: Dave Robinson.";
                s += "\nSctDaqRod Authors: Alan Barr, Bruce Gallop, Matthew Palmer, Dave Robinson and Alexander Tricoli";
                s += "\n\n" + v;
                javax.swing.JOptionPane.showMessageDialog(guiControl, s, "About SctGui", javax.swing.JOptionPane.INFORMATION_MESSAGE);
            }
           });
		return menu;
		}
    private JMenu getPreferencesMenu() {
	// Preferences menu
       PreferencesInterface prefs = PreferencesInterface.getInstance();
	   JMenuItem[] preferenceItem = new JMenuItem[prefs.getNoPreferences()];
       JMenu menu = new JMenu("Preferences");
       menu.setBackground(backgroundColor);
       menu.setFont(menuFont);
       menu.setForeground(menuColor);

       for(int i=0;i<prefs.getNoPreferences();i++) {
         switch(i) {
         case PreferencesInterface.SCT_VIEW:
         preferenceItem[i] = new JMenuItem(prefs.getDescription(i));
         preferenceItem[i].setFont(menuFont);
         menu.add(preferenceItem[i]);
             preferenceItem[i].addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                   Preference preference = new Preference(guiControl,PreferencesInterface.SCT_VIEW);
                   guiControl.selectorPane.setSCTView(preference.getValue());
                   }
              });
              break;
         case PreferencesInterface.INSTITUTE:
         preferenceItem[i] = new JMenuItem(prefs.getDescription(i));
         preferenceItem[i].setFont(menuFont);
         menu.add(preferenceItem[i]);
             preferenceItem[i].addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                   Preference preference = new Preference(guiControl,PreferencesInterface.INSTITUTE);
                   }
              });
              break;
         case PreferencesInterface.PASSWORD:
         preferenceItem[i] = new JMenuItem(prefs.getDescription(i));
         preferenceItem[i].setFont(menuFont);
         menu.add(preferenceItem[i]);
             preferenceItem[i].addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                   Preference preference = new Preference(guiControl,PreferencesInterface.PASSWORD);
                   }
              });
              break;
         case PreferencesInterface.PSVIEWER:
         preferenceItem[i] = new JMenuItem(prefs.getDescription(i));
         preferenceItem[i].setFont(menuFont);
         menu.add(preferenceItem[i]);
             preferenceItem[i].addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                   Preference preference = new Preference(guiControl,PreferencesInterface.PSVIEWER);                   
                   }
              });
              break;
         case PreferencesInterface.SCRATCH_DIR:
         preferenceItem[i] = new JMenuItem(prefs.getDescription(i));
         preferenceItem[i].setFont(menuFont);
         menu.add(preferenceItem[i]);
             preferenceItem[i].addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                   Preference preference = new Preference(guiControl,PreferencesInterface.SCRATCH_DIR);
                   }
              });
              break;
         case PreferencesInterface.UPLOAD_DIR:
         preferenceItem[i] = new JMenuItem(prefs.getDescription(i));
         preferenceItem[i].setFont(menuFont);
         menu.add(preferenceItem[i]);
             preferenceItem[i].addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                   Preference preference = new Preference(guiControl,PreferencesInterface.UPLOAD_DIR);                   
                   }
              });
              break;
         case PreferencesInterface.WEB_LOOKUP_DIR:
         preferenceItem[i] = new JMenuItem(prefs.getDescription(i));
         preferenceItem[i].setFont(menuFont);
         menu.add(preferenceItem[i]);
             preferenceItem[i].addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                   Preference preference = new Preference(guiControl,PreferencesInterface.WEB_LOOKUP_DIR);
                   }
              });
              break;
         default:
         }
		}
		return menu;
    }

	private JMenu getModuleResultMenu() {
       JMenu testDataMenu = new JMenu("Test Data Options");
       testDataMenu.setFont(menuFont);

       JMenuItem item1 = new JMenuItem("Test Report");
       item1.setFont(menuFont);
       testDataMenu.add(item1);
       item1.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
//                System.out.println("Selected "+popupCell.getSerialNo());
                   TestSelection testSelection = guiControl.tablesDisplayPane.getTestSelection();
                   if(testSelection==null) {
                       javax.swing.JOptionPane.showMessageDialog(null,"No data selected.");
                       return;
                       }
                   java.util.Map dcsMap = null;
                   if(guiControl.tablesDisplayPane.getTestControlObjectName()!=null) dcsMap=guiControl.isInterface.getDCSMap(guiControl.tablesDisplayPane.getTestControlObjectName());
                   SummaryReader.DCSInfo dcsInfo = (dcsMap!=null) ? (SummaryReader.DCSInfo) dcsMap.get(guiControl.displayPane.getLastClickedCell().getSerialNo()) : null;
                   SummaryReader.Test testResult = (SummaryReader.Test) guiControl.isInterface.getTest(testSelection.getResultObjectHeader(),guiControl.displayPane.getLastClickedCell().getSerialNo(), dcsInfo);
                   if(testResult!=null) SummaryReader.ReaderReportUtilities.showTestReport(guiControl.displayPane.getLastClickedCell().getSerialNo(),testResult);
                   else javax.swing.JOptionPane.showMessageDialog(null,"No data for this module");
              }
          });
       item1 = new JMenuItem("Analysis Plots");
       item1.setFont(menuFont);
       testDataMenu.add(item1);
       item1.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                  TestSelection testSelection = guiControl.tablesDisplayPane.getTestSelection();
                  if(testSelection==null) {
                       javax.swing.JOptionPane.showMessageDialog(null,"No data selected.");
                       return;
                       }
                  String objectname;
                  if((objectname = testSelection.getTestObjectHeader())!=null) {
                     guiControl.isInterface.launchExternalViewer(objectname+guiControl.displayPane.getLastClickedCell().getSerialNo());
                     }
                 }
          });
       item1 = new JMenuItem("Raw Data Plots");
       item1.setFont(menuFont);
       testDataMenu.add(item1);
       item1.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                  TestSelection testSelection = guiControl.tablesDisplayPane.getTestSelection();
                  if(testSelection==null) {
                       javax.swing.JOptionPane.showMessageDialog(null,"No data selected.");
                       return;
                       }
		     java.util.Vector oVector = guiControl.isInterface.getDataObjectList(guiControl.displayPane.getLastClickedCell().getSerialNo(), 0);
		     if(oVector==null || oVector.size()==0) {
                       javax.swing.JOptionPane.showMessageDialog(null,"No raw data objects available for plotting.");
                       return;
                       }
		     String objectname = null;
		     if(oVector.size()==1) objectname=(String)oVector.elementAt(0);
		     else {
		       guiUtilities.ChoiceDialog cd = new guiUtilities.ChoiceDialog(guiControl,"Select Raw Data object",100,16,oVector.toArray(),false);
		       cd.pack();
                       cd.setLocationRelativeTo(guiControl);
                       cd.setVisible(true);
		       Object[] objectnames = cd.getSelectedObjects();
		       if(objectnames==null || objectnames.length!=1) return;
		       objectname = (String)objectnames[0];
		       }
		     if(objectname==null) return;
                     guiControl.isInterface.launchExternalViewer(objectname);
                     }
          });
       item1 = new JMenuItem("Fitted Data Plots");
       item1.setFont(menuFont);
       testDataMenu.add(item1);
       item1.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                 TestSelection testSelection = guiControl.tablesDisplayPane.getTestSelection();
                  if(testSelection==null) {
                       javax.swing.JOptionPane.showMessageDialog(null,"No data selected.");
                       return;
                       }
		     java.util.Vector oVector = guiControl.isInterface.getDataObjectList(guiControl.displayPane.getLastClickedCell().getSerialNo(), 1);
		     if(oVector==null || oVector.size()==0) {
                       javax.swing.JOptionPane.showMessageDialog(null,"No fit data objects available for plotting.");
                       return;
                       }
		     String objectname = null;
		     if(oVector.size()==1) objectname=(String)oVector.elementAt(0);
		     else {
		       guiUtilities.ChoiceDialog cd = new guiUtilities.ChoiceDialog(guiControl,"Select Fit Data object",100,16,oVector.toArray(),false);
		       cd.pack();
                       cd.setLocationRelativeTo(guiControl);
                       cd.setVisible(true);
		       Object[] objectnames = cd.getSelectedObjects();
		       if(objectnames==null || objectnames.length!=1) return;
		       objectname = (String)objectnames[0];
		       }
		     if(objectname==null) return;
                     guiControl.isInterface.launchExternalViewer(objectname);
	     

                     }
          });
       item1 = new JMenuItem("Analysis Plots (with Options)");
       item1.setFont(menuFont);
       testDataMenu.add(item1);
       item1.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                  TestSelection testSelection = guiControl.tablesDisplayPane.getTestSelection();
                  if(testSelection==null) {
                       javax.swing.JOptionPane.showMessageDialog(null,"No data selected.");
                       return;
                       }
                  String objectname;
                  if((objectname = testSelection.getTestObjectHeader())!=null) {
                     guiControl.isInterface.launchExternalViewerWithOptions(objectname+guiControl.displayPane.getLastClickedCell().getSerialNo());
                     }
                 }
          });


       item1 = new JMenuItem("Module Configuration");
       item1.setFont(menuFont);
       testDataMenu.add(item1);
       item1.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
            javax.swing.JFrame frame2 = new javax.swing.JFrame("Module Configuration");
            frame2.getContentPane().add(new GuiComponents.Panels.ModuleConfigurationPanel(guiControl.displayPane.getLastClickedCell().getSerialNo()));
            frame2.pack();
            frame2.show();
                     }

          });
       item1 = new JMenuItem("Dump test result");
       item1.setFont(menuFont);
       testDataMenu.add(item1);
       item1.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                     TestSelection testSelection = guiControl.tablesDisplayPane.getTestSelection();
                     if(testSelection==null)  {
                       javax.swing.JOptionPane.showMessageDialog(null,"No data selected.");
                       return;
                       }
                     String resultObject=testSelection.getResultObjectHeader()+guiControl.displayPane.getLastClickedCell().getSerialNo();
                     if(!Sct.IS.SctNames.getISRepository().contains(resultObject)) {
                           javax.swing.JOptionPane.showMessageDialog(null,"No result for "+guiControl.displayPane.getLastClickedCell().getSerialNo());
                           return;
                           }
                     TestSummaryIS result = new TestSummaryIS();
                     Sct.IS.SctNames.getISRepository().getValue(resultObject, result);
                     java.util.Vector thisData = new java.util.Vector();
                     thisData.addElement(guiControl.displayPane.getLastClickedCell().getSerialNo()+".txt");
                     thisData.addElement(result.dataString);
                     guiUtilities.RawDataViewer.getInstance().showNewData(thisData,"Result String for "+resultObject);
                     }

          });
       item1 = new JMenuItem("Full SCTDB Module Report");
       item1.setFont(menuFont);
       testDataMenu.add(item1);
       item1.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
             String sn = guiControl.displayPane.getLastClickedCell().getSerialNo();
             ProdDatabase.ReportDownloader d = new ProdDatabase.ReportDownloader(guiUtilities.HTMLViewer.MODULESUMMARY,sn,0);
             if(d.isOkToStart()) d.start();
             }

          });
       item1 = new JMenuItem("Edit/Add Module Comments...");
       item1.setFont(menuFont);
       item1.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                showModuleCommentPrompt(guiControl.displayPane.getLastClickedCell().getSerialNo());
                     }

          });
       testDataMenu.add(item1);
	   return testDataMenu;

       }

	private JMenu getDCSPanelMenu() {
       JMenu thisMenu = new JMenu("DCS");

       thisMenu.setFont(menuFont);

       JMenuItem item = new JMenuItem("IS Server Name...");
       item.setFont(menuFont);
       thisMenu.add(item);
       item.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                   Preference preference = new Preference(guiControl,PreferencesInterface.DCS_SERVER_NAME);
                   }
              });

       item = new JMenuItem("LV/HV Control for all modules...");
       item.setFont(menuFont);
       thisMenu.add(item);
       item.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
		  DDCModuleSwitcher ddcModuleSwitcher = new DDCModuleSwitcher(ConfigurationInterface.getInstance().getSerialNumberList(guiControl.selectorPane.getViewCategoryIndex(),guiControl.selectorPane.getViewIndex()));
                  ddcModuleSwitcher.setLocationRelativeTo(guiControl);
		  }
              });


       return thisMenu;
       }

	private JMenu getDDCModuleMenu() {
      JMenu ddcModuleMenu = new JMenu("DDC Commands");
      ddcModuleMenu.setFont(menuFont);

      JMenuItem item1 = new JMenuItem("LV/HV Control...");
      item1.setFont(menuFont);
      ddcModuleMenu.add(item1);
      item1.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
	          String sn = guiControl.displayPane.getLastClickedCell().getSerialNo();
                  DDCModuleSwitcher ddcModuleSwitcher = new DDCModuleSwitcher(ConfigurationInterface.getInstance().getSerialNumberList(guiControl.selectorPane.getViewCategoryIndex(),guiControl.selectorPane.getViewIndex()),sn);
                  ddcModuleSwitcher.setLocationRelativeTo(guiControl);
	      }
          });
	return ddcModuleMenu;

     }
	private JMenu getSystemMenu() {
	   JMenu systemMenu = new JMenu("System");
       systemMenu.setBackground(backgroundColor);
       systemMenu.setFont(menuFont);
       systemMenu.setForeground(menuColor);

       JMenuItem menuItem = new JMenuItem("Refresh");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                    SystemInterface.getInstance().refresh();
                    guiControl.displayPane.refreshDisplay();
                     }
             });
       systemMenu.add(menuItem);

       menuItem = new JMenuItem("Hardware Configuration...");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
            if(hardwareWindow==null) {
                hardwareWindow = new javax.swing.JFrame("Hardware Configuration");
                hardwareWindow.getContentPane().add(new HardwareConfigurationPanel());
                hardwareWindow.pack();
                java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
                java.awt.Dimension WSize = hardwareWindow.getSize();
                hardwareWindow.setLocation((screenSize.width-(int)WSize.getWidth())/2,(screenSize.height-(int)WSize.getHeight())/2);
                hardwareWindow.addWindowListener(new java.awt.event.WindowAdapter() {
                            public void windowClosing(java.awt.event.WindowEvent e) {
                                  hardwareWindow=null;
//                                e.getWindow().dispose();
                            }
                        });
                }
            hardwareWindow.setVisible(true);
              }
          });
       systemMenu.add(menuItem);

       menuItem = new JMenuItem("Service Status...");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
       public void actionPerformed(java.awt.event.ActionEvent evt) {
            if(serviceStatusFrame==null) serviceStatusFrame = new ServiceStatusFrame();
	    else serviceStatusFrame.refresh();
              }
          });
       systemMenu.add(menuItem);

       systemMenu.addSeparator();
       menuItem = new JMenuItem("Exit SctGUI");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                    guiControl.exitForm(null);                   
                     }
             });
       systemMenu.add(menuItem);
	   return systemMenu;
	   }

    private JMenu getDisplayMenu() {
       JMenu menu = new JMenu("Display");
       menu.setBackground(backgroundColor);
       menu.setFont(menuFont);
       menu.setForeground(menuColor);

       JMenuItem item;
       for(int i=1;i<DisplayStatus.statusNames.length;i++) {
          item = new JMenuItem(DisplayStatus.statusNames[i]);
          item.setFont(menuFont);
          switch(i) {
             case DisplayStatus.MODULE_GROUP_SELECTION:  // ModuleGroup Selection
             item.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                   guiControl.displayStatus.setModuleGroupSelectionMode();
                   updateToolBar(DisplayStatus.MODULE_GROUP_SELECTION);
                     }
             });
             break;
             case DisplayStatus.IS_MONITOR:  // Data Manager
             item.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                   guiControl.displayStatus.setISMode();
                   updateToolBar(DisplayStatus.IS_MONITOR);
                     }
             });
             break;
             case DisplayStatus.TEST_DATA:   // Test Results
             item.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                    guiControl.displayStatus.setTestDataMode(-1);   
                   updateToolBar(DisplayStatus.TEST_DATA);
                     }
             });
             break;
             case DisplayStatus.SCTDB_DATA:    // SCTDB Data
             item.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                     guiControl.displayStatus.setSCTDBDataMode(-1);
                     updateToolBar(DisplayStatus.SCTDB_DATA);
                     }
             });
             break;
             case DisplayStatus.DCS_DATA:    // DCS Data
             item.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                     guiControl.displayStatus.setDCSDataMode();
                     updateToolBar(DisplayStatus.DCS_DATA);
                     }
             });
             break;
            case DisplayStatus.SCTDB_RODDAQ_DATA:    // SctRodDaq Archive Data
             item.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                     guiControl.displayStatus.setSCTDBRodDaqMode(-1);
                     updateToolBar(DisplayStatus.SCTDB_RODDAQ_DATA);
                     }
             });
             break;
            case DisplayStatus.IV_DATA:    // SctRodDaq Archive Data
             item.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                    String ivdirectory = PreferencesInterface.getInstance().getPreference(PreferencesInterface.IV_DIRECTORY);
                    if(ivdirectory.equals(PreferencesInterface.unDefinedString)) javax.swing.JOptionPane.showMessageDialog(null,"You have not defined the IV data directory. You need to do this before importing IV scan data.");
                     guiControl.displayStatus.setIVDataMode();
                     updateToolBar(DisplayStatus.IV_DATA);
                     }
             });
             break;
            case DisplayStatus.PROBE_DATA:    // SctRodDaq Archive Data
             item.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
		     guiControl.displayStatus.setProbeDataMode();
                     updateToolBar(DisplayStatus.PROBE_DATA);
                     }
             });
             break;
             default:
             }
          menu.add(item);

          }
          return menu;
      }


   private JMenu getModuleProbeMenu() {
      JMenu probePanelMenu = new JMenu("DDC Commands");
      JMenuItem item1 = new JMenuItem("Hard Reset...");
      item1.setFont(menuFont);
      probePanelMenu.add(item1);
      item1.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
	          String sn = guiControl.displayPane.getLastClickedCell().getSerialNo();
		  ConfigurationInterface conf = ConfigurationInterface.getInstance();
		  String crate = conf.getSNInfo(sn,SNInfo.DCS_CRATE);
		  String channel = conf.getSNInfo(sn,SNInfo.DCS_CHANNEL);
		  int response = javax.swing.JOptionPane.showConfirmDialog(null,"Hard Reset "+sn+" (Crate "+crate+", Channel "+channel+")?","Confirm Hard Reset request",javax.swing.JOptionPane.OK_CANCEL_OPTION);
                  if(response!=javax.swing.JOptionPane.YES_OPTION) return;
		  try {
		    Sct_SctApi.SctApiDDCServer api = SystemInterface.getInstance().getSctApiDDCServer();
		    if(api!=null) api.ddcHardReset(Integer.valueOf(crate).intValue(), Integer.valueOf(channel).intValue());
		    else javax.swing.JOptionPane.showMessageDialog(null,"null SctApi pointer");
		    }catch(Exception e){javax.swing.JOptionPane.showMessageDialog(null,"Exception "+e.toString());}

              }
          });
		return probePanelMenu;
      }

    private JMenu getProbeMenu() {
       JMenu probeMenu = new JMenu("Actions");

       JMenuItem item = new JMenuItem("ReProbe modules");
       item.setFont(menuFont);
       probeMenu.add(item);
       item.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
		  try {
                 SctApiInfo.getInstance().probe(guiControl.selectorPane.getViewCategoryIndex(),guiControl.selectorPane.getViewIndex());
		     guiControl.displayPane.setDisplayColors();
		     guiControl.displayPane.refreshDisplay();
		     guiControl.tablesDisplayPane.updateTables();
		    }catch(Exception e){javax.swing.JOptionPane.showMessageDialog(null,"Exception "+e.toString());}
		  }
              });

       item = new JMenuItem("Probe & Fix");
       item.setFont(menuFont);
       probeMenu.add(item);
       item.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
//		  	int response = JOptionPane.showConfirmDialog(null,"Probe & Fix (probe, reset all modules not returning events, send configurations and reprobe)?","Confirm",javax.swing.JOptionPane.OK_CANCEL_OPTION);
//                  if(response!=javax.swing.JOptionPane.YES_OPTION) return;

			try {
		    	Sct_SctApi.SctApiDDCServer api = SystemInterface.getInstance().getSctApiDDCServer();
			if(api==null) {
				javax.swing.JOptionPane.showMessageDialog(null,"Null ref to SctApiDDCServer");
				return;
				}
			Sct_SctApi.SctApiIPC SctApi = SystemInterface.getInstance().getSctApi();
		  	if(SctApi==null) {
		    		javax.swing.JOptionPane.showMessageDialog(null,"null SctApi pointer");
		    		return;
		    		}
		  	ConfigurationInterface conf = ConfigurationInterface.getInstance();

			// probe modules and create list of modules not returning events
                 	SctApiInfo.getInstance().probe(guiControl.selectorPane.getViewCategoryIndex(),guiControl.selectorPane.getViewIndex(),true);

			int nresets = SctApiInfo.getInstance().getErrorModuleList().size();

			if(nresets==0) javax.swing.JOptionPane.showMessageDialog(null,"All modules are returning events.");
			if(nresets>0 && javax.swing.JOptionPane.showConfirmDialog(null,"You need to reset "+Integer.toString(nresets)+" modules. Continue?","Confirm",javax.swing.JOptionPane.OK_CANCEL_OPTION)==javax.swing.JOptionPane.YES_OPTION) {

				// reset any modules not returning events
				for (java.util.Iterator i = SctApiInfo.getInstance().getErrorModuleList().iterator(); i.hasNext(); ) {
               			String sn=(String)i.next();
		  			String crate = conf.getSNInfo(sn,SNInfo.DCS_CRATE);
		  			String channel = conf.getSNInfo(sn,SNInfo.DCS_CHANNEL);
					System.out.println("SctGUI: Sending hard reset to "+sn+" (DCS crate/channel "+crate+"/"+channel+")");
		    			api.ddcHardReset(Integer.valueOf(crate).intValue(), Integer.valueOf(channel).intValue());
					}

		    		// resend configurations
				System.out.println("SctGUI: Sending module configurations...");
              		SctApi.sendABCDModules(Sct_SctApi.BankType.PHYSICS_CONFIG);

				// now reprobe modules
                  	SctApiInfo.getInstance().probe(guiControl.selectorPane.getViewCategoryIndex(),guiControl.selectorPane.getViewIndex());

				}

		    // redisplay the probe data
		     	guiControl.displayPane.setDisplayColors();
		     	guiControl.displayPane.refreshDisplay();
		     	guiControl.tablesDisplayPane.updateTables();
		    }catch(Exception e){javax.swing.JOptionPane.showMessageDialog(null,"Exception "+e.toString());}
		    }
              });

       item = new JMenuItem("Check Modules...");
       item.setFont(menuFont);
       probeMenu.add(item);
       item.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                  SctApiInfo.getInstance().checkProbe();
		  }
              });

       probeMenu.addSeparator();

       item = new JMenuItem("Hard Reset all modules...");
       item.setFont(menuFont);
       probeMenu.add(item);
       item.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
	     	  int response = javax.swing.JOptionPane.showConfirmDialog(null,"Send hard reset to all modules?","Confirm",javax.swing.JOptionPane.OK_CANCEL_OPTION);
                  if(response!=javax.swing.JOptionPane.YES_OPTION) return;
		  try {
		    Sct_SctApi.SctApiDDCServer api = SystemInterface.getInstance().getSctApiDDCServer();
		    if(api==null) {
		       javax.swing.JOptionPane.showMessageDialog(null,"null SctApi pointer");
		       return;
		       }
		    ConfigurationInterface conf = ConfigurationInterface.getInstance();
		    java.util.Set crateSet = new java.util.HashSet();
        	for (java.util.Iterator m = conf.getSerialNumberList(guiControl.selectorPane.getViewCategoryIndex(),guiControl.selectorPane.getViewIndex()).iterator(); m.hasNext(); ) {
              	String sn=(String)m.next();
			    String crate = conf.getSNInfo(sn,SNInfo.DCS_CRATE);
			      if(crateSet.contains(crate)) continue;
			      System.out.println("sctGUI - sending hard reset to DCS crate "+crate);
                  api.ddcHardResetAll(Integer.valueOf(crate).intValue());
			      crateSet.add(crate);
			      }
                    System.out.println("Hard reset completed.");
		    }catch(Exception e){javax.swing.JOptionPane.showMessageDialog(null,"Exception "+e.toString());}
		  }
              });

       item = new JMenuItem("Send Module Configurations...");
       item.setFont(menuFont);
       probeMenu.add(item);
       item.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
	     	  int response = javax.swing.JOptionPane.showConfirmDialog(null,"Send configuration to all modules?","Confirm",javax.swing.JOptionPane.OK_CANCEL_OPTION);
              if(response!=javax.swing.JOptionPane.YES_OPTION) return;
	     	  Sct_SctApi.SctApiIPC api = SystemInterface.getInstance().getSctApi();
		  if(api==null) {
		    javax.swing.JOptionPane.showMessageDialog(null,"null SctApi pointer");
		    return;
		    }
              try {api.sendABCDModules(Sct_SctApi.BankType.PHYSICS_CONFIG);}
		  catch(SctApiException s) {javax.swing.JOptionPane.showMessageDialog(null, "Failed to Configure modules: " + s.detail);}

              }
              });

		return probeMenu;


       }

    private JMenu getModuleBOCMenu() {
       JMenu moduleConfigMenu = new JMenu("Module Configuration");

       JMenuItem item = new JMenuItem("Modify...");
       item.setFont(menuFont);
       moduleConfigMenu.add(item);
	item.addActionListener(new java.awt.event.ActionListener() {
	public void actionPerformed(java.awt.event.ActionEvent evt) {
		int configParam = guiControl.selectorPane.getDisplayParameterIndex();
		String configLabel = configParamLabels[configParam];
		String queryText = configParams[configParam];
		String sn = guiControl.displayPane.getLastClickedCell().getSerialNo();
		if(configParams_links[configParam]) {
			int option = guiControl.selectorPane.getMouseClickIndex();
			if(option>=0) {
				String lbl = Integer.toString(option);
				queryText += " (link "+lbl+")";
				configLabel += lbl;
                       }
			else {
				javax.swing.JOptionPane.showMessageDialog(null,"Invalid option?");
				return;
				}
			}
		queryText += " for module "+sn;

		Double data = guiControl.displayPane.getLastClickedCell().getData();
		if(data==null) {
			javax.swing.JOptionPane.showMessageDialog(null,"No data available for "+queryText);
			return;
			}
		int data2 = (int)data.doubleValue();
		String txt = javax.swing.JOptionPane.showInputDialog(null,"Modify "+queryText,Integer.toString(data2));
		if(txt==null) return;
		int value;
		try {
			value = Integer.parseInt(txt);
		}catch(Exception e) {
			javax.swing.JOptionPane.showMessageDialog(null,"Not a valid BOC parameter value.");
			return;
			}
		if(value<0) {
			javax.swing.JOptionPane.showMessageDialog(null,"Negative values not permitted.");
			return;
			}


		try {
			Sct_SctApi.SctApiIPC api = SystemInterface.getInstance().getSctApi();
		    	if(api==null) {
		    		javax.swing.JOptionPane.showMessageDialog(null,"null IPC ref to Sctapi");
				return;
				}
			ConfigurationInterface conf = ConfigurationInterface.getInstance();
			int partition = Integer.parseInt(conf.getSNInfo(sn,SNInfo.PARTITION));
			int crate = Integer.parseInt(conf.getSNInfo(sn,SNInfo.ROD_CRATE));
			int channel = Integer.parseInt(conf.getSNInfo(sn,SNInfo.ROD_CHANNEL));
			int rod = Integer.parseInt(conf.getSNInfo(sn,SNInfo.ROD_NUMBER));
			api.modifyBOCParam(partition, crate, rod, channel, Sct.ScanDescriptors.getInstance().valueFor(configLabel), value);
			// ok it updated, now refresh display
			SctApiInfo.getInstance().probeConfig(guiControl.selectorPane.getViewCategoryIndex(),guiControl.selectorPane.getViewIndex(),guiControl.selectorPane.getDisplayParameterIndex());
		     	guiControl.displayPane.setDisplayColors();
		     	guiControl.displayPane.refreshDisplay();
		     	guiControl.tablesDisplayPane.updateTables();
		}catch(Exception e){javax.swing.JOptionPane.showMessageDialog(null,"SctApi Exception "+e.toString());}

		  }
              });

		return moduleConfigMenu;


       }

	private void showToolBar() {
	    if(displayChoiceToolbar!=null) displayChoiceToolbar.setVisible(true);
        else {
  			displayChoiceToolbar = new DisplayChoiceToolbar(guiControl);
            java.awt.Dimension s = displayChoiceToolbar.getSize();
            displayChoiceToolbar.setSize((int)s.getWidth()+1,(int)s.getHeight()+1);
            }
        }
	private void updateToolBar(int display) {
			if(displayChoiceToolbar!=null) displayChoiceToolbar.setButton(display);
			}
	private void showModuleCommentPrompt(String serialNo) {
       if(moduleCommentPrompt==null) moduleCommentPrompt=new ModuleCommentPrompt(serialNo);
       else moduleCommentPrompt.update(serialNo);
       }

	public void rebuildTestMenus() {
		for(int i=0;i<2;i++) testsMenu[i].buildAutoTestsMenu();
		}

    public void enableSubMenuItems(JMenu menu, boolean enabled) {
		// enable or disable items in submenus
		java.awt.Component[] component = menu.getMenuComponents();
		for(int i=0;i<component.length;i++) {
	    	if(component[i].getClass().getName().startsWith("javax.swing.JPopupMenu")) break;
	    	if(!component[i].getClass().getName().startsWith("javax.swing.JMenu")) continue;
	    	JMenu subMenu = (JMenu)component[i];
	    	for(int y=0;y<subMenu.getItemCount();y++) {
				JMenuItem item = subMenu.getItem(y);
				if(item!=null) item.setEnabled(enabled);
	    		}
			}
		}
}
