package DisplayGUI;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import java.util.*;

//SelectorPane itself is not a visible component.
public class SelectorPane extends JPanel implements ListSelectionListener,displayParams,SCTInfo,TestDataInfo {
    private JList displayParameters, sctViews, mouseOptions, viewCategories;
    private JSplitPane splitPane;
    private String[] defaultString = {"...None"};
    private String[] resultStatusString = {"Overall Status"};
    private String[] defectCountString = {"Total Count"};
    private String[] dataString = {"Module Data"};
	private String[] viewCats={"SCT Views","ROD Crates"};
    private String[][] displayParameterStrings = {
                        {},
                        {},
                        {"Module Probe"},
                        {"Raw Data","Fitted Data","Analysed Data"},
                        {},
                        {},
                        {},
                        {},
			  	{"Result Status","IVScan Data","DCS Conditions"}
                       };
    private String[][] mouseOptionStrings = {
                        {},
                        {"Group 0","Group 1","Group 2","Group 3","Group 4","Group 5","Group 6","Group 7"},
                        {"Link 0","Link 1"},
                        {"List Data objects"},
                        {},
                        {},
			 			{},
			 			{"Current Conditions"},
			 			{}
                       	};
    private gui guiControl;

    private boolean selectionListenerIsActive=true;

    private Map viewMap;


    public SelectorPane(gui parent) {
        guiControl=parent;

        buildViewMap();

		sctViews = new JList();
		viewCategories = new JList();
		displayParameters = new JList();
        mouseOptions = new JList();

		viewCategories.setListData(viewCats);
		sctViews.setListData(SCTViewNames);

        int viewIndex=0;
        try {
         Preferences.PreferencesInterface prefs = Preferences.PreferencesInterface.getInstance();
         String preferredView = prefs.getPreference(Preferences.PreferencesInterface.SCT_VIEW);
         if(!preferredView.equals(Preferences.PreferencesInterface.unDefinedString)) {
            Integer i1 = (Integer)viewMap.get(preferredView);
            if(i1!=null) viewIndex = i1.intValue();
            }
        }catch(Exception e){}
        sctViews.setSelectedIndex(viewIndex);
		viewCategories.setSelectedIndex(0);

		JScrollPane viewScrollPane = defineScrollPane(sctViews,"View Options");
		JScrollPane viewCategoryScrollPane = defineScrollPane(viewCategories,"Views");
		JScrollPane paramScrollPane = defineScrollPane(displayParameters,"Data");
		JScrollPane mouseScrollPane = defineScrollPane(mouseOptions,"Data Options");

        //Create a split pane with the two scroll panes in it
        JSplitPane topSplit = new JSplitPane(JSplitPane.VERTICAL_SPLIT, paramScrollPane, mouseScrollPane);
        topSplit.setDividerSize(8);
        topSplit.setDividerLocation(0.50);
        topSplit.setBorder(new javax.swing.border.EmptyBorder(0,0,0,0));

        JSplitPane bottomSplit = new JSplitPane(JSplitPane.VERTICAL_SPLIT, viewCategoryScrollPane, viewScrollPane);
        bottomSplit.setDividerSize(8);
        bottomSplit.setDividerLocation(0.25);
        bottomSplit.setBorder(new javax.swing.border.EmptyBorder(0,0,0,0));

        splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
                                   topSplit, bottomSplit);
//        splitPane.setOneTouchExpandable(true);
        splitPane.setDividerLocation(0.6);
        splitPane.setDividerSize(8);
        splitPane.setBorder(new javax.swing.border.EmptyBorder(0,0,0,0));

        //Provide minimum sizes for the two components in the split pane
        Dimension minimumSize = new Dimension(95, 16);
        paramScrollPane.setMinimumSize(minimumSize);
        mouseScrollPane.setMinimumSize(minimumSize);
        viewScrollPane.setMinimumSize(minimumSize);
		viewCategoryScrollPane.setMinimumSize(minimumSize);

        //Provide a preferred size for the split pane
        splitPane.setPreferredSize(new Dimension(95,yOffSet+56*boxHeight)); 
        add(splitPane);
        setBackground(backgroundColor);
        setBorder(new javax.swing.border.TitledBorder(null, "Display", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Arial", 0, 11), new java.awt.Color(0, 0, 153)));

    }


	private JScrollPane defineScrollPane(JList list, String title) {
        list.setFont(displayScaleFont);
        list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        list.addListSelectionListener(this);
        JScrollPane scrollPane = new JScrollPane(list);
        JLabel header = new JLabel(title);
        header.setForeground(darkBlueColor);
        header.setBackground(backgroundColor);
        header.setFont(displayTextFont);
        scrollPane.setColumnHeaderView(header);
        scrollPane.getColumnHeader().setBackground(backgroundColor);
        scrollPane.setBorder(new javax.swing.border.EmptyBorder(0,0,0,0));
        scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        scrollPane.getVerticalScrollBar().setBackground(backgroundColor);
		return scrollPane;
		}
		
    public void initParamList() {

        selectionListenerIsActive=false;

        int displayType = guiControl.displayStatus.getStatus();
        boolean noParameters=true, noOptions=true, noViews=false;
        int previousParameterSelection = getDisplayParameterIndex();
        int previousOptionSelection = getMouseClickIndex();
		int previousViewCategorySelection = getViewCategoryIndex();
		int previousViewSelection = getViewIndex();

		// first rebuild the views
		if(previousViewCategorySelection==ConfigurationInterface.SCT_VIEWS) {
			sctViews.setListData(SCTViewNames);
			}
		else {
			Set viewList = ConfigurationInterface.getInstance().getRodCrateViewMap().keySet();
			if(viewList.size()>0) sctViews.setListData(viewList.toArray());
			else noViews=true;
			}

        int nParams = sctViews.getModel().getSize();
        int selectionIndex=0;

        if(nParams>0) {
           if(previousViewSelection<nParams && previousViewSelection>=0) selectionIndex=previousViewSelection;
           else if(previousViewSelection>=nParams) selectionIndex =sctViews.getLastVisibleIndex();
		   sctViews.setSelectedIndex(selectionIndex);
           }
		if(noViews) sctViews.setListData(defaultString);

        TestSelection testSelection = guiControl.tablesDisplayPane.getTestSelection();
        int testIndex = (testSelection!=null) ? testSelection.getTestIndex() : -1;

        if(displayType==DisplayStatus.UNDEFINED) {
			selectionListenerIsActive=true;
			return;
			}
        switch(displayType) {
            case DisplayStatus.MODULE_GROUP_SELECTION:
            	displayParameters.setListData(configParams);
			// if not yet defined, or module group selection, set module group selection
			if(previousParameterSelection<=0) mouseOptions.setListData(mouseOptionStrings[displayType]);
			// otherwise could be link options or no options
			else {
				if(configParams_links[previousParameterSelection])mouseOptions.setListData(linkOptions);
				else mouseOptions.setListData(dataString);
 				}
               noParameters=false;
               noOptions=false;
               break;
            case DisplayStatus.IS_MONITOR:
               displayParameters.setListData(displayParameterStrings[displayType]);
			   mouseOptions.setListData(mouseOptionStrings[displayType]);
               noParameters=false;
               noOptions=false;
               break;
            case DisplayStatus.TEST_DATA:
	    	case DisplayStatus.SCTDB_DATA:
	    	case DisplayStatus.SCTDB_RODDAQ_DATA:
               if(testIndex!=-1 && testIndex!=TEST_DEFAULT) {
                    displayParameters.setListData(testParameters[testIndex]);
                    noParameters=false;
                    switch(previousParameterSelection) {
                    case 0: // Result Status
                    case -1: // no previous selection - will be set to 0 by default
                        mouseOptions.setListData(resultStatusString);
                        noOptions=false;
                        break;
                    case 1: // Defect count
		    		case 2: // bad channel count
                        mouseOptions.setListData(defectCountString);
                        noOptions=false;
                        break;
                    case 3:
                        mouseOptions.setListData(dcsOptions);
                        noOptions=false;
                        break;
                    default:
                        if(testIndex==TEST_RXTHRESHOLD || testIndex==TEST_RXTHRESHOLDBASEDONCONFIGREGISTER ||testIndex==TEST_RXDELAY) mouseOptions.setListData(linkOptions);
                        else mouseOptions.setListData(chipOptions);
                        noOptions=false;
                        }
                    }

               break;
            case DisplayStatus.DCS_DATA:
               displayParameters.setListData(dcsOptions);
               noParameters=false;
               mouseOptions.setListData(mouseOptionStrings[displayType]);
               noOptions=false;
               break;
            case DisplayStatus.IV_DATA:
               IVTestSelection iv = guiControl.tablesDisplayPane.getIVTestSelection();
               if(iv==null) {
                 displayParameters.setListData(defaultString);
                 noParameters=true;
                 mouseOptions.setListData(defaultString);
                 noOptions=true;
                 }
              else {
                 displayParameters.setListData(displayParameterStrings[displayType]);
                 noParameters=false;
                 switch(previousParameterSelection) {
                    case 0:
                    case -1: // no previous selection - will be set to 0 by default
                        mouseOptions.setListData(resultStatusString);
                        noOptions=false;
                        break;
                    case 1:                      
                        mouseOptions.setListData(iv.getVList());
                        noOptions=false;
                        break;
                    case 2:                      
                        mouseOptions.setListData(dcsOptions);
                        noOptions=false;
                        break;
                        }
                 }
               break;
	     case DisplayStatus.PROBE_DATA:
                 displayParameters.setListData(displayParameterStrings[displayType]);
               noParameters=false;
               mouseOptions.setListData(mouseOptionStrings[displayType]);
               noOptions=false;
               break;

            case DisplayStatus.UNDEFINED:
            default:
              displayParameters.setListData(defaultString);
              mouseOptions.setListData(defaultString);
            }

        int numParameters = displayParameters.getModel().getSize();
        selectionIndex=0;
        if(numParameters>0) {
           if(previousParameterSelection<numParameters && previousParameterSelection>=0) selectionIndex=previousParameterSelection;
           else if(previousParameterSelection>=numParameters) selectionIndex =displayParameters.getLastVisibleIndex();
		   displayParameters.setSelectedIndex(selectionIndex);
           }

        numParameters = mouseOptions.getModel().getSize();
        selectionIndex=0;
        if(numParameters>0) {
            if(previousOptionSelection<numParameters && previousOptionSelection>=0) selectionIndex= previousOptionSelection;
            else if(previousOptionSelection>=numParameters) selectionIndex = mouseOptions.getLastVisibleIndex();
			mouseOptions.setSelectedIndex(selectionIndex);
            }

        if(noParameters) displayParameters.setListData(defaultString);
        if(noOptions) mouseOptions.setListData(defaultString);

        setVisible(true);
        selectionListenerIsActive=true;
    }
   public String getIVParameter() {
     if(getDisplayParameterIndex()==0) return "Result Status";
     return getOption();
     }

   public int getDisplayParameterIndex() {
     return displayParameters.getSelectedIndex();
     }
   public int getMouseClickIndex() {
     return mouseOptions.getSelectedIndex();
     }
   public void setMouseClickIndex(int row) {
     mouseOptions.setSelectedIndex(row);
     }
   public int getViewIndex() {
     return sctViews.getSelectedIndex();
     }

   public int getViewCategoryIndex() {
   	 return viewCategories.getSelectedIndex();
	 }
   public String getOption() {
      if(getMouseClickIndex()==-1) return null;
      return (String)mouseOptions.getSelectedValue();
      }
   public String getDisplayParameter() {
     if(getDisplayParameterIndex()==-1) return null;
     return (String)displayParameters.getSelectedValue();
     }
   public String getViewName() {
   	if(getViewIndex()<0) return null;
	return (String)sctViews.getSelectedValue();
	}

    public JSplitPane getSplitPane() {
        return splitPane;
    }

    public void valueChanged(ListSelectionEvent e) {

		if(!selectionListenerIsActive) return;
        if (e.getValueIsAdjusting()) return;
        JList theList = (JList)e.getSource();
        if (theList.isSelectionEmpty()) return;  // huh?

        int index = theList.getSelectedIndex();

        if(theList==sctViews) {
        	guiControl.tablesDisplayPane.updateTables();
            return;
            }

		if(theList==viewCategories) {  // change of view category, requires change in view options
			initParamList();
        	guiControl.tablesDisplayPane.updateTables();
			return;
			}

        switch(guiControl.displayStatus.getStatus()) {
            case DisplayStatus.MODULE_GROUP_SELECTION:
			if(theList==displayParameters) {
				initParamList(); // in case the mouse option list needs changing
				if(getDisplayParameterIndex()>0) SctApiInfo.getInstance().probeConfig(getViewCategoryIndex(),getViewIndex(),getDisplayParameterIndex());
				}
               	guiControl.tablesDisplayPane.updateTables(true); // for module group, just update tables
                  guiControl.colorScalePane.setColorScale();
                  guiControl.displayPane.setDisplayColors();
			guiControl.displayPane.refreshDisplay();
			//if(getDisplayParameterIndex()>0)guiControl.tablesDisplayPane.listTestResults();
               	break;
            case DisplayStatus.IS_MONITOR:
//              guiControl.isInterface.setServerName(index);
              	guiControl.tablesDisplayPane.checkForRetrievedObjects();
              	guiControl.colorScalePane.setColorScale();
              	guiControl.displayPane.setDisplayColors();
              	guiControl.displayPane.refreshDisplay();
              	guiControl.tablesDisplayPane.listISObjects(guiControl.displayPane.getLastClickedCell());
              	break;
           	case DisplayStatus.TEST_DATA:
           	case DisplayStatus.SCTDB_DATA:
	   		case DisplayStatus.SCTDB_RODDAQ_DATA:
              	if(theList==displayParameters) initParamList();
              	guiControl.colorScalePane.setColorScale();
              	guiControl.displayPane.setDisplayColors();
              	guiControl.displayPane.refreshDisplay();
              	guiControl.tablesDisplayPane.listTestResults();
	      		break;
	   		case DisplayStatus.DCS_DATA:
	      		if(theList==displayParameters) { // must resubscribe, depending on which DCS data we are looking at
	          		guiControl.isInterface.unsubscribe();
		  			guiControl.isInterface.subscribe();
		  			}
	      		guiControl.colorScalePane.setColorScale();
              	guiControl.displayPane.setDisplayColors();
              	guiControl.displayPane.refreshDisplay();
              	guiControl.tablesDisplayPane.listTestResults();
              	break;
	   		case DisplayStatus.IV_DATA:
	   		case DisplayStatus.PROBE_DATA:
              	if(theList==displayParameters) initParamList();
              	guiControl.colorScalePane.setColorScale();
              	guiControl.displayPane.setDisplayColors();
              	guiControl.displayPane.refreshDisplay();
              	guiControl.tablesDisplayPane.listTestResults();
	      		break;
           	default:
           	}

    }
  private void buildViewMap() {
     viewMap = new HashMap();
     for(int i=0;i<SCTViewNames.length;i++) viewMap.put(SCTViewNames[i], new Integer(i));
     }
  public void setSCTView(String value) {
     if(!viewMap.containsKey(value)) return;
     sctViews.setSelectedIndex(((Integer)viewMap.get(value)).intValue());
	 guiControl.tablesDisplayPane.updateTables();
     }


}
