/*
 * TestsMenu class
 * This class defines the tests menus and
 * the methods to access them
 *
 * @author robinson
 *
 */
package DisplayGUI;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JMenu;

import Sct_CalibrationController.TestRequest;
import Sct_CalibrationController.SequenceRequest;
import Sct_CalibrationController.CalibrationControllerPackage.*;

import GuiComponents.System.*;


/** Menu bar for GUI */
public class TestsMenu extends JMenuBar implements displayParams,TestDataInfo {
    gui guiControl;


    private JMenu autoTestsMenu = new JMenu();


	public static final int BOCTESTS_MENU=0;
	public static final int DIGITALTESTS_MENU=1;
	public static final int ANALOGUETESTS_MENU=2;
	public static final int TIMINGTESTS_MENU=3;
	public static final int QUICKTESTS_MENU=4;
	public static final int SEQUENCES_MENU=5;
	public static final int ALLTESTS_MENU=6;
	public static final int SCTDAQTESTS_MENU=7;

	private int N_MENUS=8;

	private JMenu[] guiMenu;

	public TestsMenu(gui guiControl) {
		this.guiControl = guiControl;

        autoTestsMenu = new JMenu("Complete Test List");
	   	autoTestsMenu.setFont(menuFont);
		buildAutoTestsMenu();


		guiMenu = new JMenu[N_MENUS];
		for(int imenu=0;imenu<N_MENUS;imenu++) {
			switch(imenu) {
				case BOCTESTS_MENU:
					guiMenu[imenu]=getBOCMenu();
					break;
				case ANALOGUETESTS_MENU:
					guiMenu[imenu]=getAnalogueMenu();
					break;
				case DIGITALTESTS_MENU:
					guiMenu[imenu]=getDigitalMenu();
					break;
				case TIMINGTESTS_MENU:
					guiMenu[imenu]=getTimingTestsMenu();
					break;
				case QUICKTESTS_MENU:
                	guiMenu[imenu]=getQuickTestsMenu();
					break;
				case SEQUENCES_MENU:
					guiMenu[imenu]=getSequencesMenu();
					break;
				case ALLTESTS_MENU:
					guiMenu[imenu]=autoTestsMenu;
					break;
				case SCTDAQTESTS_MENU:
					guiMenu[imenu]=getSCTDAQMenu();
					break;
				default:
				}
			guiMenu[imenu].setFont(menuFont);
			guiMenu[imenu].setBackground(backgroundColor);
            //guiMenu[imenu].setForeground(menuColor);
			}
		}

	public JMenu getMenu(int menu) {
		return guiMenu[menu];
		}
	public javax.swing.JPopupMenu getPopupMenu(int menu) {
		return guiMenu[menu].getPopupMenu();
		}

	private JMenu getSCTDAQMenu() {
       JMenu sctdaqmenu = new JMenu("SCTDAQ Exerciser");

       sctdaqmenu.setFont(menuFont);

       JMenuItem menuItem = new JMenuItem("Ping");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                     //There is no ping method atm
                     //System.out.println(si.getTestApi().ping(""));
                     }
             });
       sctdaqmenu.add(menuItem);
       sctdaqmenu.addSeparator();

       for(int i=0;i<testIsSCTDAQ.length;i++) {
         if(!testIsDefined[i] || !testIsSCTDAQ[i]) continue;
         menuItem = new JMenuItem(testMenuNames[i]);
         menuItem.setFont(menuFont);
         sctdaqmenu.add(menuItem);
         switch(i) {
            case TEST_FULLBYPASS:
              menuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                   guiControl.displayStatus.setTestDataMode(-1);
                   try {
                       if((SystemInterface.getInstance().getTestHighLevelApi())!=null) SystemInterface.getInstance().getTestHighLevelApi().fullBypass();
                   } catch (SctTestApi.NoDataException nde) {}
//                   System.out.println("Start full bypass test via sct_api_server");
//                   if(sctdbAutoDownload.isSelected()) guiControl.isInterface.downloadTests(TEST_FULLBYPASS);
                   }
              });
            case TEST_PIPELINE:
              menuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
//                    guiControl.displayStatus.setTestDataMode(-1);
                    try {
                        if((SystemInterface.getInstance().getTestHighLevelApi())!=null) SystemInterface.getInstance().getTestHighLevelApi().pipeline();
                    } catch (SctTestApi.NoDataException nde) {}
//                   System.out.println("Start pipeline test via sct_api_server");
//                   if(sctdbAutoDownload.isSelected()) guiControl.isInterface.downloadTests(TEST_PIPELINE);
                   }
              });
            break;
            case TEST_STROBEDELAY:
              menuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
//                    guiControl.displayStatus.setTestDataMode(-1); 
                    try {
                        if((SystemInterface.getInstance().getTestHighLevelApi())!=null) SystemInterface.getInstance().getTestHighLevelApi().strobeDelay();
                    } catch (SctTestApi.NoDataException nde) {}
//                    System.out.println("Start strobe delay test via sct_api_server");
//                    if(sctdbAutoDownload.isSelected()) guiControl.isInterface.downloadTests(TEST_STROBEDELAY);
                   }
              });
             break;
            case TEST_3PTGAIN:
              menuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
//                    guiControl.displayStatus.setTestDataMode(-1);
                    try {
                        if((SystemInterface.getInstance().getTestHighLevelApi())!=null) SystemInterface.getInstance().getTestHighLevelApi().threePointGain();
                    } catch (SctTestApi.NoDataException nde) {}
//                    System.out.println("Start 3PtGain test via sct_api_server");
//                    if(sctdbAutoDownload.isSelected()) guiControl.isInterface.downloadTests(TEST_3PTGAIN);
                   }
              });
             break;
            case TEST_TRIM:
              menuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
//                    guiControl.displayStatus.setTestDataMode(-1);
                    try {
                        if((SystemInterface.getInstance().getTestHighLevelApi())!=null) SystemInterface.getInstance().getTestHighLevelApi().trim();
                    } catch (SctTestApi.NoDataException nde) {}
                   //System.out.println("Start Trim Range test via sct_api_server");
//                   if(sctdbAutoDownload.isSelected()) guiControl.isInterface.downloadTests(TEST_TRIM);
                   }
              });
             break;
            case TEST_NPTGAIN:
              menuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
//                    guiControl.displayStatus.setTestDataMode(-1);
                    try {
                        if((SystemInterface.getInstance().getTestHighLevelApi())!=null) SystemInterface.getInstance().getTestHighLevelApi().responseCurve();
                    } catch (SctTestApi.NoDataException nde) {}
                   //System.out.println("Start NPtGain test via sct_api_server");
//                   if(sctdbAutoDownload.isSelected()) guiControl.isInterface.downloadTests(TEST_NPTGAIN);
                   }
              });
             break;
            case TEST_NOISE:
              menuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
//                    guiControl.displayStatus.setTestDataMode(-1);
                    try {
                        if((SystemInterface.getInstance().getTestHighLevelApi())!=null) SystemInterface.getInstance().getTestHighLevelApi().noiseOccupancy();
                    } catch (SctTestApi.NoDataException nde) {}
                   //System.out.println("Start Noise test via sct_api_server");
//                   if(sctdbAutoDownload.isSelected()) guiControl.isInterface.downloadTests(TEST_NOISE);
                   }
              });
             break;
            case TEST_TIMEWALK:
              menuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
//                    guiControl.displayStatus.setTestDataMode(-1);
                    try {
                        if((SystemInterface.getInstance().getTestHighLevelApi())!=null) SystemInterface.getInstance().getTestHighLevelApi().timeWalk();
                    } catch (SctTestApi.NoDataException nde) {}
                   //System.out.println("Start TimeWalk test via sct_api_server");
//                   if(sctdbAutoDownload.isSelected()) guiControl.isInterface.downloadTests(TEST_TIMEWALK);
                   }
              });
             break;
             default:
             }
         }
    return sctdaqmenu;
    }

	private JMenu getSequencesMenu() {
   JMenu sequencesMenu = new JMenu("Sequences");
       sequencesMenu.setFont(menuFont);

       JMenuItem menuItem = new JMenuItem("Characterization");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
	       public void actionPerformed(java.awt.event.ActionEvent evt) {
		   try{
		       SystemInterface.getInstance().getCalibrationController().doSequence(SystemInterface.getInstance().getCalibrationController().getSequenceLibrary().characterizationSequence());
		   }catch(Throwable e){
		       System.out.println("Error communicating with CalibrationController [GuiMenuBar2:CharacterizationSequence]");
		   };
	       };
	   });

       sequencesMenu.add(menuItem);
	   return sequencesMenu;
	   }

	private JMenu getQuickTestsMenu() {
       JMenu quickTestsMenu = new JMenu("Quick Tests");
       quickTestsMenu.setFont(menuFont);

       JMenuItem menuItem = new JMenuItem("Quick NMask");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
	       public void actionPerformed(java.awt.event.ActionEvent evt) {
		   menuDoTest(menuGetTestByName("QuickNMask"));
	       }
	   });
       quickTestsMenu.add(menuItem);

       menuItem = new JMenuItem("Five point gain");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                     menuDoTest(menuGetTestByName("FivePointGain"));
		 }
	   });
       quickTestsMenu.add(menuItem);

       menuItem = new JMenuItem("QuickTrim");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                     menuDoTest(menuGetTestByName("QuickTrimRange") );
		 }
             });

       quickTestsMenu.add(menuItem);
	   return quickTestsMenu;
	   }

	private JMenu getTimingTestsMenu() {

       JMenu timingTestsMenu = new JMenu("Timing Tests");
       timingTestsMenu.setFont(menuFont);

       JMenuItem menuItem = new JMenuItem("Synch Trigger Noise");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
	       public void actionPerformed(java.awt.event.ActionEvent evt) {
		   menuDoTest(menuGetTestByName("SynchTriggerNoise"));
	       }
	   });

       timingTestsMenu.add(menuItem);

       menuItem = new JMenuItem("Double Trigger Noise");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
 //                    guiControl.displayStatus.setTestDataMode(-1);
                     menuDoTest(menuGetTestByName("DoubleTriggerNoise"));
                     }
             });

       timingTestsMenu.add(menuItem);

       menuItem = new JMenuItem("Pre Trigger Noise");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
	       public void actionPerformed(java.awt.event.ActionEvent evt) {
		   menuDoTest(menuGetTestByName("PreTriggerNoise"));
	       }
	   });

       timingTestsMenu.add(menuItem);

       menuItem = new JMenuItem("TIM clock delay");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
	       public void actionPerformed(java.awt.event.ActionEvent evt) {
		   menuDoTest(menuGetTestByName("TimClockDelay"));
	       }
	   });

       timingTestsMenu.add(menuItem);
       return timingTestsMenu;
	   }

	private JMenu getAnalogueMenu() {

       JMenu analogueTestsMenu = new JMenu("Analogue Tests");
       analogueTestsMenu.setFont(menuFont);


       JMenuItem menuItem = new JMenuItem("Strobe Delay");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
 //                    guiControl.displayStatus.setTestDataMode(-1);
                     menuDoTest(menuGetTestByName("StrobeDelay"));
                     }
             });

       analogueTestsMenu.add(menuItem);

       menuItem = new JMenuItem("3PtGain");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
 //                    guiControl.displayStatus.setTestDataMode(-1);
                     menuDoTest(menuGetTestByName("ThreePointGain"));
                     }
             });

       analogueTestsMenu.add(menuItem);

       menuItem = new JMenuItem("Trim");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
 //                    guiControl.displayStatus.setTestDataMode(-1);
                     menuDoTest(menuGetTestByName("TrimRange") );
                     }
             });

       analogueTestsMenu.add(menuItem);

       menuItem = new JMenuItem("ResponseCurve");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
 //                    guiControl.displayStatus.setTestDataMode(-1);
                     menuDoTest(menuGetTestByName("ResponseCurve"));
                     }
             });

       analogueTestsMenu.add(menuItem);

       menuItem = new JMenuItem("Noise Occupancy");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
 //                    guiControl.displayStatus.setTestDataMode(-1);
                     menuDoTest(menuGetTestByName("NoiseOccupancy") );
                     }
             });

       analogueTestsMenu.add(menuItem);

       menuItem = new JMenuItem("Time Walk");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
 //                    guiControl.displayStatus.setTestDataMode(-1);
                     menuDoTest(menuGetTestByName("TimeWalk"));
                     }
             });

       analogueTestsMenu.add(menuItem);
       return analogueTestsMenu;
	   }

	private JMenu getDigitalMenu() {

       JMenu digitalTestsMenu = new JMenu("Digital Tests");
       digitalTestsMenu.setFont(menuFont);

       JMenuItem menuItem = new JMenuItem("NMask");
       menuItem.setFont(menuFont);

       menuItem.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
//                    guiControl.displayStatus.setTestDataMode(-1);
                     menuDoTest(menuGetTestByName("NMask"));
                     }
             });


       digitalTestsMenu.add(menuItem);

       menuItem = new JMenuItem("Pipeline");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
 //                    guiControl.displayStatus.setTestDataMode(-1);
	             menuDoTest(menuGetTestByName("Pipeline"));
                     }
             });

       digitalTestsMenu.add(menuItem);

       menuItem = new JMenuItem("FullByPass (Barrel)");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
 //                    guiControl.displayStatus.setTestDataMode(-1);
                     menuDoTest(menuGetTestByName("FullBypassBarrel") );
                 }
             });

       digitalTestsMenu.add(menuItem);

       menuItem = new JMenuItem("FullByPass (Endcap)");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
 //                    guiControl.displayStatus.setTestDataMode(-1);
		     menuDoTest(menuGetTestByName("FullBypassEndcap") );
                 }
             });

       digitalTestsMenu.add(menuItem);
	   return digitalTestsMenu;
	   }

	private JMenu getBOCMenu() {
       JMenu bocTestsMenu = new JMenu("Boc Setup");
       bocTestsMenu.setFont(menuFont);
       JMenuItem menuItem = new JMenuItem("Raw Scan");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
	       public void actionPerformed(java.awt.event.ActionEvent evt) {
		   try{
		       SystemInterface.getInstance().getCalibrationController().doScan(SystemInterface.getInstance().getCalibrationController().getScanLibrary().rawScan(10,false));
		   }catch(Throwable e){
		       javax.swing.JOptionPane.showMessageDialog(null,"Error communicating with Calibration Controller [GuiMenuBar2:RawScan]");
		   }
	       }
	   });

       bocTestsMenu.add(menuItem);

       /*       menuItem = new JMenuItem("RX Threshold (Old Clk/2)");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
	       public void actionPerformed(java.awt.event.ActionEvent evt) {
		   //Sct_CalibrationController.ScanRequest request = si.getCalibrationController().getScanLibrary().rawScan(10);
		   //request.getScan().configure((short)50, 0., 200., 5.);
		   //si.getCalibrationController().doScan( request );
		   menuDoTest(si.getCalibrationController().getTestLibrary().getTestRequestByName("RxThreshold")  );
	       }
	   });
	   bocTestsMenu.add(menuItem); */

       menuItem = new JMenuItem("RX Threshold (Config Register)");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
	       public void actionPerformed(java.awt.event.ActionEvent evt) {
		   menuDoTest(menuGetTestByName("RxThresholdBasedOnConfigRegister")  );
	       }
	   });
       bocTestsMenu.add(menuItem);

       menuItem = new JMenuItem("RX Delay");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
	       public void actionPerformed(java.awt.event.ActionEvent evt) {
		   //Sct_CalibrationController.ScanRequest request = si.getCalibrationController().getScanLibrary().rawScan(10);
		   //request.getScan().configure((short)20, 0., 24., 1.);
		   //si.getCalibrationController().doScan( request );
		   menuDoTest(menuGetTestByName("RxDelay")  );
	       }
	   });
       bocTestsMenu.add(menuItem);

       /*menuItem = new JMenuItem("TX Current (old, no analysis)");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
	       public void actionPerformed(java.awt.event.ActionEvent evt) {
		   Sct_CalibrationController.ScanRequest request = si.getCalibrationController().getScanLibrary().rawScan(10,false);
		   request.getScanCorba().configure((short)53, 60., 200., 10.);
		   si.getCalibrationController().doScan( request );
	       }
	   });
	   bocTestsMenu.add(menuItem);*/

       menuItem = new JMenuItem("TX Current (Config Register)");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
	       public void actionPerformed(java.awt.event.ActionEvent evt) {
		   menuDoTest(menuGetTestByName("TxCurrent")  );
	       }
	   });
       bocTestsMenu.add(menuItem);

       //******
       menuItem = new JMenuItem("Mark-space ratio");
       menuItem.setFont(menuFont);
       menuItem.addActionListener(new java.awt.event.ActionListener() {
	       public void actionPerformed(java.awt.event.ActionEvent evt) {
		   menuDoTest(menuGetTestByName("MarkSpaceRatio")  );
	       }
	   });
       bocTestsMenu.add(menuItem);
       return bocTestsMenu;
	   }


    private void menuDoTest(Sct_CalibrationController.TestRequest tr){
		try{
	    	Sct_CalibrationController.CalibrationController cc = SystemInterface.getInstance().getCalibrationController();
	    	if (tr==null)javax.swing.JOptionPane.showMessageDialog(null,"Attempt to send a null TestRequest to the CalbrationController");
	    	else cc.doTest(tr);
	    }
	    catch(Throwable e){javax.swing.JOptionPane.showMessageDialog(null,"GUI Error trying to communicate with Calibration Controller! [GuiMenuBar2:menuDoTest]" + e.toString());}
		return;
    }

    private Sct_CalibrationController.TestRequest menuGetTestByName(String name){
		Sct_CalibrationController.CalibrationController cc = SystemInterface.getInstance().getCalibrationController();
		if (cc==null) javax.swing.JOptionPane.showMessageDialog(null,"Could not get Calibration Controller");
		Sct_CalibrationController.TestLibrary tl = cc.getTestLibrary();
		if (tl==null) javax.swing.JOptionPane.showMessageDialog(null,"Could not get Test Libarary from Calibration Controller");
		Sct_CalibrationController.TestRequest tr = tl.getTestRequestByName(name);
		if (tr==null) javax.swing.JOptionPane.showMessageDialog(null,"Could not get test with name "+name+" from library");
		return tr;
    }

	 public void buildAutoTestsMenu(){
       //System.out.println("Rebuilding auto tests menu...");
      if(autoTestsMenu.getItemCount()>1) return;  // already done
      autoTestsMenu.removeAll();
	  SystemInterface si = SystemInterface.getInstance();
      if (si!=null) {
	  Sct_CalibrationController.CalibrationController cc = SystemInterface.getInstance().getCalibrationController();
	  if (cc!=null) {
	      Sct_CalibrationController.TestLibrary tl = cc.getTestLibrary();
	      if (tl!=null) {
		  String[] testNames = si.getCalibrationController().getTestLibrary().getTestNames();
		  for (int i=0; i<testNames.length; ++i) {
		      final String name = testNames[i];
		      JMenuItem menuItem = new JMenuItem(name);
		      menuItem.setFont(menuFont);
		      menuItem.addActionListener(new java.awt.event.ActionListener() {
			      public void actionPerformed(java.awt.event.ActionEvent evt) {
				  menuDoTest(menuGetTestByName(name));
			      }
			  });

		      autoTestsMenu.add(menuItem);
		  };
	      };
	  };
      };
     //System.out.println("Auto Tests Menu has "+autoTestsMenu.getItemCount()+" items");
      if (autoTestsMenu.getItemCount()==0){
	  JMenuItem menuItem = new JMenuItem("Not available");
	  menuItem.setFont(menuFont);
	  autoTestsMenu.add(menuItem);
      }
   }



}
