package webscripts;
// getDBfiles.java
// @Author Dave Robinson
//
// 21052003 Introduce controlFlag - bit 1 set means create upload files but don't upload
//                                  bit 2 set means don't show gui and immediately create upload files


import java.io.*;
import java.util.*;
import java.sql.*;
import ProdDatabase.*;
import ProdDatabase.SCTDBInfo;
import DisplayGUI.*;

public class generalScripts {



public static Vector getPositionMap(int i) throws Exception {

       Vector list = new Vector();
       StringBuffer sqlStat = new StringBuffer("SELECT ser_no,posn FROM assm_items WHERE");
       sqlStat.append(" assm_ser_no="+DisplayGUI.SCTInfo.SCTSerialNumbers[i]);
       sqlStat.append(" ORDER BY ser_no");

//      System.out.println(sqlStat.toString());
       Statement statement = SCTDBInterface.getInstance().connection.createStatement();
       ResultSet resultSet = statement.executeQuery(sqlStat.toString());
       int recordCount=0;

       for(boolean n = resultSet.next() ; n==true ; n=resultSet.next() ){
          String sn = resultSet.getString(1);
          String posn = resultSet.getString(2);
          DisplayGUI.SCTDBAssemblyInfo a = new DisplayGUI.SCTDBAssemblyInfo(posn);
	  list.addElement(sn+"="+DisplayGUI.SCTInfo.SCTViewNames[i]+", Row "+a.getRow()+", Position "+a.getPos());
          }
       statement.close();
       return list;
       }

       
public static void makeDataReferenceFile(String targetDir, int testIndex, boolean isWarm) throws Exception {

       Set s = new HashSet();
       StringBuffer sqlStat = new StringBuffer("SELECT tests.ser_no,tests.TEST_no,tests.locn_name,tests.test_date");
       switch(testIndex) {
         case SCTDBInfo.SCTDB_TEST_3PTGAIN:
            for(int y=0;y<SCTDBInfo.chipDBnames.length;y++) sqlStat.append(","+SCTDBInfo.chipDBnames[y]+"RC_MN_ENC,"+SCTDBInfo.chipDBnames[y]+"RC_MN_GAIN");
            break;
         case SCTDBInfo.SCTDB_TEST_STROBEDELAY:
            for(int y=0;y<SCTDBInfo.chipDBnames.length;y++) sqlStat.append(","+SCTDBInfo.chipDBnames[y]+"DELAY");
            break;
         case SCTDBInfo.SCTDB_TEST_NOISE:
            for(int y=0;y<SCTDBInfo.chipDBnames.length;y++) sqlStat.append(","+SCTDBInfo.chipDBnames[y]+"NO_NSE,"+SCTDBInfo.chipDBnames[y]+"NO_MN_OCC");
         default:
	 }

       sqlStat.append(" FROM "+SCTDBInfo.sctdaqDBTableNames[testIndex]+",tests,sct_tstdcsinfo,SCT_TSTDAQINFO");
       sqlStat.append(" WHERE tests.TEST_name = '"+SCTDBInfo.sctdaqDBTestNames[testIndex]+"'");
       sqlStat.append(" AND tests.ser_no IN (SELECT ser_no FROM items WHERE locn_name='Oxford')");
       sqlStat.append(" AND tests.TEST_no = "+SCTDBInfo.sctdaqDBTableNames[testIndex]+".TEST_no");
       sqlStat.append(" AND SCT_TSTDAQINFO.version NOT LIKE 'SctRodDaq%' AND SCT_TSTDAQINFO.test_no=tests.TEST_no");  // SCTDAQ data only
       if(isWarm) sqlStat.append(" AND sct_tstdcsinfo.t0>10 AND sct_tstdcsinfo.TEST_no=tests.TEST_no");
       else  sqlStat.append(" AND sct_tstdcsinfo.t0<=10 AND sct_tstdcsinfo.TEST_no=tests.TEST_no");
       sqlStat.append(" ORDER BY tests.ser_no,tests.TEST_date DESC,tests.TEST_no DESC");

       BufferedWriter out = null;
       String endtab = isWarm ? "_Reference_warm.txt" : "_Reference_cold.txt";
       File oFile = new File(targetDir,SCTDBInfo.sctdaqDBTestNames[testIndex]+endtab);

       System.out.println("Downloading reference "+SCTDBInfo.sctdaqNormalTestNames[testIndex]+" data to "+oFile.getAbsolutePath());

       Statement statement = SCTDBInterface.getInstance().connection.createStatement();
       System.out.println(sqlStat.toString());
       ResultSet resultSet = statement.executeQuery(sqlStat.toString());
       for(boolean n = resultSet.next() ; n==true ; n=resultSet.next() ){
            int rc=1;

	    String sn = resultSet.getString(rc++);
	    if(s.contains(sn)) continue;
	    s.add(sn);

	    StringBuffer theLine = new StringBuffer("SN="+sn);
            theLine.append("TESTNO="+resultSet.getString(rc++));
	    System.out.println("sn="+sn+", location="+resultSet.getString(rc++)+" date="+guiUtilities.DaveUtils.extractDate(resultSet.getString(rc++)));
            theLine.append("DATA=");
            for(int j=0;j<12;j++) {
	      if(j>0) theLine.append(";");
              theLine.append(resultSet.getString(rc++));
	      if(testIndex!=SCTDBInfo.SCTDB_TEST_STROBEDELAY) {
	        theLine.append("*");
	        theLine.append(resultSet.getString(rc++));
		}
	      }
	    if(out==null) out = new BufferedWriter(new FileWriter(oFile));
	    out.write(theLine.toString());
	    out.newLine();
	    }
         if(out!=null) out.close();
	 statement.close();
	 }

public static void makeDataReferenceFile2(String targetDir, int testIndex, boolean isWarm) throws Exception {

       java.util.regex.Pattern runPattern = java.util.regex.Pattern.compile("(\\d+)-(\\d+)");
       java.util.regex.Pattern linePattern = java.util.regex.Pattern.compile("(.*)DEF=XX(CFACTORS=.*TESTNO=)(\\d+)(LOC=.*)");
       java.text.DecimalFormat nf = new java.text.DecimalFormat("#.##");

       String[] testfnames={"","","FullBypass","Pipeline","StrobeDelay","3PtGain","TrimRange","NPtGain","NoiseOccupancy","TimeWalk","",""};
       for(int barrelType=0;barrelType<4;barrelType++) {
       Set s = new HashSet();
       Vector slist = new Vector();

       String temp = isWarm ? "warm" : "cold";

       StringBuffer sqlStat = new StringBuffer("SELECT tests.ser_no,tests.TEST_no,tests.locn_name,tests.test_date,tests.run_no");
       sqlStat.append(",sct_tstdcsinfo.T0,sct_tstdcsinfo.T1,sct_tstdcsinfo.Vdet,sct_tstdcsinfo.Idet,sct_tstdcsinfo.Vcc,sct_tstdcsinfo.Icc,sct_tstdcsinfo.Vdd,sct_tstdcsinfo.Idd");
       for(int y=0;y<SCTDBInfo.chipDBnames.length;y++) {
           for(int d=0;d<SCTDBInfo.DBChipParameterNames[testIndex].length;d++) {
	     if(testIndex==SCTDBInfo.SCTDB_TEST_FULLBYPASS && d>=2) continue;
	     if(testIndex==SCTDBInfo.SCTDB_TEST_3PTGAIN && d>=9) continue;
	     if(testIndex==SCTDBInfo.SCTDB_TEST_NPTGAIN && d>=9) continue;
	     sqlStat.append(","+SCTDBInfo.chipDBnames[y]+SCTDBInfo.DBChipParameterNames[testIndex][d]);
             }
	 }
       sqlStat.append(" FROM "+SCTDBInfo.sctdaqDBTableNames[testIndex]+",tests,sct_tstdcsinfo,SCT_TSTDAQINFO");
       sqlStat.append(" WHERE tests.TEST_name = '"+SCTDBInfo.sctdaqDBTestNames[testIndex]+"'");
       sqlStat.append(" AND tests.ser_no IN (SELECT ser_no FROM assm_items WHERE assm_ser_no="+DisplayGUI.SCTInfo.SCTSerialNumbers[barrelType]+")");
       sqlStat.append(" AND tests.TEST_no = "+SCTDBInfo.sctdaqDBTableNames[testIndex]+".TEST_no");
       sqlStat.append(" AND SCT_TSTDAQINFO.version NOT LIKE 'SctRodDaq%' AND SCT_TSTDAQINFO.test_no=tests.TEST_no");  // SCTDAQ data only
       if(isWarm) sqlStat.append(" AND sct_tstdcsinfo.t0>10 AND sct_tstdcsinfo.TEST_no=tests.TEST_no");
       else  sqlStat.append(" AND sct_tstdcsinfo.t0<=10 AND sct_tstdcsinfo.TEST_no=tests.TEST_no");
       sqlStat.append(" ORDER BY tests.ser_no,tests.TEST_date DESC,tests.TEST_no DESC");

       BufferedWriter out = null;
       int bnum = barrelType+3;
       String barreltab = "_Barrel"+Integer.toString(bnum);
       String endtab = isWarm ? "_Reference_warm.txt" : "_Reference_cold.txt";
       File oFile = new File(targetDir,testfnames[testIndex]+barreltab+endtab);

       System.out.println("Downloading "+temp+" reference "+SCTDBInfo.sctdaqNormalTestNames[testIndex]+" data from Barrel "+Integer.toString(bnum)+" to "+oFile.getAbsolutePath());

       Statement statement = SCTDBInterface.getInstance().connection.createStatement();
//       System.out.println(sqlStat.toString());
       ResultSet resultSet = statement.executeQuery(sqlStat.toString());

       StringBuffer testnoString = new StringBuffer();

       for(boolean n = resultSet.next() ; n==true ; n=resultSet.next() ){
            int rc=1;

	    String sn = resultSet.getString(rc++);
	    if(s.contains(sn)) continue;
	    s.add(sn);

            String testno = resultSet.getString(rc++);
	    String location = resultSet.getString(rc++);
	    String date = guiUtilities.DaveUtils.extractDate(resultSet.getString(rc++));
            String runscan = resultSet.getString(rc++);
	    String run="-1";
	    String scan="-1";
	    java.util.regex.Matcher matcher = runPattern.matcher(runscan);
	    if(matcher.matches()) {
	       run = runscan.substring(matcher.start(1),matcher.end(1));
	       scan = runscan.substring(matcher.start(2),matcher.end(2));
	       }
	    StringBuffer DCSline = new StringBuffer("DCS=");
	    String t0 = resultSet.getString(rc++);
	    String t1 = resultSet.getString(rc++);
	    double t0d = Double.valueOf(t0).doubleValue();
	    double t1d = Double.valueOf(t1).doubleValue();
	    String tdiff = nf.format(Math.abs(t0d-t1d));
	    DCSline.append(t0+"*"+t1+"*"+tdiff);
	    for(int d=0;d<6;d++) {
	       DCSline.append("*");
	       if(d==1) {  // IDet convert uA to nA
	          String idet = resultSet.getString(rc++);
		  double idetd = Double.valueOf(idet).doubleValue();
		  idetd *= 1000.;
		  idet = nf.format(idetd);
		  DCSline.append(idet);
		  }
	       else DCSline.append(resultSet.getString(rc++));
	       }

            StringBuffer dataline = new StringBuffer("DATA=");
            for(int j=0;j<SCTDBInfo.chipDBnames.length;j++) {
                dataline.append(SCTDBInfo.chipDBnames[j].substring(0,2)+":");
                for(int d=0;d<SCTDBInfo.DBChipParameterNames[testIndex].length;d++) {
		   if(testIndex==SCTDBInfo.SCTDB_TEST_FULLBYPASS && d>=2) continue;
	           if(testIndex==SCTDBInfo.SCTDB_TEST_3PTGAIN && d>=9) continue;
	           if(testIndex==SCTDBInfo.SCTDB_TEST_NPTGAIN && d>=9) continue;
		   if(d>0) dataline.append("*");
		   dataline.append(resultSet.getString(rc++));
                   }
		   // now we need to add blank entries for the sctroddaq-only params
		  if(testIndex==SCTDBInfo.SCTDB_TEST_3PTGAIN || testIndex==SCTDBInfo.SCTDB_TEST_NPTGAIN) {
		   for(int d=0;d<3;d++) dataline.append("*.");  // 3 types of slope params
		   }
		  if(testIndex==SCTDBInfo.SCTDB_TEST_NOISE) {
		   for(int d=0;d<2;d++) dataline.append("*.");  // badness params
		   }
		  dataline.append(";");
	        }

//            RUN=${runno}SCAN=${scanno}SN=${sn}${DATAline}${DCSline}DEF=${defectcount}*${cCount}${CONFline}
	    String theLine = "RUN="+run+"SCAN="+scan+"SN="+sn+dataline+DCSline+"DEF=XXCFACTORS=1.00*1.00*1.00*1.00*1.00*1.00*1.00*1.00*1.00*1.00*1.00*1.00CONF=TESTNO="+testno+"LOC="+location+"DATE="+date;
            slist.addElement(theLine);
	    if(testnoString.length()!=0) testnoString.append(",");
            testnoString.append(testno);


	    }
	statement.close();

	Map defectCount = new HashMap();
        Map chanCount = new HashMap();

	if(slist.size()>0) {

	System.out.println("Get defects for "+testfnames[testIndex]+"...");
	sqlStat = new StringBuffer();
        sqlStat.append("SELECT defects.defect_name,defects.chan_1st,defects.chan_last,defects.test_no");
        sqlStat.append(" FROM defects");
        sqlStat.append(" WHERE defects.TEST_no IN ("+testnoString.toString()+")");
        statement = SCTDBInterface.getInstance().connection.createStatement();
//        System.out.println(sqlStat.toString());
        resultSet = statement.executeQuery(sqlStat.toString());

       for(boolean n = resultSet.next() ; n==true ; n=resultSet.next() ){
         String name = resultSet.getString(1);
	 String chan1 = resultSet.getString(2);
	 String chan2 = resultSet.getString(3);
	 String testno = resultSet.getString(4);
	 if(defectCount.containsKey(testno)) {
	    int i = ((Integer)defectCount.get(testno)).intValue();
	    i++;
	    defectCount.put(testno,new Integer(i));
	    }
	 else defectCount.put(testno,new Integer(1));
	 int i1 = Integer.valueOf(chan1).intValue();
	 int i2 = Integer.valueOf(chan2).intValue();
	 Map tmpMap;
	 if(chanCount.containsKey(testno)) tmpMap = (Map)chanCount.get(testno);
	 else tmpMap = new HashMap();
	 for (int x=i1;x<=i2;x++) {
	       Integer xi = new Integer(x);
	       if(!tmpMap.containsKey(xi)) tmpMap.put(xi,"1");
	       }
	 chanCount.put(testno,tmpMap);
	 }
	 statement.close();

	 }

	 for(int i=0;i<slist.size();i++) {
	   String line = (String)slist.elementAt(i);
           java.util.regex.Matcher matcher = linePattern.matcher(line);
	   //("(.*)DEF=0*0(CFACTORS=.*TESTNO=)(.*)(LOC=.*)");
	   if(matcher.matches()) {
	      String b1 = line.substring(matcher.start(1),matcher.end(1));
	      String b2 = line.substring(matcher.start(2),matcher.end(2));
	      String testno = line.substring(matcher.start(3),matcher.end(3));
	      String b3 = line.substring(matcher.start(4),matcher.end(4));
	      String defC="0";
	      String chanC="0";
	      if(defectCount.containsKey(testno)) defC = ((Integer)defectCount.get(testno)).toString();
	      if(chanCount.containsKey(testno)) {
	         Map tmpMap = (Map)chanCount.get(testno);
		 int size = tmpMap.keySet().size();
	         chanC = Integer.toString(size);
		 }
	      line = b1+"DEF="+defC+"*"+chanC+b2+testno+b3;
	      }
	   else System.out.println("**ERROR cant match line "+line);
	   // write file
	   if(out==null) {
	       System.out.println("opening "+oFile.getAbsolutePath());
	       out = new BufferedWriter(new FileWriter(oFile));
	       }
	    out.write(line);
	    out.newLine();
	    }

         if(out!=null) out.close();

	 }
	 }




}
